@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -1269,7 +1269,7 @@ class GTKFileChooserUI extends SynthFileChooserUI {
             super(FilePane.ACTION_EDIT_FILE_NAME);
         }
         public void actionPerformed(ActionEvent e) {
-            if (getFileName().equals("")) {
+            if (getFileName().isEmpty()) {
                 return;
             }
             JFileChooser fc = getFileChooser();
