@@ -50,6 +50,7 @@ class Constants {
         1.10 to 1.10.X 54,0
         1.11 to 1.11.X 55,0
         1.12 to 1.12.X 56,0
+        1.13 to 1.13.X 57,0
     */
 
     public static final Package.Version JAVA_MIN_CLASS_VERSION =
@@ -79,6 +80,9 @@ class Constants {
     public static final Package.Version JAVA12_MAX_CLASS_VERSION =
             Package.Version.of(56, 00);
 
+    public static final Package.Version JAVA13_MAX_CLASS_VERSION =
+            Package.Version.of(57, 00);
+
     public static final int JAVA_PACKAGE_MAGIC = 0xCAFED00D;
 
     public static final Package.Version JAVA5_PACKAGE_VERSION =
@@ -95,7 +99,7 @@ class Constants {
 
     // upper limit, should point to the latest class version
     public static final Package.Version JAVA_MAX_CLASS_VERSION =
-            JAVA12_MAX_CLASS_VERSION;
+            JAVA13_MAX_CLASS_VERSION;
 
     // upper limit should point to the latest package version, for version info!.
     public static final Package.Version MAX_PACKAGE_VERSION =
