@@ -25,13 +25,13 @@ import com.sun.org.apache.bcel.internal.classfile.LocalVariable;
 
 /**
  * This class represents a local variable within a method. It contains its
- * scope, name and type. The generated LocalVariable object can be obtained
- * with getLocalVariable which needs the instruction list and the constant
- * pool as parameters.
+ * scope, name and type. The generated LocalVariable object can be obtained with
+ * getLocalVariable which needs the instruction list and the constant pool as
+ * parameters.
  *
- * @version $Id$
- * @see     LocalVariable
- * @see     MethodGen
+ * @version $Id: LocalVariableGen.java 1749603 2016-06-21 20:50:19Z ggregory $
+ * @see LocalVariable
+ * @see MethodGen
  */
 public class LocalVariableGen implements InstructionTargeter, NamedAndTyped, Cloneable {
 
@@ -40,22 +40,21 @@ public class LocalVariableGen implements InstructionTargeter, NamedAndTyped, Clo
     private Type type;
     private InstructionHandle start;
     private InstructionHandle end;
-    private int orig_index; // never changes; used to match up with LocalVariableTypeTable entries
-    private boolean live_to_end;
-
 
     /**
-     * Generate a local variable that with index `index'. Note that double and long
-     * variables need two indexs. Index indices have to be provided by the user.
+     * Generate a local variable that with index `index'. Note that double and
+     * long variables need two indexs. Index indices have to be provided by the
+     * user.
      *
      * @param index index of local variable
      * @param name its name
      * @param type its type
-     * @param start from where the instruction is valid (null means from the start)
+     * @param start from where the instruction is valid (null means from the
+     * start)
      * @param end until where the instruction is valid (null means to the end)
      */
-    public LocalVariableGen(final int index, final String name, final Type type, final InstructionHandle start,
-            final InstructionHandle end) {
+    public LocalVariableGen(final int index, final String name, final Type type,
+            final InstructionHandle start, final InstructionHandle end) {
         if ((index < 0) || (index > Const.MAX_SHORT)) {
             throw new ClassGenException("Invalid index index: " + index);
         }
@@ -64,137 +63,90 @@ public class LocalVariableGen implements InstructionTargeter, NamedAndTyped, Clo
         this.index = index;
         setStart(start);
         setEnd(end);
-        this.orig_index = index;
-        this.live_to_end = end == null;
     }
 
-
-    /**
-     * Generate a local variable that with index `index'. Note that double and long
-     * variables need two indexs. Index indices have to be provided by the user.
-     *
-     * @param index index of local variable
-     * @param name its name
-     * @param type its type
-     * @param start from where the instruction is valid (null means from the start)
-     * @param end until where the instruction is valid (null means to the end)
-     * @param orig_index index of local variable prior to any changes to index
-     */
-    public LocalVariableGen(final int index, final String name, final Type type, final InstructionHandle start,
-            final InstructionHandle end, final int orig_index) {
-        this(index, name, type, start, end);
-        this.orig_index = orig_index;
-    }
-
-
     /**
      * Get LocalVariable object.
      *
-     * This relies on that the instruction list has already been dumped to byte code or
-     * or that the `setPositions' methods has been called for the instruction list.
+     * This relies on that the instruction list has already been dumped to byte
+     * code or or that the `setPositions' methods has been called for the
+     * instruction list.
      *
-     * Note that due to the conversion from byte code offset to InstructionHandle,
-     * it is impossible to tell the difference between a live range that ends BEFORE
-     * the last insturction of the method or a live range that ends AFTER the last
-     * instruction of the method.  Hence the live_to_end flag to differentiate
-     * between these two cases.
+     * Note that for local variables whose scope end at the last instruction of
+     * the method's code, the JVM specification is ambiguous: both a
+     * start_pc+length ending at the last instruction and start_pc+length ending
+     * at first index beyond the end of the code are valid.
      *
      * @param cp constant pool
      */
-    public LocalVariable getLocalVariable( final ConstantPoolGen cp ) {
+    public LocalVariable getLocalVariable(final ConstantPoolGen cp) {
         int start_pc = 0;
         int length = 0;
         if ((start != null) && (end != null)) {
             start_pc = start.getPosition();
             length = end.getPosition() - start_pc;
-            if ((end.getNext() == null) && live_to_end) {
+            if (end.getNext() == null) {
                 length += end.getInstruction().getLength();
             }
         }
         final int name_index = cp.addUtf8(name);
         final int signature_index = cp.addUtf8(type.getSignature());
         return new LocalVariable(start_pc, length, name_index, signature_index, index, cp
-                .getConstantPool(), orig_index);
+                .getConstantPool());
     }
 
-
-    public void setIndex( final int index ) {
+    public void setIndex(final int index) {
         this.index = index;
     }
 
-
     public int getIndex() {
         return index;
     }
 
-
-    public int getOrigIndex() {
-        return orig_index;
-    }
-
-
-    public void setLiveToEnd( final boolean live_to_end) {
-        this.live_to_end = live_to_end;
-    }
-
-
-    public boolean getLiveToEnd() {
-        return live_to_end;
-    }
-
-
     @Override
-    public void setName( final String name ) {
+    public void setName(final String name) {
         this.name = name;
     }
 
-
     @Override
     public String getName() {
         return name;
     }
 
-
     @Override
-    public void setType( final Type type ) {
+    public void setType(final Type type) {
         this.type = type;
     }
 
-
     @Override
     public Type getType() {
         return type;
     }
 
-
     public InstructionHandle getStart() {
         return start;
     }
 
-
     public InstructionHandle getEnd() {
         return end;
     }
 
-
-    public void setStart( final InstructionHandle start ) { // TODO could be package-protected?
+    public void setStart(final InstructionHandle start) { // TODO could be package-protected?
         BranchInstruction.notifyTarget(this.start, start, this);
         this.start = start;
     }
 
-
-    public void setEnd( final InstructionHandle end ) { // TODO could be package-protected?
+    public void setEnd(final InstructionHandle end) { // TODO could be package-protected?
         BranchInstruction.notifyTarget(this.end, end, this);
         this.end = end;
     }
 
-
     /**
      * @param old_ih old target, either start or end
      * @param new_ih new target
      */
     @Override
-    public void updateTarget( final InstructionHandle old_ih, final InstructionHandle new_ih ) {
+    public void updateTarget(final InstructionHandle old_ih, final InstructionHandle new_ih) {
         boolean targeted = false;
         if (start == old_ih) {
             targeted = true;
@@ -222,11 +174,10 @@ public class LocalVariableGen implements InstructionTargeter, NamedAndTyped, Clo
      * @return true, if ih is target of this variable
      */
     @Override
-    public boolean containsTarget( final InstructionHandle ih ) {
+    public boolean containsTarget(final InstructionHandle ih) {
         return (start == ih) || (end == ih);
     }
 
-
     @Override
     public int hashCode() {
         // If the user changes the name or type, problems with the targeter hashmap will occur.
@@ -234,13 +185,12 @@ public class LocalVariableGen implements InstructionTargeter, NamedAndTyped, Clo
         return name.hashCode() ^ type.hashCode();
     }
 
-
     /**
      * We consider to local variables to be equal, if the use the same index and
      * are valid in the same range.
      */
     @Override
-    public boolean equals( final Object o ) {
+    public boolean equals(final Object o) {
         if (!(o instanceof LocalVariableGen)) {
             return false;
         }
@@ -248,13 +198,11 @@ public class LocalVariableGen implements InstructionTargeter, NamedAndTyped, Clo
         return (l.index == index) && (l.start == start) && (l.end == end);
     }
 
-
     @Override
     public String toString() {
         return "LocalVariableGen(" + name + ", " + type + ", " + start + ", " + end + ")";
     }
 
-
     @Override
     public Object clone() {
         try {
