@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -46,6 +46,7 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Objects;
+import java.util.ServiceLoader;
 import java.util.Set;
 
 import javax.lang.model.element.Modifier;
@@ -417,6 +418,17 @@ public class ClientCodeWrapper {
         public String toString() {
             return wrappedToString(getClass(), clientJavaFileManager);
         }
+
+        @Override @DefinedBy(Api.COMPILER)
+        public <S> ServiceLoader<S> getServiceLoader(Location location, Class<S> service) throws IOException {
+            try {
+                return clientJavaFileManager.getServiceLoader(location, service);
+            } catch (ClientCodeException e) {
+                throw e;
+            } catch (RuntimeException | Error e) {
+                throw new ClientCodeException(e);
+            }
+        }
     }
 
     protected class WrappedStandardJavaFileManager extends WrappedJavaFileManager
@@ -436,6 +448,18 @@ public class ClientCodeWrapper {
             }
         }
 
+        @Override @DefinedBy(Api.COMPILER)
+        public Iterable<? extends JavaFileObject> getJavaFileObjectsFromPaths(Collection<? extends Path> paths) {
+            try {
+                return ((StandardJavaFileManager)clientJavaFileManager).getJavaFileObjectsFromPaths(paths);
+            } catch (ClientCodeException e) {
+                throw e;
+            } catch (RuntimeException | Error e) {
+                throw new ClientCodeException(e);
+            }
+        }
+
+        @Deprecated(since = "13")
         @Override @DefinedBy(Api.COMPILER)
         public Iterable<? extends JavaFileObject> getJavaFileObjectsFromPaths(Iterable<? extends Path> paths) {
             try {
@@ -556,6 +580,18 @@ public class ClientCodeWrapper {
                 throw new ClientCodeException(e);
             }
         }
+
+        @Override @DefinedBy(Api.COMPILER)
+        public void setLocationForModule(Location location, String moduleName, Collection<? extends Path> paths) throws IOException {
+            try {
+                System.out.println("invoking wrapped setLocationForModule");
+                ((StandardJavaFileManager)clientJavaFileManager).setLocationForModule(location, moduleName, paths);
+            } catch (ClientCodeException e) {
+                throw e;
+            } catch (RuntimeException | Error e) {
+                throw new ClientCodeException(e);
+            }
+        }
     }
 
     protected class WrappedFileObject implements FileObject {
