@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -831,8 +831,8 @@ public class TypeEnter implements Completer {
 
             annotate.annotateLater(tree.mods.annotations, baseEnv,
                         sym, tree.pos());
+            attr.attribTypeVariables(tree.typarams, baseEnv, false);
 
-            attr.attribTypeVariables(tree.typarams, baseEnv);
             for (JCTypeParameter tp : tree.typarams)
                 annotate.queueScanTreeAndTypeAnnotate(tp, baseEnv, sym, tree.pos());
 
@@ -939,6 +939,12 @@ public class TypeEnter implements Completer {
                 env.info.scope.enter(superSym);
             }
 
+            if (!tree.typarams.isEmpty()) {
+                for (JCTypeParameter tvar : tree.typarams) {
+                    chk.checkNonCyclic(tvar, (TypeVar)tvar.type);
+                }
+            }
+
             finishClass(tree, env);
 
             if (allowTypeAnnos) {
