@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -852,9 +852,6 @@ public class Arguments {
             doclintOpts.add(DocLint.XHTML_VERSION_PREFIX + format);
         }
 
-        // standard doclet normally generates H1, H2,
-        // so for now, allow user comments to assume that
-        doclintOpts.add(DocLint.XIMPLICIT_HEADERS + "2");
         return List.from(doclintOpts.toArray(new String[doclintOpts.size()]));
     }
 
