@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -188,7 +188,7 @@ public class DocCommentParser {
                     if (isFileContent) {
                         switch (phase) {
                             case PREAMBLE:
-                                if (peek("body")) {
+                                if (isEndPreamble()) {
                                     trees.add(html());
                                     if (textStart == -1) {
                                         textStart = bp;
@@ -200,7 +200,7 @@ public class DocCommentParser {
                                 }
                                 break;
                             case BODY:
-                                if (peek("/body")) {
+                                if (isEndBody()) {
                                     addPendingText(trees, lastNonWhite);
                                     break loop;
                                 }
@@ -787,6 +787,94 @@ public class DocCommentParser {
         }
     }
 
+    /**
+     * Returns whether this is the end of the preamble of an HTML file.
+     * The preamble ends with start of {@code body} element followed by
+     * possible whitespace and the start of a {@code main} element.
+     *
+     * @return whether this is the end of the preamble
+     */
+    boolean isEndPreamble() {
+        final int savedpos = bp;
+        try {
+            if (ch == '<')
+                nextChar();
+
+            if (isIdentifierStart(ch)) {
+                String name = StringUtils.toLowerCase(readIdentifier().toString());
+                switch (name) {
+                    case "body":
+                        // Check if also followed by <main>
+                        // 1. skip rest of <body>
+                        while (ch != -1 && ch != '>') {
+                            nextChar();
+                        }
+                        if (ch == '>') {
+                            nextChar();
+                        }
+                        // 2. skip any whitespce
+                        while (ch != -1 && Character.isWhitespace(ch)) {
+                            nextChar();
+                        }
+                        // 3. check if looking at "<main..."
+                        if (ch == '<') {
+                            nextChar();
+                            if (isIdentifierStart(ch)) {
+                                name = StringUtils.toLowerCase(readIdentifier().toString());
+                                if (name.equals("main")) {
+                                    return false;
+                                }
+                            }
+                        }
+                        // if <body> is _not_ followed by <main> then this is the
+                        // end of the preamble
+                        return true;
+
+                    case "main":
+                        // <main> is unconditionally the end of the preamble
+                        return true;
+                }
+            }
+            return false;
+        } finally {
+            bp = savedpos;
+            ch = buf[bp];
+        }
+    }
+
+    /**
+     * Returns whether this is the end of the main body of the content in a standalone
+     * HTML file.
+     * The content ends with the closing tag for a {@code main} or {@code body} element.
+     *
+     * @return whether this is the end of the main body of the content
+     */
+    boolean isEndBody() {
+        final int savedpos = bp;
+        try {
+            if (ch == '<')
+                nextChar();
+
+            if (ch == '/') {
+                nextChar();
+                if (isIdentifierStart(ch)) {
+                    String name = StringUtils.toLowerCase(readIdentifier().toString());
+                    switch (name) {
+                        case "body":
+                        case "main":
+                            return true;
+                    }
+                }
+            }
+
+            return false;
+        } finally {
+            bp = savedpos;
+            ch = buf[bp];
+        }
+
+    }
+
     boolean peek(String s) {
         final int savedpos = bp;
         try {
@@ -1032,6 +1120,14 @@ public class DocCommentParser {
         return names.fromChars(buf, start, bp - start);
     }
 
+    protected Name readSystemPropertyName() {
+        int pos = bp;
+        nextChar();
+        while (bp < buflen && Character.isUnicodeIdentifierPart(ch) || ch == '.')
+            nextChar();
+        return names.fromChars(buf, pos, bp - pos);
+    }
+
     protected boolean isDecimalDigit(char ch) {
         return ('0' <= ch && ch <= '9');
     }
@@ -1360,6 +1456,28 @@ public class DocCommentParser {
                 }
             },
 
+            // @systemProperty property-name
+            new TagParser(Kind.INLINE, DCTree.Kind.SYSTEM_PROPERTY) {
+                public DCTree parse(int pos) throws ParseException {
+                    skipWhitespace();
+                    if (ch == '}') {
+                        throw new ParseException("dc.no.content");
+                    }
+                    Name propertyName = readSystemPropertyName();
+                    if (propertyName == null) {
+                        throw new ParseException("dc.no.content");
+                    }
+                    skipWhitespace();
+                    if (ch != '}') {
+                        nextChar();
+                        throw new ParseException("dc.unexpected.content");
+                    } else {
+                        nextChar();
+                        return m.at(pos).newSystemPropertyTree(propertyName);
+                    }
+                }
+            },
+
             // @throws class-name description
             new TagParser(Kind.BLOCK, DCTree.Kind.THROWS) {
                 public DCTree parse(int pos) throws ParseException {
