@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -53,14 +53,10 @@ public class Names {
     public final Name empty;
     public final Name hyphen;
     public final Name one;
-    public final Name period;
-    public final Name semicolon;
     public final Name slash;
-    public final Name slashequals;
 
     // keywords
     public final Name _class;
-    public final Name _default;
     public final Name _super;
     public final Name _this;
     public final Name var;
@@ -74,6 +70,7 @@ public class Names {
     public final Name uses;
     public final Name open;
     public final Name with;
+    public final Name yield;
 
     // field and method names
     public final Name _name;
@@ -83,24 +80,16 @@ public class Names {
     public final Name clinit;
     public final Name clone;
     public final Name close;
-    public final Name compareTo;
     public final Name deserializeLambda;
     public final Name desiredAssertionStatus;
     public final Name equals;
     public final Name error;
-    public final Name family;
     public final Name finalize;
-    public final Name forName;
     public final Name forRemoval;
     public final Name getClass;
-    public final Name getClassLoader;
-    public final Name getComponentType;
-    public final Name getDeclaringClass;
-    public final Name getMessage;
     public final Name hasNext;
     public final Name hashCode;
     public final Name init;
-    public final Name initCause;
     public final Name iterator;
     public final Name length;
     public final Name next;
@@ -114,12 +103,10 @@ public class Names {
 
     // class names
     public final Name java_io_Serializable;
-    public final Name java_lang_AutoCloseable;
     public final Name java_lang_Class;
     public final Name java_lang_Cloneable;
     public final Name java_lang_Enum;
     public final Name java_lang_Object;
-    public final Name java_lang_invoke_MethodHandle;
 
     // names of builtin classes
     public final Name Array;
@@ -189,7 +176,6 @@ public class Names {
 
     // other identifiers
     public final Name T;
-    public final Name deprecated;
     public final Name ex;
     public final Name module_info;
     public final Name package_info;
@@ -217,14 +203,10 @@ public class Names {
         empty = fromString("");
         hyphen = fromString("-");
         one = fromString("1");
-        period = fromString(".");
-        semicolon = fromString(";");
         slash = fromString("/");
-        slashequals = fromString("/=");
 
         // keywords
         _class = fromString("class");
-        _default = fromString("default");
         _super = fromString("super");
         _this = fromString("this");
         var = fromString("var");
@@ -238,6 +220,7 @@ public class Names {
         uses = fromString("uses");
         open = fromString("open");
         with = fromString("with");
+        yield = fromString("yield");
 
         // field and method names
         _name = fromString("name");
@@ -247,24 +230,16 @@ public class Names {
         clinit = fromString("<clinit>");
         clone = fromString("clone");
         close = fromString("close");
-        compareTo = fromString("compareTo");
         deserializeLambda = fromString("$deserializeLambda$");
         desiredAssertionStatus = fromString("desiredAssertionStatus");
         equals = fromString("equals");
         error = fromString("<error>");
-        family = fromString("family");
         finalize = fromString("finalize");
-        forName = fromString("forName");
         forRemoval = fromString("forRemoval");
         getClass = fromString("getClass");
-        getClassLoader = fromString("getClassLoader");
-        getComponentType = fromString("getComponentType");
-        getDeclaringClass = fromString("getDeclaringClass");
-        getMessage = fromString("getMessage");
         hasNext = fromString("hasNext");
         hashCode = fromString("hashCode");
         init = fromString("<init>");
-        initCause = fromString("initCause");
         iterator = fromString("iterator");
         length = fromString("length");
         next = fromString("next");
@@ -279,12 +254,10 @@ public class Names {
 
         // class names
         java_io_Serializable = fromString("java.io.Serializable");
-        java_lang_AutoCloseable = fromString("java.lang.AutoCloseable");
         java_lang_Class = fromString("java.lang.Class");
         java_lang_Cloneable = fromString("java.lang.Cloneable");
         java_lang_Enum = fromString("java.lang.Enum");
         java_lang_Object = fromString("java.lang.Object");
-        java_lang_invoke_MethodHandle = fromString("java.lang.invoke.MethodHandle");
 
         // names of builtin classes
         Array = fromString("Array");
@@ -354,7 +327,6 @@ public class Names {
 
         // other identifiers
         T = fromString("T");
-        deprecated = fromString("deprecated");
         ex = fromString("ex");
         module_info = fromString("module-info");
         package_info = fromString("package-info");
