@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2004, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -31,7 +31,7 @@ extern "C" {
 #endif
 
 /* Classfile version number for this information */
-#define JVM_CLASSFILE_MAJOR_VERSION 54
+#define JVM_CLASSFILE_MAJOR_VERSION 57
 #define JVM_CLASSFILE_MINOR_VERSION 0
 
 /* Flags */
@@ -107,8 +107,11 @@ enum {
     JVM_CONSTANT_NameAndType            = 12,
     JVM_CONSTANT_MethodHandle           = 15,  // JSR 292
     JVM_CONSTANT_MethodType             = 16,  // JSR 292
+    JVM_CONSTANT_Dynamic                = 17,
     JVM_CONSTANT_InvokeDynamic          = 18,
-    JVM_CONSTANT_ExternalMax            = 18
+    JVM_CONSTANT_Module                 = 19,
+    JVM_CONSTANT_Package                = 20,
+    JVM_CONSTANT_ExternalMax            = 20 
 };
 
 /* JVM_CONSTANT_MethodHandle subtypes */
