@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -860,7 +860,7 @@ public abstract class CubicCurve2D implements Shape, Cloneable {
      *          the specified offset.
      * @since 1.2
      */
-    public static double getFlatnessSq(double coords[], int offset) {
+    public static double getFlatnessSq(double[] coords, int offset) {
         return getFlatnessSq(coords[offset + 0], coords[offset + 1],
                              coords[offset + 2], coords[offset + 3],
                              coords[offset + 4], coords[offset + 5],
@@ -880,7 +880,7 @@ public abstract class CubicCurve2D implements Shape, Cloneable {
      *          the specified offset.
      * @since 1.2
      */
-    public static double getFlatness(double coords[], int offset) {
+    public static double getFlatness(double[] coords, int offset) {
         return getFlatness(coords[offset + 0], coords[offset + 1],
                            coords[offset + 2], coords[offset + 3],
                            coords[offset + 4], coords[offset + 5],
@@ -1000,9 +1000,9 @@ public abstract class CubicCurve2D implements Shape, Cloneable {
      * the 6 right coordinates
      * @since 1.2
      */
-    public static void subdivide(double src[], int srcoff,
-                                 double left[], int leftoff,
-                                 double right[], int rightoff) {
+    public static void subdivide(double[] src, int srcoff,
+                                 double[] left, int leftoff,
+                                 double[] right, int rightoff) {
         double x1 = src[srcoff + 0];
         double y1 = src[srcoff + 1];
         double ctrlx1 = src[srcoff + 2];
@@ -1065,7 +1065,7 @@ public abstract class CubicCurve2D implements Shape, Cloneable {
      * @return the number of roots, or -1 if the equation is a constant.
      * @since 1.2
      */
-    public static int solveCubic(double eqn[]) {
+    public static int solveCubic(double[] eqn) {
         return solveCubic(eqn, eqn);
     }
 
@@ -1086,7 +1086,7 @@ public abstract class CubicCurve2D implements Shape, Cloneable {
      * @return the number of roots, or -1 if the equation is a constant
      * @since 1.3
      */
-    public static int solveCubic(double eqn[], double res[]) {
+    public static int solveCubic(double[] eqn, double[] res) {
         // From Graphics Gems:
         // http://tog.acm.org/resources/GraphicsGems/gems/Roots3And4.c
         final double d = eqn[3];
@@ -1369,7 +1369,7 @@ public abstract class CubicCurve2D implements Shape, Cloneable {
         return (x1 < 0 && x2 > 0) || (x1 > 0 && x2 < 0);
     }
 
-    private static double solveEqn(double eqn[], int order, double t) {
+    private static double solveEqn(double[] eqn, int order, double t) {
         double v = eqn[order];
         while (--order >= 0) {
             v = v * t + eqn[order];
