@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1995, 2018, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -160,7 +160,7 @@ public abstract class ColorModel implements Transparency{
      * The total number of bits in the pixel.
      */
     protected int pixel_bits;
-    int nBits[];
+    int[] nBits;
     int transparency = Transparency.TRANSLUCENT;
     boolean supportsAlpha = true;
     boolean isAlphaPremultiplied = false;
@@ -645,17 +645,17 @@ public abstract class ColorModel implements Transparency{
         int pixel=0,length=0;
         switch (transferType) {
             case DataBuffer.TYPE_BYTE:
-               byte bdata[] = (byte[])inData;
+               byte[] bdata = (byte[])inData;
                pixel = bdata[0] & 0xff;
                length = bdata.length;
             break;
             case DataBuffer.TYPE_USHORT:
-               short sdata[] = (short[])inData;
+               short[] sdata = (short[])inData;
                pixel = sdata[0] & 0xffff;
                length = sdata.length;
             break;
             case DataBuffer.TYPE_INT:
-               int idata[] = (int[])inData;
+               int[] idata = (int[])inData;
                pixel = idata[0];
                length = idata.length;
             break;
@@ -712,17 +712,17 @@ public abstract class ColorModel implements Transparency{
         int pixel=0,length=0;
         switch (transferType) {
             case DataBuffer.TYPE_BYTE:
-               byte bdata[] = (byte[])inData;
+               byte[] bdata = (byte[])inData;
                pixel = bdata[0] & 0xff;
                length = bdata.length;
             break;
             case DataBuffer.TYPE_USHORT:
-               short sdata[] = (short[])inData;
+               short[] sdata = (short[])inData;
                pixel = sdata[0] & 0xffff;
                length = sdata.length;
             break;
             case DataBuffer.TYPE_INT:
-               int idata[] = (int[])inData;
+               int[] idata = (int[])inData;
                pixel = idata[0];
                length = idata.length;
             break;
@@ -779,17 +779,17 @@ public abstract class ColorModel implements Transparency{
         int pixel=0,length=0;
         switch (transferType) {
             case DataBuffer.TYPE_BYTE:
-               byte bdata[] = (byte[])inData;
+               byte[] bdata = (byte[])inData;
                pixel = bdata[0] & 0xff;
                length = bdata.length;
             break;
             case DataBuffer.TYPE_USHORT:
-               short sdata[] = (short[])inData;
+               short[] sdata = (short[])inData;
                pixel = sdata[0] & 0xffff;
                length = sdata.length;
             break;
             case DataBuffer.TYPE_INT:
-               int idata[] = (int[])inData;
+               int[] idata = (int[])inData;
                pixel = idata[0];
                length = idata.length;
             break;
@@ -842,17 +842,17 @@ public abstract class ColorModel implements Transparency{
         int pixel=0,length=0;
         switch (transferType) {
             case DataBuffer.TYPE_BYTE:
-               byte bdata[] = (byte[])inData;
+               byte[] bdata = (byte[])inData;
                pixel = bdata[0] & 0xff;
                length = bdata.length;
             break;
             case DataBuffer.TYPE_USHORT:
-               short sdata[] = (short[])inData;
+               short[] sdata = (short[])inData;
                pixel = sdata[0] & 0xffff;
                length = sdata.length;
             break;
             case DataBuffer.TYPE_INT:
-               int idata[] = (int[])inData;
+               int[] idata = (int[])inData;
                pixel = idata[0];
                length = idata.length;
             break;
@@ -1333,7 +1333,7 @@ public abstract class ColorModel implements Transparency{
      * @since 1.4
      */
     public int getDataElement(float[] normComponents, int normOffset) {
-        int components[] = getUnnormalizedComponents(normComponents,
+        int[] components = getUnnormalizedComponents(normComponents,
                                                      normOffset, null, 0);
         return getDataElement(components, 0);
     }
@@ -1381,7 +1381,7 @@ public abstract class ColorModel implements Transparency{
      */
     public Object getDataElements(float[] normComponents, int normOffset,
                                   Object obj) {
-        int components[] = getUnnormalizedComponents(normComponents,
+        int[] components = getUnnormalizedComponents(normComponents,
                                                      normOffset, null, 0);
         return getDataElements(components, 0, obj);
     }
@@ -1442,7 +1442,7 @@ public abstract class ColorModel implements Transparency{
     public float[] getNormalizedComponents(Object pixel,
                                            float[] normComponents,
                                            int normOffset) {
-        int components[] = getComponents(pixel, null, 0);
+        int[] components = getComponents(pixel, null, 0);
         return getNormalizedComponents(components, 0,
                                        normComponents, normOffset);
     }
