@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -114,7 +114,7 @@ public class Introspector {
     private Class<?> beanClass;
     private BeanInfo explicitBeanInfo;
     private BeanInfo superBeanInfo;
-    private BeanInfo additionalBeanInfo[];
+    private BeanInfo[] additionalBeanInfo;
 
     private boolean propertyChangeSource = false;
 
@@ -310,7 +310,7 @@ public class Introspector {
                         Character.isUpperCase(name.charAt(0))){
             return name;
         }
-        char chars[] = name.toCharArray();
+        char[] chars = name.toCharArray();
         chars[0] = Character.toLowerCase(chars[0]);
         return new String(chars);
     }
@@ -364,7 +364,6 @@ public class Introspector {
      */
     public static void flushCaches() {
         ThreadGroupContext.getContext().clearBeanInfoCache();
-        ClassInfo.clear();
     }
 
     /**
@@ -388,7 +387,6 @@ public class Introspector {
             throw new NullPointerException();
         }
         ThreadGroupContext.getContext().removeBeanInfo(clz);
-        ClassInfo.remove(clz);
     }
 
     //======================================================================
@@ -442,9 +440,9 @@ public class Introspector {
         // event sets and locate PropertyChangeListeners before we
         // look for properties.
         BeanDescriptor bd = getTargetBeanDescriptor();
-        MethodDescriptor mds[] = getTargetMethodInfo();
-        EventSetDescriptor esds[] = getTargetEventInfo();
-        PropertyDescriptor pds[] = getTargetPropertyInfo();
+        MethodDescriptor[] mds = getTargetMethodInfo();
+        EventSetDescriptor[] esds = getTargetEventInfo();
+        PropertyDescriptor[] pds = getTargetPropertyInfo();
 
         int defaultEvent = getTargetDefaultEventIndex();
         int defaultProperty = getTargetDefaultPropertyIndex();
@@ -509,7 +507,7 @@ public class Introspector {
         processPropertyDescriptors();
 
         // Allocate and populate the result array.
-        PropertyDescriptor result[] =
+        PropertyDescriptor[] result =
                 properties.values().toArray(new PropertyDescriptor[properties.size()]);
 
         // Set the default index.
@@ -907,7 +905,7 @@ public class Introspector {
 
         if (explicitEvents == null && superBeanInfo != null) {
             // We have no explicit BeanInfo events.  Check with our parent.
-            EventSetDescriptor supers[] = superBeanInfo.getEventSetDescriptors();
+            EventSetDescriptor[] supers = superBeanInfo.getEventSetDescriptors();
             for (int i = 0 ; i < supers.length; i++) {
                 addEvent(supers[i]);
             }
@@ -918,7 +916,7 @@ public class Introspector {
         }
 
         for (int i = 0; i < additionalBeanInfo.length; i++) {
-            EventSetDescriptor additional[] = additionalBeanInfo[i].getEventSetDescriptors();
+            EventSetDescriptor[] additional = additionalBeanInfo[i].getEventSetDescriptors();
             if (additional != null) {
                 for (int j = 0 ; j < additional.length; j++) {
                     addEvent(additional[j]);
@@ -1003,14 +1001,14 @@ public class Introspector {
 
         if (explicitMethods == null && superBeanInfo != null) {
             // We have no explicit BeanInfo methods.  Check with our parent.
-            MethodDescriptor supers[] = superBeanInfo.getMethodDescriptors();
+            MethodDescriptor[] supers = superBeanInfo.getMethodDescriptors();
             for (int i = 0 ; i < supers.length; i++) {
                 addMethod(supers[i]);
             }
         }
 
         for (int i = 0; i < additionalBeanInfo.length; i++) {
-            MethodDescriptor additional[] = additionalBeanInfo[i].getMethodDescriptors();
+            MethodDescriptor[] additional = additionalBeanInfo[i].getMethodDescriptors();
             if (additional != null) {
                 for (int j = 0 ; j < additional.length; j++) {
                     addMethod(additional[j]);
@@ -1032,7 +1030,7 @@ public class Introspector {
         }
 
         // Allocate and populate the result array.
-        MethodDescriptor result[] = new MethodDescriptor[methods.size()];
+        MethodDescriptor[] result = new MethodDescriptor[methods.size()];
         result = methods.values().toArray(result);
 
         return result;
@@ -1138,7 +1136,7 @@ public class Introspector {
     private boolean isEventHandler(Method m) {
         // We assume that a method is an event handler if it has a single
         // argument, whose type inherit from java.util.Event.
-        Type argTypes[] = m.getGenericParameterTypes();
+        Type[] argTypes = m.getGenericParameterTypes();
         if (argTypes.length != 1) {
             return false;
         }
@@ -1154,7 +1152,7 @@ public class Introspector {
      * parameter list on a given class.
      */
     private static Method internalFindMethod(Class<?> start, String methodName,
-                                                 int argCount, Class<?> args[]) {
+                                                 int argCount, Class<?>[] args) {
         // For overriden methods we need to find the most derived version.
         // So we start with the given class and walk up the superclass chain.
         for (Class<?> cl = start; cl != null; cl = cl.getSuperclass()) {
