@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,7 +25,6 @@
 
 package java.io;
 
-import java.lang.reflect.InvocationTargetException;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.security.Security;
@@ -33,15 +32,9 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.Objects;
 import java.util.Optional;
-import java.util.concurrent.atomic.AtomicBoolean;
-import java.util.function.BinaryOperator;
 import java.util.function.Function;
 
 import jdk.internal.access.SharedSecrets;
-import jdk.internal.util.StaticProperty;
-
-import static java.lang.System.Logger.Level.INFO;
-import static java.lang.System.Logger.Level.ERROR;
 
 /**
  * Filter classes, array lengths, and graph metrics during deserialization.
@@ -70,16 +63,16 @@ import static java.lang.System.Logger.Level.ERROR;
  * {@link Status#UNDECIDED UNDECIDED}.
  * Filters should be designed for the specific use case and expected types.
  * A filter designed for a particular use may be passed a class that is outside
- * of the scope of the filter. If the purpose of the filter is to reject classes
+ * of the scope of the filter. If the purpose of the filter is to black-list classes
  * then it can reject a candidate class that matches and report UNDECIDED for others.
  * A filter may be called with class equals {@code null}, {@code arrayLength} equal -1,
  * the depth, number of references, and stream size and return a status
  * that reflects only one or only some of the values.
- * This allows a filter to be specific about the choice it is reporting and
+ * This allows a filter to specific about the choice it is reporting and
  * to use other filters without forcing either allowed or rejected status.
  *
  * <p>
- * Typically, a custom filter should check if a process-wide filter
+ * Typically, a custom filter should check if a system-wide filter
  * is configured and defer to it if so. For example,
  * <pre>{@code
  * ObjectInputFilter.Status checkInput(FilterInfo info) {
@@ -87,7 +80,7 @@ import static java.lang.System.Logger.Level.ERROR;
  *     if (serialFilter != null) {
  *         ObjectInputFilter.Status status = serialFilter.checkInput(info);
  *         if (status != ObjectInputFilter.Status.UNDECIDED) {
- *             // The process-wide filter overrides this filter
+ *             // The system-wide filter overrides this filter
  *             return status;
  *         }
  *     }
@@ -203,8 +196,8 @@ public interface ObjectInputFilter {
     }
 
     /**
-     * A utility class to set and get the process-wide filter or create a filter
-     * from a pattern string. If a process-wide filter is set, it will be
+     * A utility class to set and get the system-wide filter or create a filter
+     * from a pattern string. If a system-wide filter is set, it will be
      * used for each {@link ObjectInputStream} that does not set its own filter.
      * <p>
      * When setting the filter, it should be stateless and idempotent,
@@ -212,7 +205,7 @@ public interface ObjectInputFilter {
      * <p>
      * The filter is configured during the initialization of the {@code ObjectInputFilter.Config}
      * class. For example, by calling {@link #getSerialFilter() Config.getSerialFilter}.
-     * If the system property {@code jdk.serialFilter} is defined, it is used
+     * If the system property {@systemProperty jdk.serialFilter} is defined, it is used
      * to configure the filter.
      * If the system property is not defined, and the {@link java.security.Security}
      * property {@code jdk.serialFilter} is defined then it is used to configure the filter.
@@ -225,178 +218,86 @@ public interface ObjectInputFilter {
      * @since 9
      */
     final class Config {
-        /*
-         * Filter Factory implementation notes:
-         *
-         * <p>The JVM-wide deserialization filter factory and the static JVM-wide filter
-         * can be configured from system properties during the initialization of the
-         * {@code ObjectInputFilter.Config} class.
-         *
-         * <p>If the Java virtual machine is started with the system property
-         * {@code jdk.serialFilterFactory} or the {@link java.security.Security} property
-         * of the same name, its value names the class to configure the JVM-wide deserialization
-         * filter factory.
-         * If the system property is not defined, and the {@link java.security.Security} property
-         * {@code jdk.serialFilterFactory} is defined then it is used to configure the filter factory.
-         * If it remains unset, the filter factory is a builtin filter factory compatible
-         * with previous versions.
-         *
-         * <p>The class must be public, must have a public zero-argument constructor, implement the
-         * {@link BinaryOperator {@literal BinaryOperator<ObjectInputFilter>}} interface, provide its implementation and
-         * be accessible via the {@linkplain ClassLoader#getSystemClassLoader() application class loader}.
-         * If the filter factory constructor is not invoked successfully, an {@link ExceptionInInitializerError}
-         * is thrown and subsequent use of the filter factory for deserialization fails with
-         * {@link IllegalStateException}.
-         *
-         * <p>Setting the {@code jdk.serialFilterFactory} with {@link System#setProperty(String, String)
-         * System.setProperty} <em>does not set the filter factory</em>.
-         * The syntax for the system property value and security property value is the
-         * fully qualified class name of the deserialization filter factory.
-         *
-         * <p>The JVM-wide filter factory is a function invoked when each {@link ObjectInputStream} is
-         * {@linkplain ObjectInputStream#ObjectInputStream() constructed} and when the
-         * {@linkplain ObjectInputStream#setObjectInputFilter(ObjectInputFilter) stream-specific filter is set}.
-         * The parameters are the current filter and a requested filter and it
-         * returns the filter to be used for the stream. When invoked from the
-         * {@linkplain ObjectInputStream#ObjectInputStream(InputStream) ObjectInputStream constructors},
-         * the first parameter is {@code null} and the second parameter is the
-         * {@linkplain ObjectInputFilter.Config#getSerialFilter() static JVM-wide filter}.
-         * When invoked from {@link ObjectInputStream#setObjectInputFilter ObjectInputStream.setObjectInputFilter},
-         * the first parameter is the filter currently set on the stream (which was set in the constructor),
-         * and the second parameter is the filter given to {@code ObjectInputStream.setObjectInputFilter}.
-         * The current and new filter may each be {@code null} and the factory may return {@code null}.
-         * If the current filter is {@code non-null}, the filter factory must return a {@code non-null} filter;
-         * this is to prevent unintentional disabling of filtering after it has been enabled.
-         * The factory may throw runtime exceptions to signal incorrect use or invalid parameters.
-         * Note that the filter factory implementation can also use any contextual information
-         * at its disposal, for example, extracted from the application thread context, or its call stack,
-         * to compose and combine a new filter. It is not restricted to only use its two parameters.
-         */
-
         /* No instances. */
         private Config() {}
 
         /**
-         * Lock object for process-wide filter.
+         * Lock object for system-wide filter.
          */
         private final static Object serialFilterLock = new Object();
 
         /**
          * Debug: Logger
          */
-        private static final System.Logger configLog;
+        private final static System.Logger configLog;
 
         /**
-         * The property name for the filter.
+         * Logger for debugging.
+         */
+        static void filterLog(System.Logger.Level level, String msg, Object... args) {
+            if (configLog != null) {
+                configLog.log(level, msg, args);
+            }
+        }
+
+        /**
+         * The name for the system-wide deserialization filter.
          * Used as a system property and a java.security.Security property.
          */
-        private static final String SERIAL_FILTER_PROPNAME = "jdk.serialFilter";
+        private final static String SERIAL_FILTER_PROPNAME = "jdk.serialFilter";
 
         /**
-         * The property name for the filter factory.
-         * Used as a system property and a java.security.Security property.
+         * The system-wide filter; may be null.
+         * Lookup the filter in java.security.Security or
+         * the system property.
          */
-        private static final String SERIAL_FILTER_FACTORY_PROPNAME = "jdk.serialFilterFactory";
-
-        /**
-         * Current static filter, synchronized with {@code serialFilterLock}
-         */
-        private static ObjectInputFilter serialFilter;
-
-        /**
-         * The serial filter factory.
-         */
-        private static final BinaryOperator<ObjectInputFilter> serialFilterFactory;
-
+        private final static ObjectInputFilter configuredFilter;
 
         static {
-            /*
-             * Initialize the configuration containing the filter factory, static filter, and logger.
-             * <ul>
-             * <li>The logger is created.
-             * <li>The property 'jdk.serialFilter" is read, either as a system property or a security property,
-             *     and if set, defines the configured static JVM-wide filter and is logged.
-             * <li>The property jdk.serialFilterFactory is read, either as a system property or a security property,
-             *     and if set, defines the initial filter factory and is logged.
-             * </ul>
-             */
+            configuredFilter = AccessController
+                    .doPrivileged((PrivilegedAction<ObjectInputFilter>) () -> {
+                        String props = System.getProperty(SERIAL_FILTER_PROPNAME);
+                        if (props == null) {
+                            props = Security.getProperty(SERIAL_FILTER_PROPNAME);
+                        }
+                        if (props != null) {
+                            System.Logger log =
+                                    System.getLogger("java.io.serialization");
+                            log.log(System.Logger.Level.INFO,
+                                    "Creating serialization filter from {0}", props);
+                            try {
+                                return createFilter(props);
+                            } catch (RuntimeException re) {
+                                log.log(System.Logger.Level.ERROR,
+                                        "Error configuring filter: {0}", re);
+                            }
+                        }
+                        return null;
+                    });
+            configLog = (configuredFilter != null) ? System.getLogger("java.io.serialization") : null;
 
-            // Initialize the logger.
-            configLog = System.getLogger("java.io.serialization");
-
-            // Get the values of the system properties, if they are defined
-            String factoryClassName = StaticProperty.jdkSerialFilterFactory();
-            if (factoryClassName == null) {
-                // Fallback to security property
-                factoryClassName = AccessController.doPrivileged((PrivilegedAction<String>) () ->
-                        Security.getProperty(SERIAL_FILTER_FACTORY_PROPNAME));
-            }
-
-            String filterString = StaticProperty.jdkSerialFilter();
-            if (filterString == null) {
-                // Fallback to security property
-                filterString = AccessController.doPrivileged((PrivilegedAction<String>) () ->
-                        Security.getProperty(SERIAL_FILTER_PROPNAME));
-            }
-
-            // Initialize the static filter if the jdk.serialFilter is present
-            ObjectInputFilter filter = null;
-            if (filterString != null) {
-                configLog.log(INFO,
-                        "Creating deserialization filter from {0}", filterString);
-                try {
-                    filter = createFilter(filterString);
-                } catch (RuntimeException re) {
-                    configLog.log(ERROR,
-                            "Error configuring filter: {0}", re);
-                }
-            }
-            serialFilter = filter;
-
-            // Initialize the filter factory if the jdk.serialFilterFactory is defined
-            // otherwise use the builtin filter factory.
-            if (factoryClassName == null) {
-                serialFilterFactory = new BuiltinFilterFactory();
-            } else {
-                try {
-                    // Load using the system class loader, the named class may be an application class.
-                    Class<?> factoryClass = Class.forName(factoryClassName, true,
-                            ClassLoader.getSystemClassLoader());
-                    @SuppressWarnings("unchecked")
-                    BinaryOperator<ObjectInputFilter> factory =
-                            (BinaryOperator<ObjectInputFilter>)
-                            factoryClass.getConstructor().newInstance(new Object[0]);
-                    configLog.log(INFO,
-                            "Creating deserialization filter factory for {0}", factoryClassName);
-                    serialFilterFactory = factory;
-                } catch (RuntimeException | ClassNotFoundException | NoSuchMethodException |
-                        IllegalAccessException | InstantiationException | InvocationTargetException ex) {
-                    Throwable th = (ex instanceof InvocationTargetException) ? ex.getCause() : ex;
-                    configLog.log(ERROR,
-                            "Error configuring filter factory: {0}", (Object)th);
-                    // Do not continue if configuration not initialized
-                    throw new ExceptionInInitializerError(th);
-                }
-            }
             // Setup shared secrets for RegistryImpl to use.
             SharedSecrets.setJavaObjectInputFilterAccess(Config::createFilter2);
         }
 
         /**
-         * Returns the process-wide serialization filter or {@code null} if not configured.
+         * Current configured filter.
+         */
+        private static volatile ObjectInputFilter serialFilter = configuredFilter;
+
+        /**
+         * Returns the system-wide serialization filter or {@code null} if not configured.
          *
-         * @return the process-wide serialization filter or {@code null} if not configured
+         * @return the system-wide serialization filter or {@code null} if not configured
          */
         public static ObjectInputFilter getSerialFilter() {
-            synchronized (serialFilterLock) {
-                return serialFilter;
-            }
+            return serialFilter;
         }
 
         /**
-         * Set the process-wide filter if it has not already been configured or set.
+         * Set the system-wide filter if it has not already been configured or set.
          *
-         * @param filter the serialization filter to set as the process-wide filter; not null
+         * @param filter the serialization filter to set as the system-wide filter; not null
          * @throws SecurityException if there is security manager and the
          *       {@code SerializablePermission("serialFilter")} is not granted
          * @throws IllegalStateException if the filter has already been set {@code non-null}
@@ -415,22 +316,6 @@ public interface ObjectInputFilter {
             }
         }
 
-        /*
-         * Returns the serial filter factory singleton.
-         * This package private method is *only* called by {@link ObjectInputStream#ObjectInputStream()},
-         * {@link ObjectInputStream#ObjectInputStream(InputStream)}, and
-         * {@link ObjectInputStream#setObjectInputFilter(ObjectInputFilter)}.
-         *
-         * @return the serial filter factory
-         * @throws IllegalStateException if the filter factory initialization is incomplete
-         */
-        /* package-private */
-        static BinaryOperator<ObjectInputFilter> getSerialFilterFactorySingleton() {
-            if (serialFilterFactory == null)
-                throw new IllegalStateException("Serial filter factory initialization incomplete");
-            return serialFilterFactory;
-        }
-
         /**
          * Returns an ObjectInputFilter from a string of patterns.
          * <p>
@@ -514,7 +399,7 @@ public interface ObjectInputFilter {
 
         /**
          * Implementation of ObjectInputFilter that performs the checks of
-         * the process-wide serialization filter. If configured, it will be
+         * the system-wide serialization filter. If configured, it will be
          * used for all ObjectInputStreams that do not set their own filters.
          *
          */
@@ -790,49 +675,5 @@ public interface ObjectInputFilter {
                 return pattern;
             }
         }
-
-        /**
-         * Builtin Deserialization filter factory.
-         * The builtin deserialization filter factory implements the behavior of earlier versions of
-         * setting the static serial filter in the {@link ObjectInputStream} constructor and
-         * {@link ObjectInputStream#setObjectInputFilter} in cooperation with {@code ObjectInputStream}.
-         * Checking that the stream-specific filter can only be set once and throwing
-         * {@link IllegalStateException} is handled by
-         * {@link ObjectInputStream#setObjectInputFilter(ObjectInputFilter)}.
-         *
-         * @see Config#getSerialFilterFactory()
-         */
-        private static final class BuiltinFilterFactory implements BinaryOperator<ObjectInputFilter> {
-            /**
-             * Returns the {@code ObjectInputFilter} to be used for an ObjectInputStream.
-             *
-             * <p>When invoked from the
-             * {@linkplain ObjectInputStream#ObjectInputStream(InputStream) ObjectInputStream constructors},
-             * the first parameter is {@code null} and the second parameter is the
-             * {@linkplain ObjectInputFilter.Config#getSerialFilter() static JVM-wide filter};
-             * the value returned is {@code newFilter}, the static JVM-wide filter.
-             * <p>
-             * When invoked from
-             * {@link ObjectInputStream#setObjectInputFilter(ObjectInputFilter) setObjectInputFilter}
-             * to set the stream-specific filter, the value is {@code newFilter} to replace the
-             * previous filter.
-             *
-             * @param oldFilter the current filter, may be null
-             * @param newFilter a new filter, may be null
-             * @return an ObjectInputFilter, the new Filter, may be null
-             */
-            @Override
-            public ObjectInputFilter apply(ObjectInputFilter oldFilter, ObjectInputFilter newFilter) {
-                return newFilter;
-            }
-
-            /**
-             * Returns the class name of this builtin deserialization filter factory.
-             * @return returns the class name of this builtin deserialization filter factory
-             */
-            public String toString() {
-                return this.getClass().getName();
-            }
-        }
     }
 }
