@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -853,17 +853,6 @@ public class ObjectStreamClass implements Serializable {
             throw new InternalError("Unexpected call when not initialized");
     }
 
-    /**
-     * Throws InvalidClassException if not initialized.
-     * To be called in cases where an uninitialized class descriptor indicates
-     * a problem in the serialization stream.
-     */
-    final void checkInitialized() throws InvalidClassException {
-        if (!initialized) {
-            throw new InvalidClassException("Class descriptor should be initialized");
-        }
-    }
-
     /**
      * Throws an InvalidClassException if object instances referencing this
      * class descriptor should not be allowed to deserialize.  This method does
@@ -1120,10 +1109,6 @@ public class ObjectStreamClass implements Serializable {
             } catch (IllegalAccessException ex) {
                 // should not occur, as access checks have been suppressed
                 throw new InternalError(ex);
-            } catch (InstantiationError err) {
-                var ex = new InstantiationException();
-                ex.initCause(err);
-                throw ex;
             }
         } else {
             throw new UnsupportedOperationException();
@@ -2150,7 +2135,7 @@ public class ObjectStreamClass implements Serializable {
                 switch (typeCodes[i]) {
                     case 'L':
                     case '[':
-                        vals[offsets[i]] = unsafe.getObject(obj, readKeys[i]);
+                        vals[offsets[i]] = unsafe.getReference(obj, readKeys[i]);
                         break;
 
                     default:
@@ -2205,7 +2190,7 @@ public class ObjectStreamClass implements Serializable {
                                 obj.getClass().getName());
                         }
                         if (!dryRun)
-                            unsafe.putObject(obj, key, val);
+                            unsafe.putReference(obj, key, val);
                         break;
 
                     default:
