@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1995, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -27,7 +27,6 @@ package java.lang;
 
 import java.io.PrintStream;
 import java.util.Arrays;
-import jdk.internal.misc.VM;
 
 /**
  * A thread group represents a set of threads. In addition, a thread
@@ -82,11 +81,11 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * Constructs a new thread group. The parent of this new group is
      * the thread group of the currently running thread.
      * <p>
-     * The <code>checkAccess</code> method of the parent thread group is
+     * The {@code checkAccess} method of the parent thread group is
      * called with no arguments; this may result in a security exception.
      *
      * @param   name   the name of the new thread group.
-     * @exception  SecurityException  if the current thread cannot create a
+     * @throws  SecurityException  if the current thread cannot create a
      *               thread in the specified thread group.
      * @see     java.lang.ThreadGroup#checkAccess()
      * @since   1.0
@@ -99,14 +98,14 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * Creates a new thread group. The parent of this new group is the
      * specified thread group.
      * <p>
-     * The <code>checkAccess</code> method of the parent thread group is
+     * The {@code checkAccess} method of the parent thread group is
      * called with no arguments; this may result in a security exception.
      *
      * @param     parent   the parent thread group.
      * @param     name     the name of the new thread group.
-     * @exception  NullPointerException  if the thread group argument is
-     *               <code>null</code>.
-     * @exception  SecurityException  if the current thread cannot create a
+     * @throws    NullPointerException  if the thread group argument is
+     *               {@code null}.
+     * @throws    SecurityException  if the current thread cannot create a
      *               thread in the specified thread group.
      * @see     java.lang.SecurityException
      * @see     java.lang.ThreadGroup#checkAccess()
@@ -147,13 +146,13 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Returns the parent of this thread group.
      * <p>
-     * First, if the parent is not <code>null</code>, the
-     * <code>checkAccess</code> method of the parent thread group is
+     * First, if the parent is not {@code null}, the
+     * {@code checkAccess} method of the parent thread group is
      * called with no arguments; this may result in a security exception.
      *
      * @return  the parent of this thread group. The top-level thread group
-     *          is the only thread group whose parent is <code>null</code>.
-     * @exception  SecurityException  if the current thread cannot modify
+     *          is the only thread group whose parent is {@code null}.
+     * @throws  SecurityException  if the current thread cannot modify
      *               this thread group.
      * @see        java.lang.ThreadGroup#checkAccess()
      * @see        java.lang.SecurityException
@@ -185,8 +184,8 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * daemon thread group is automatically destroyed when its last
      * thread is stopped or its last thread group is destroyed.
      *
-     * @return  <code>true</code> if this thread group is a daemon thread group;
-     *          <code>false</code> otherwise.
+     * @return  {@code true} if this thread group is a daemon thread group;
+     *          {@code false} otherwise.
      * @since   1.0
      */
     public final boolean isDaemon() {
@@ -206,16 +205,16 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Changes the daemon status of this thread group.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      * <p>
      * A daemon thread group is automatically destroyed when its last
      * thread is stopped or its last thread group is destroyed.
      *
-     * @param      daemon   if <code>true</code>, marks this thread group as
+     * @param      daemon   if {@code true}, marks this thread group as
      *                      a daemon thread group; otherwise, marks this
      *                      thread group as normal.
-     * @exception  SecurityException  if the current thread cannot modify
+     * @throws     SecurityException  if the current thread cannot modify
      *               this thread group.
      * @see        java.lang.SecurityException
      * @see        java.lang.ThreadGroup#checkAccess()
@@ -230,24 +229,24 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * Sets the maximum priority of the group. Threads in the thread
      * group that already have a higher priority are not affected.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      * <p>
-     * If the <code>pri</code> argument is less than
+     * If the {@code pri} argument is less than
      * {@link Thread#MIN_PRIORITY} or greater than
      * {@link Thread#MAX_PRIORITY}, the maximum priority of the group
      * remains unchanged.
      * <p>
      * Otherwise, the priority of this ThreadGroup object is set to the
-     * smaller of the specified <code>pri</code> and the maximum permitted
+     * smaller of the specified {@code pri} and the maximum permitted
      * priority of the parent of this thread group. (If this thread group
      * is the system thread group, which has no parent, then its maximum
-     * priority is simply set to <code>pri</code>.) Then this method is
-     * called recursively, with <code>pri</code> as its argument, for
+     * priority is simply set to {@code pri}.) Then this method is
+     * called recursively, with {@code pri} as its argument, for
      * every thread group that belongs to this thread group.
      *
      * @param      pri   the new priority of the thread group.
-     * @exception  SecurityException  if the current thread cannot modify
+     * @throws     SecurityException  if the current thread cannot modify
      *               this thread group.
      * @see        #getMaxPriority
      * @see        java.lang.SecurityException
@@ -280,9 +279,9 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * argument or one of its ancestor thread groups.
      *
      * @param   g   a thread group.
-     * @return  <code>true</code> if this thread group is the thread group
+     * @return  {@code true} if this thread group is the thread group
      *          argument or one of its ancestor thread groups;
-     *          <code>false</code> otherwise.
+     *          {@code false} otherwise.
      * @since   1.0
      */
     public final boolean parentOf(ThreadGroup g) {
@@ -298,11 +297,11 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * Determines if the currently running thread has permission to
      * modify this thread group.
      * <p>
-     * If there is a security manager, its <code>checkAccess</code> method
+     * If there is a security manager, its {@code checkAccess} method
      * is called with this thread group as its argument. This may result
-     * in throwing a <code>SecurityException</code>.
+     * in throwing a {@code SecurityException}.
      *
-     * @exception  SecurityException  if the current thread is not allowed to
+     * @throws     SecurityException  if the current thread is not allowed to
      *               access this thread group.
      * @see        java.lang.SecurityManager#checkAccess(java.lang.ThreadGroup)
      * @since      1.0
@@ -425,7 +424,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
         ThreadGroup[] groupsSnapshot = null;
         synchronized (this) {
             if (destroyed) {
-                return 0;
+                return n;
             }
             int nt = nthreads;
             if (nt > list.length - n) {
@@ -559,7 +558,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
         ThreadGroup[] groupsSnapshot = null;
         synchronized (this) {
             if (destroyed) {
-                return 0;
+                return n;
             }
             int ng = ngroups;
             if (ng > list.length - n) {
@@ -589,13 +588,13 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Stops all threads in this thread group.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      * <p>
-     * This method then calls the <code>stop</code> method on all the
+     * This method then calls the {@code stop} method on all the
      * threads in this thread group and in all of its subgroups.
      *
-     * @exception  SecurityException  if the current thread is not allowed
+     * @throws     SecurityException  if the current thread is not allowed
      *               to access this thread group or any of the threads in
      *               the thread group.
      * @see        java.lang.SecurityException
@@ -614,13 +613,13 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Interrupts all threads in this thread group.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      * <p>
-     * This method then calls the <code>interrupt</code> method on all the
+     * This method then calls the {@code interrupt} method on all the
      * threads in this thread group and in all of its subgroups.
      *
-     * @exception  SecurityException  if the current thread is not allowed
+     * @throws     SecurityException  if the current thread is not allowed
      *               to access this thread group or any of the threads in
      *               the thread group.
      * @see        java.lang.Thread#interrupt()
@@ -651,13 +650,13 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Suspends all threads in this thread group.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      * <p>
-     * This method then calls the <code>suspend</code> method on all the
+     * This method then calls the {@code suspend} method on all the
      * threads in this thread group and in all of its subgroups.
      *
-     * @exception  SecurityException  if the current thread is not allowed
+     * @throws     SecurityException  if the current thread is not allowed
      *               to access this thread group or any of the threads in
      *               the thread group.
      * @see        java.lang.Thread#suspend()
@@ -712,13 +711,13 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Resumes all threads in this thread group.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      * <p>
-     * This method then calls the <code>resume</code> method on all the
+     * This method then calls the {@code resume} method on all the
      * threads in this thread group and in all of its sub groups.
      *
-     * @exception  SecurityException  if the current thread is not allowed to
+     * @throws     SecurityException  if the current thread is not allowed to
      *               access this thread group or any of the threads in the
      *               thread group.
      * @see        java.lang.SecurityException
@@ -757,12 +756,12 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * group must be empty, indicating that all threads that had been in
      * this thread group have since stopped.
      * <p>
-     * First, the <code>checkAccess</code> method of this thread group is
+     * First, the {@code checkAccess} method of this thread group is
      * called with no arguments; this may result in a security exception.
      *
-     * @exception  IllegalThreadStateException  if the thread group is not
+     * @throws     IllegalThreadStateException  if the thread group is not
      *               empty or if the thread group has already been destroyed.
-     * @exception  SecurityException  if the current thread cannot modify this
+     * @throws     SecurityException  if the current thread cannot modify this
      *               thread group.
      * @see        java.lang.ThreadGroup#checkAccess()
      * @since      1.0
@@ -800,7 +799,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
     /**
      * Adds the specified Thread group to this group.
      * @param g the specified Thread group to be added
-     * @exception IllegalThreadStateException If the Thread group has been destroyed.
+     * @throws  IllegalThreadStateException If the Thread group has been destroyed.
      */
     private final void add(ThreadGroup g){
         synchronized (this) {
@@ -878,7 +877,7 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * @param  t
      *         the Thread to be added
      *
-     * @throws  IllegalThreadStateException
+     * @throws IllegalThreadStateException
      *          if the Thread group has been destroyed
      */
     void add(Thread t) {
@@ -1016,29 +1015,29 @@ class ThreadGroup implements Thread.UncaughtExceptionHandler {
      * does not have a specific {@link Thread.UncaughtExceptionHandler}
      * installed.
      * <p>
-     * The <code>uncaughtException</code> method of
-     * <code>ThreadGroup</code> does the following:
+     * The {@code uncaughtException} method of
+     * {@code ThreadGroup} does the following:
      * <ul>
      * <li>If this thread group has a parent thread group, the
-     *     <code>uncaughtException</code> method of that parent is called
+     *     {@code uncaughtException} method of that parent is called
      *     with the same two arguments.
      * <li>Otherwise, this method checks to see if there is a
      *     {@linkplain Thread#getDefaultUncaughtExceptionHandler default
      *     uncaught exception handler} installed, and if so, its
-     *     <code>uncaughtException</code> method is called with the same
+     *     {@code uncaughtException} method is called with the same
      *     two arguments.
-     * <li>Otherwise, this method determines if the <code>Throwable</code>
+     * <li>Otherwise, this method determines if the {@code Throwable}
      *     argument is an instance of {@link ThreadDeath}. If so, nothing
      *     special is done. Otherwise, a message containing the
      *     thread's name, as returned from the thread's {@link
      *     Thread#getName getName} method, and a stack backtrace,
-     *     using the <code>Throwable</code>'s {@link
+     *     using the {@code Throwable}'s {@link
      *     Throwable#printStackTrace printStackTrace} method, is
      *     printed to the {@linkplain System#err standard error stream}.
      * </ul>
      * <p>
      * Applications can override this method in subclasses of
-     * <code>ThreadGroup</code> to provide alternative handling of
+     * {@code ThreadGroup} to provide alternative handling of
      * uncaught exceptions.
      *
      * @param   t   the thread that is about to exit.
