@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -134,8 +134,9 @@ import java.util.Spliterator;
  *   it already contains: It leaves the limit unchanged and sets the position
  *   to zero.  </p></li>
  *
- *   <li><p> {@link #slice} creates a subsequence of a buffer: It leaves the
- *   limit and the position unchanged. </p></li>
+ *   <li><p> The {@link #slice} and {@link #slice(int,int) slice(index,length)}
+ *   methods create a subsequence of a buffer: They leave the limit and the
+ *   position unchanged. </p></li>
  *
  *   <li><p> {@link #duplicate} creates a shallow copy of a buffer: It leaves
  *   the limit and the position unchanged. </p></li>
@@ -345,8 +346,8 @@ public abstract class Buffer {
         if (newLimit > capacity | newLimit < 0)
             throw createLimitException(newLimit);
         limit = newLimit;
-        if (position > newLimit) position = newLimit;
-        if (mark > newLimit) mark = -1;
+        if (position > limit) position = limit;
+        if (mark > limit) mark = -1;
         return this;
     }
 
@@ -599,6 +600,39 @@ public abstract class Buffer {
      */
     public abstract Buffer slice();
 
+    /**
+     * Creates a new buffer whose content is a shared subsequence of
+     * this buffer's content.
+     *
+     * <p> The content of the new buffer will start at position {@code index}
+     * in this buffer, and will contain {@code length} elements. Changes to
+     * this buffer's content will be visible in the new buffer, and vice versa;
+     * the two buffers' position, limit, and mark values will be independent.
+     *
+     * <p> The new buffer's position will be zero, its capacity and its limit
+     * will be {@code length}, its mark will be undefined. The new buffer will
+     * be direct if, and only if, this buffer is direct, and it will be
+     * read-only if, and only if, this buffer is read-only.  </p>
+     *
+     * @param   index
+     *          The position in this buffer at which the content of the new
+     *          buffer will start; must be non-negative and no larger than
+     *          {@link #limit() limit()}
+     *
+     * @param   length
+     *          The number of elements the new buffer will contain; must be
+     *          non-negative and no larger than {@code limit() - index}
+     *
+     * @return  The new buffer
+     *
+     * @throws  IndexOutOfBoundsException
+     *          If {@code index} is negative or greater than {@code limit()},
+     *          {@code length} is negative, or {@code length > limit() - index}
+     *
+     * @since 13
+     */
+    public abstract Buffer slice(int index, int length);
+
     /**
      * Creates a new buffer that shares this buffer's content.
      *
@@ -637,18 +671,16 @@ public abstract class Buffer {
      * @return  The current position value, before it is incremented
      */
     final int nextGetIndex() {                          // package-private
-        int p = position;
-        if (p >= limit)
+        if (position >= limit)
             throw new BufferUnderflowException();
-        position = p + 1;
-        return p;
+        return position++;
     }
 
     final int nextGetIndex(int nb) {                    // package-private
-        int p = position;
-        if (limit - p < nb)
+        if (limit - position < nb)
             throw new BufferUnderflowException();
-        position = p + nb;
+        int p = position;
+        position += nb;
         return p;
     }
 
@@ -660,18 +692,16 @@ public abstract class Buffer {
      * @return  The current position value, before it is incremented
      */
     final int nextPutIndex() {                          // package-private
-        int p = position;
-        if (p >= limit)
+        if (position >= limit)
             throw new BufferOverflowException();
-        position = p + 1;
-        return p;
+        return position++;
     }
 
     final int nextPutIndex(int nb) {                    // package-private
-        int p = position;
-        if (limit - p < nb)
+        if (limit - position < nb)
             throw new BufferOverflowException();
-        position = p + nb;
+        int p = position;
+        position += nb;
         return p;
     }
 
@@ -697,22 +727,10 @@ public abstract class Buffer {
         return mark;
     }
 
-    final void truncate() {                             // package-private
-        mark = -1;
-        position = 0;
-        limit = 0;
-        capacity = 0;
-    }
-
     final void discardMark() {                          // package-private
         mark = -1;
     }
 
-    static void checkBounds(int off, int len, int size) { // package-private
-        if ((off | len | (off + len) | (size - (off + len))) < 0)
-            throw new IndexOutOfBoundsException();
-    }
-
     static {
         // setup access to this package in SharedSecrets
         SharedSecrets.setJavaNioAccess(
@@ -721,14 +739,6 @@ public abstract class Buffer {
                 public JavaNioAccess.BufferPool getDirectBufferPool() {
                     return Bits.BUFFER_POOL;
                 }
-                @Override
-                public ByteBuffer newDirectByteBuffer(long addr, int cap, Object ob) {
-                    return new DirectByteBuffer(addr, cap, ob);
-                }
-                @Override
-                public void truncate(Buffer buf) {
-                    buf.truncate();
-                }
             });
     }
 
