@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2017, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -152,7 +152,7 @@ implements java.io.Serializable
      * Each chain is ordered bottom-to-top (i.e., with the signer certificate
      * first and the (root) certificate authority last). The signer
      * certificates are copied from the array. Subsequent changes to
-     * the array will not affect this UnresolvedPermission.
+     * the array will not affect this UnsolvedPermission.
      */
     public UnresolvedPermission(String type,
                                 String name,
@@ -164,63 +164,59 @@ implements java.io.Serializable
         if (type == null)
                 throw new NullPointerException("type can't be null");
 
-        // Perform a defensive copy and reassign certs if we have a non-null
-        // reference
-        if (certs != null) {
-            certs = certs.clone();
-        }
-
         this.type = type;
         this.name = name;
         this.actions = actions;
-
         if (certs != null) {
             // Extract the signer certs from the list of certificates.
-            for (int i = 0; i < certs.length; i++) {
+            for (int i=0; i<certs.length; i++) {
                 if (!(certs[i] instanceof X509Certificate)) {
                     // there is no concept of signer certs, so we store the
-                    // entire cert array.  No further processing is necessary.
-                    this.certs = certs;
-                    return;
+                    // entire cert array
+                    this.certs = certs.clone();
+                    break;
                 }
             }
 
-            // Go through the list of certs and see if all the certs are
-            // signer certs.
-            int i = 0;
-            int count = 0;
-            while (i < certs.length) {
-                count++;
-                while (((i + 1) < certs.length) &&
-                       ((X509Certificate)certs[i]).getIssuerX500Principal().equals(
-                           ((X509Certificate)certs[i + 1]).getSubjectX500Principal())) {
+            if (this.certs == null) {
+                // Go through the list of certs and see if all the certs are
+                // signer certs.
+                int i = 0;
+                int count = 0;
+                while (i < certs.length) {
+                    count++;
+                    while (((i+1) < certs.length) &&
+                           ((X509Certificate)certs[i]).getIssuerDN().equals(
+                               ((X509Certificate)certs[i+1]).getSubjectDN())) {
+                        i++;
+                    }
                     i++;
                 }
-                i++;
-            }
-            if (count == certs.length) {
-                // All the certs are signer certs, so we store the entire
-                // array.  No further processing is needed.
-                this.certs = certs;
-                return;
-            }
+                if (count == certs.length) {
+                    // All the certs are signer certs, so we store the entire
+                    // array
+                    this.certs = certs.clone();
+                }
 
-            // extract the signer certs
-            ArrayList<java.security.cert.Certificate> signerCerts =
-                new ArrayList<>();
-            i = 0;
-            while (i < certs.length) {
-                signerCerts.add(certs[i]);
-                while (((i + 1) < certs.length) &&
-                    ((X509Certificate)certs[i]).getIssuerX500Principal().equals(
-                      ((X509Certificate)certs[i + 1]).getSubjectX500Principal())) {
-                    i++;
+                if (this.certs == null) {
+                    // extract the signer certs
+                    ArrayList<java.security.cert.Certificate> signerCerts =
+                        new ArrayList<>();
+                    i = 0;
+                    while (i < certs.length) {
+                        signerCerts.add(certs[i]);
+                        while (((i+1) < certs.length) &&
+                            ((X509Certificate)certs[i]).getIssuerDN().equals(
+                              ((X509Certificate)certs[i+1]).getSubjectDN())) {
+                            i++;
+                        }
+                        i++;
+                    }
+                    this.certs =
+                        new java.security.cert.Certificate[signerCerts.size()];
+                    signerCerts.toArray(this.certs);
                 }
-                i++;
             }
-            this.certs =
-                new java.security.cert.Certificate[signerCerts.size()];
-            signerCerts.toArray(this.certs);
         }
     }
 
@@ -313,7 +309,6 @@ implements java.io.Serializable
      *
      * @return false.
      */
-    @Override
     public boolean implies(Permission p) {
         return false;
     }
@@ -334,7 +329,6 @@ implements java.io.Serializable
      * type (class) name, permission name, actions, and
      * certificates as this object.
      */
-    @Override
     public boolean equals(Object obj) {
         if (obj == this)
             return true;
@@ -408,7 +402,7 @@ implements java.io.Serializable
      *
      * @return a hash code value for this object.
      */
-    @Override
+
     public int hashCode() {
         int hash = type.hashCode();
         if (name != null)
@@ -428,7 +422,6 @@ implements java.io.Serializable
      *
      * @return the empty string "".
      */
-    @Override
     public String getActions()
     {
         return "";
@@ -496,7 +489,6 @@ implements java.io.Serializable
      *
      * @return information about this UnresolvedPermission.
      */
-    @Override
     public String toString() {
         return "(unresolved " + type + " " + name + " " + actions + ")";
     }
@@ -508,7 +500,7 @@ implements java.io.Serializable
      * @return a new PermissionCollection object suitable for
      * storing UnresolvedPermissions.
      */
-    @Override
+
     public PermissionCollection newPermissionCollection() {
         return new UnresolvedPermissionCollection();
     }
@@ -599,7 +591,7 @@ implements java.io.Serializable
                 cfs.put(certType, cf);
             }
             // parse the certificate
-            byte[] encoded = IOUtils.readExactlyNBytes(ois, ois.readInt());
+            byte[] encoded = IOUtils.readNBytes(ois, ois.readInt());
             ByteArrayInputStream bais = new ByteArrayInputStream(encoded);
             try {
                 certList.add(cf.generateCertificate(bais));
