@@ -78,14 +78,12 @@ class ZoneName {
         "Africa/Brazzaville", "Africa_Western", "Africa/Lagos",
         "Africa/Bujumbura", "Africa_Central", "Africa/Maputo",
         "Africa/Cairo", "Europe_Eastern", "Europe/Bucharest",
-        "Africa/Casablanca", "Europe_Western", "Atlantic/Canary",
         "Africa/Ceuta", "Europe_Central", "Europe/Paris",
         "Africa/Conakry", "GMT", "Atlantic/Reykjavik",
         "Africa/Dakar", "GMT", "Atlantic/Reykjavik",
         "Africa/Dar_es_Salaam", "Africa_Eastern", "Africa/Nairobi",
         "Africa/Djibouti", "Africa_Eastern", "Africa/Nairobi",
         "Africa/Douala", "Africa_Western", "Africa/Lagos",
-        "Africa/El_Aaiun", "Europe_Western", "Atlantic/Canary",
         "Africa/Freetown", "GMT", "Atlantic/Reykjavik",
         "Africa/Gaborone", "Africa_Central", "Africa/Maputo",
         "Africa/Harare", "Africa_Central", "Africa/Maputo",
@@ -113,7 +111,7 @@ class ZoneName {
         "Africa/Nouakchott", "GMT", "Atlantic/Reykjavik",
         "Africa/Ouagadougou", "GMT", "Atlantic/Reykjavik",
         "Africa/Porto-Novo", "Africa_Western", "Africa/Lagos",
-        "Africa/Sao_Tome", "Africa_Western", "Africa/Lagos",
+        "Africa/Sao_Tome", "GMT", "Atlantic/Reykjavik",
         "Africa/Timbuktu", "GMT", "Atlantic/Reykjavik",
         "Africa/Tripoli", "Europe_Eastern", "Europe/Bucharest",
         "Africa/Tunis", "Europe_Central", "Europe/Paris",
@@ -158,7 +156,7 @@ class ZoneName {
         "America/Cayenne", "French_Guiana", "America/Cayenne",
         "America/Cayman", "America_Eastern", "America/New_York",
         "America/Chicago", "America_Central", "America/Chicago",
-        "America/Chihuahua", "America_Central", "America/Chicago",
+        "America/Chihuahua", "Mexico_Pacific", "America/Mazatlan",
         "America/Coral_Harbour", "America_Eastern", "America/New_York",
         "America/Cordoba", "Argentina", "America/Buenos_Aires",
         "America/Costa_Rica", "America_Central", "America/Chicago",
@@ -238,7 +236,7 @@ class ZoneName {
         "America/North_Dakota/Beulah", "America_Central", "America/Chicago",
         "America/North_Dakota/Center", "America_Central", "America/Chicago",
         "America/North_Dakota/New_Salem", "America_Central", "America/Chicago",
-        "America/Ojinaga", "America_Central", "America/Chicago",
+        "America/Ojinaga", "America_Mountain", "America/Denver",
         "America/Panama", "America_Eastern", "America/New_York",
         "America/Pangnirtung", "America_Eastern", "America/New_York",
         "America/Paramaribo", "Suriname", "America/Paramaribo",
@@ -282,7 +280,7 @@ class ZoneName {
         "America/Winnipeg", "America_Central", "America/Chicago",
         "America/Yakutat", "Alaska", "America/Juneau",
         "America/Yellowknife", "America_Mountain", "America/Denver",
-        "Antarctica/Casey", "Casey", "Antarctica/Casey",
+        "Antarctica/Casey", "Australia_Western", "Australia/Perth",
         "Antarctica/Davis", "Davis", "Antarctica/Davis",
         "Antarctica/DumontDUrville", "DumontDUrville", "Antarctica/DumontDUrville",
         "Antarctica/Macquarie", "Macquarie", "Antarctica/Macquarie",
@@ -358,9 +356,10 @@ class ZoneName {
         "Asia/Oral", "Kazakhstan_Western", "Asia/Aqtobe",
         "Asia/Phnom_Penh", "Indochina", "Asia/Bangkok",
         "Asia/Pontianak", "Indonesia_Western", "Asia/Jakarta",
-        "Asia/Pyongyang", "Pyongyang", "Asia/Pyongyang",
+        "Asia/Pyongyang", "Korea", "Asia/Seoul",
         "Asia/Qatar", "Arabian", "Asia/Riyadh",
-        "Asia/Qyzylorda", "Kazakhstan_Eastern", "Asia/Almaty",
+        "Asia/Qostanay", "Kazakhstan_Eastern", "Asia/Almaty",
+        "Asia/Qyzylorda", "Kazakhstan_Western", "Asia/Aqtobe",
         "Asia/Rangoon", "Myanmar", "Asia/Rangoon",
         "Asia/Riyadh", "Arabian", "Asia/Riyadh",
         "Asia/Saigon", "Indochina", "Asia/Bangkok",
@@ -501,7 +500,7 @@ class ZoneName {
         "Europe/Vatican", "Europe_Central", "Europe/Paris",
         "Europe/Vienna", "Europe_Central", "Europe/Paris",
         "Europe/Vilnius", "Europe_Eastern", "Europe/Bucharest",
-        "Europe/Volgograd", "Moscow", "Europe/Moscow",
+        "Europe/Volgograd", "Volgograd", "Europe/Volgograd",
         "Europe/Warsaw", "Europe_Central", "Europe/Paris",
         "Europe/Zagreb", "Europe_Central", "Europe/Paris",
         "Europe/Zaporozhye", "Europe_Eastern", "Europe/Bucharest",
@@ -775,7 +774,6 @@ class ZoneName {
         "SystemV/YST9YDT", "America/Anchorage",
 
         // From tzdb
-        "Africa/Accra", "Africa/Abidjan",
         "Africa/Addis_Ababa", "Africa/Nairobi",
         "Africa/Asmara", "Africa/Nairobi",
         "Africa/Asmera", "Africa/Nairobi",
@@ -810,64 +808,48 @@ class ZoneName {
         "Africa/Ouagadougou", "Africa/Abidjan",
         "Africa/Porto-Novo", "Africa/Lagos",
         "Africa/Timbuktu", "Africa/Abidjan",
-        "America/Anguilla", "America/Puerto_Rico",
-        "America/Antigua", "America/Puerto_Rico",
+        "America/Anguilla", "America/Port_of_Spain",
+        "America/Antigua", "America/Port_of_Spain",
         "America/Argentina/ComodRivadavia", "America/Argentina/Catamarca",
-        "America/Aruba", "America/Puerto_Rico",
-        "America/Atikokan", "America/Panama",
+        "America/Aruba", "America/Curacao",
         "America/Atka", "America/Adak",
-        "America/Blanc-Sablon", "America/Puerto_Rico",
         "America/Buenos_Aires", "America/Argentina/Buenos_Aires",
         "America/Catamarca", "America/Argentina/Catamarca",
         "America/Cayman", "America/Panama",
-        "America/Coral_Harbour", "America/Panama",
+        "America/Coral_Harbour", "America/Atikokan",
         "America/Cordoba", "America/Argentina/Cordoba",
-        "America/Creston", "America/Phoenix",
-        "America/Curacao", "America/Puerto_Rico",
-        "America/Dominica", "America/Puerto_Rico",
+        "America/Dominica", "America/Port_of_Spain",
         "America/Ensenada", "America/Tijuana",
         "America/Fort_Wayne", "America/Indiana/Indianapolis",
-        "America/Godthab", "America/Nuuk",
-        "America/Grenada", "America/Puerto_Rico",
-        "America/Guadeloupe", "America/Puerto_Rico",
+        "America/Grenada", "America/Port_of_Spain",
+        "America/Guadeloupe", "America/Port_of_Spain",
         "America/Indianapolis", "America/Indiana/Indianapolis",
         "America/Jujuy", "America/Argentina/Jujuy",
         "America/Knox_IN", "America/Indiana/Knox",
-        "America/Kralendijk", "America/Puerto_Rico",
+        "America/Kralendijk", "America/Curacao",
         "America/Louisville", "America/Kentucky/Louisville",
-        "America/Lower_Princes", "America/Puerto_Rico",
-        "America/Marigot", "America/Puerto_Rico",
+        "America/Lower_Princes", "America/Curacao",
+        "America/Marigot", "America/Port_of_Spain",
         "America/Mendoza", "America/Argentina/Mendoza",
         "America/Montreal", "America/Toronto",
-        "America/Montserrat", "America/Puerto_Rico",
-        "America/Nassau", "America/Toronto",
-        "America/Nipigon", "America/Toronto",
-        "America/Pangnirtung", "America/Iqaluit",
-        "America/Port_of_Spain", "America/Puerto_Rico",
+        "America/Montserrat", "America/Port_of_Spain",
         "America/Porto_Acre", "America/Rio_Branco",
-        "America/Rainy_River", "America/Winnipeg",
         "America/Rosario", "America/Argentina/Cordoba",
         "America/Santa_Isabel", "America/Tijuana",
         "America/Shiprock", "America/Denver",
-        "America/St_Barthelemy", "America/Puerto_Rico",
-        "America/St_Kitts", "America/Puerto_Rico",
-        "America/St_Lucia", "America/Puerto_Rico",
-        "America/St_Thomas", "America/Puerto_Rico",
-        "America/St_Vincent", "America/Puerto_Rico",
-        "America/Thunder_Bay", "America/Toronto",
-        "America/Tortola", "America/Puerto_Rico",
-        "America/Virgin", "America/Puerto_Rico",
-        "America/Yellowknife", "America/Edmonton",
-        "Antarctica/DumontDUrville", "Pacific/Port_Moresby",
+        "America/St_Barthelemy", "America/Port_of_Spain",
+        "America/St_Kitts", "America/Port_of_Spain",
+        "America/St_Lucia", "America/Port_of_Spain",
+        "America/St_Thomas", "America/Port_of_Spain",
+        "America/St_Vincent", "America/Port_of_Spain",
+        "America/Tortola", "America/Port_of_Spain",
+        "America/Virgin", "America/Port_of_Spain",
         "Antarctica/McMurdo", "Pacific/Auckland",
         "Antarctica/South_Pole", "Pacific/Auckland",
-        "Antarctica/Syowa", "Asia/Riyadh",
-        "Antarctica/Vostok", "Asia/Urumqi",
-        "Arctic/Longyearbyen", "Europe/Berlin",
+        "Arctic/Longyearbyen", "Europe/Oslo",
         "Asia/Aden", "Asia/Riyadh",
         "Asia/Ashkhabad", "Asia/Ashgabat",
         "Asia/Bahrain", "Asia/Qatar",
-        "Asia/Brunei", "Asia/Kuching",
         "Asia/Calcutta", "Asia/Kolkata",
         "Asia/Chongqing", "Asia/Shanghai",
         "Asia/Chungking", "Asia/Shanghai",
@@ -876,7 +858,6 @@ class ZoneName {
         "Asia/Istanbul", "Europe/Istanbul",
         "Asia/Kashgar", "Asia/Urumqi",
         "Asia/Katmandu", "Asia/Kathmandu",
-        "Asia/Kuala_Lumpur", "Asia/Singapore",
         "Asia/Kuwait", "Asia/Riyadh",
         "Asia/Macao", "Asia/Macau",
         "Asia/Muscat", "Asia/Dubai",
@@ -889,12 +870,10 @@ class ZoneName {
         "Asia/Ulan_Bator", "Asia/Ulaanbaatar",
         "Asia/Vientiane", "Asia/Bangkok",
         "Atlantic/Faeroe", "Atlantic/Faroe",
-        "Atlantic/Jan_Mayen", "Europe/Berlin",
-        "Atlantic/Reykjavik", "Africa/Abidjan",
+        "Atlantic/Jan_Mayen", "Europe/Oslo",
         "Atlantic/St_Helena", "Africa/Abidjan",
         "Australia/ACT", "Australia/Sydney",
         "Australia/Canberra", "Australia/Sydney",
-        "Australia/Currie", "Australia/Hobart",
         "Australia/LHI", "Australia/Lord_Howe",
         "Australia/NSW", "Australia/Sydney",
         "Australia/North", "Australia/Darwin",
@@ -928,32 +907,23 @@ class ZoneName {
         "Etc/UCT", "Etc/UTC",
         "Etc/Universal", "Etc/UTC",
         "Etc/Zulu", "Etc/UTC",
-        "Europe/Amsterdam", "Europe/Brussels",
         "Europe/Belfast", "Europe/London",
         "Europe/Bratislava", "Europe/Prague",
         "Europe/Busingen", "Europe/Zurich",
-        "Europe/Copenhagen", "Europe/Berlin",
         "Europe/Guernsey", "Europe/London",
         "Europe/Isle_of_Man", "Europe/London",
         "Europe/Jersey", "Europe/London",
-        "Europe/Kiev", "Europe/Kyiv",
         "Europe/Ljubljana", "Europe/Belgrade",
-        "Europe/Luxembourg", "Europe/Brussels",
         "Europe/Mariehamn", "Europe/Helsinki",
-        "Europe/Monaco", "Europe/Paris",
         "Europe/Nicosia", "Asia/Nicosia",
-        "Europe/Oslo", "Europe/Berlin",
         "Europe/Podgorica", "Europe/Belgrade",
         "Europe/San_Marino", "Europe/Rome",
         "Europe/Sarajevo", "Europe/Belgrade",
         "Europe/Skopje", "Europe/Belgrade",
-        "Europe/Stockholm", "Europe/Berlin",
         "Europe/Tiraspol", "Europe/Chisinau",
-        "Europe/Uzhgorod", "Europe/Kyiv",
         "Europe/Vaduz", "Europe/Zurich",
         "Europe/Vatican", "Europe/Rome",
         "Europe/Zagreb", "Europe/Belgrade",
-        "Europe/Zaporozhye", "Europe/Kyiv",
         "GB", "Europe/London",
         "GB-Eire", "Europe/London",
         "GMT", "Etc/GMT",
@@ -962,15 +932,10 @@ class ZoneName {
         "GMT0", "Etc/GMT",
         "Greenwich", "Etc/GMT",
         "Hongkong", "Asia/Hong_Kong",
-        "Iceland", "Africa/Abidjan",
+        "Iceland", "Atlantic/Reykjavik",
         "Indian/Antananarivo", "Africa/Nairobi",
-        "Indian/Christmas", "Asia/Bangkok",
-        "Indian/Cocos", "Asia/Yangon",
         "Indian/Comoro", "Africa/Nairobi",
-        "Indian/Kerguelen", "Indian/Maldives",
-        "Indian/Mahe", "Asia/Dubai",
         "Indian/Mayotte", "Africa/Nairobi",
-        "Indian/Reunion", "Asia/Dubai",
         "Iran", "Asia/Tehran",
         "Israel", "Asia/Jerusalem",
         "Jamaica", "America/Jamaica",
@@ -987,20 +952,13 @@ class ZoneName {
         "NZ-CHAT", "Pacific/Chatham",
         "Navajo", "America/Denver",
         "PRC", "Asia/Shanghai",
-        "Pacific/Chuuk", "Pacific/Port_Moresby",
-        "Pacific/Enderbury", "Pacific/Kanton",
-        "Pacific/Funafuti", "Pacific/Tarawa",
         "Pacific/Johnston", "Pacific/Honolulu",
-        "Pacific/Majuro", "Pacific/Tarawa",
         "Pacific/Midway", "Pacific/Pago_Pago",
-        "Pacific/Pohnpei", "Pacific/Guadalcanal",
-        "Pacific/Ponape", "Pacific/Guadalcanal",
+        "Pacific/Ponape", "Pacific/Pohnpei",
         "Pacific/Saipan", "Pacific/Guam",
         "Pacific/Samoa", "Pacific/Pago_Pago",
-        "Pacific/Truk", "Pacific/Port_Moresby",
-        "Pacific/Wake", "Pacific/Tarawa",
-        "Pacific/Wallis", "Pacific/Tarawa",
-        "Pacific/Yap", "Pacific/Port_Moresby",
+        "Pacific/Truk", "Pacific/Chuuk",
+        "Pacific/Yap", "Pacific/Chuuk",
         "Poland", "Europe/Warsaw",
         "Portugal", "Europe/Lisbon",
         "ROC", "Asia/Taipei",
@@ -1019,6 +977,7 @@ class ZoneName {
         "US/Michigan", "America/Detroit",
         "US/Mountain", "America/Denver",
         "US/Pacific", "America/Los_Angeles",
+        "US/Pacific-New", "America/Los_Angeles",
         "US/Samoa", "Pacific/Pago_Pago",
         "UTC", "Etc/UTC",
         "Universal", "Etc/UTC",
