@@ -78,11 +78,6 @@ public class ForkJoinWorkerThread extends Thread {
     final ForkJoinPool pool;                // the pool this thread works in
     final ForkJoinPool.WorkQueue workQueue; // work-stealing mechanics
 
-    /** An AccessControlContext supporting no privileges */
-    private static final AccessControlContext INNOCUOUS_ACC =
-        new AccessControlContext(
-            new ProtectionDomain[] { new ProtectionDomain(null, null) });
-
     /**
      * Creates a ForkJoinWorkerThread operating in the given pool.
      *
@@ -104,7 +99,6 @@ public class ForkJoinWorkerThread extends Thread {
     ForkJoinWorkerThread(ForkJoinPool pool, ClassLoader ccl) {
         super("aForkJoinWorkerThread");
         super.setContextClassLoader(ccl);
-        ThreadLocalRandom.setInheritedAccessControlContext(this, INNOCUOUS_ACC);
         this.pool = pool;
         this.workQueue = pool.registerWorker(this);
     }
@@ -220,6 +214,11 @@ public class ForkJoinWorkerThread extends Thread {
                         group, "InnocuousForkJoinWorkerThreadGroup");
                 }});
 
+        /** An AccessControlContext supporting no privileges */
+        private static final AccessControlContext INNOCUOUS_ACC =
+            new AccessControlContext(
+                new ProtectionDomain[] { new ProtectionDomain(null, null) });
+
         InnocuousForkJoinWorkerThread(ForkJoinPool pool) {
             super(pool,
                   ClassLoader.getSystemClassLoader(),
@@ -237,7 +236,8 @@ public class ForkJoinWorkerThread extends Thread {
 
         @Override // paranoically
         public void setContextClassLoader(ClassLoader cl) {
-            throw new SecurityException("setContextClassLoader");
+            if (cl != null && ClassLoader.getSystemClassLoader() != cl)
+                throw new SecurityException("setContextClassLoader");
         }
     }
 }
