@@ -203,7 +203,7 @@ public abstract class AbstractPreferences extends Preferences {
      */
     protected AbstractPreferences(AbstractPreferences parent, String name) {
         if (parent==null) {
-            if (!name.equals(""))
+            if (!name.isEmpty())
                 throw new IllegalArgumentException("Root name '"+name+
                                                    "' must be \"\"");
             this.absolutePath = "/";
@@ -212,7 +212,7 @@ public abstract class AbstractPreferences extends Preferences {
             if (name.indexOf('/') != -1)
                 throw new IllegalArgumentException("Name '" + name +
                                                  "' contains '/'");
-            if (name.equals(""))
+            if (name.isEmpty())
               throw new IllegalArgumentException("Illegal name: empty string");
 
             root = parent.root;
@@ -848,7 +848,7 @@ public abstract class AbstractPreferences extends Preferences {
         synchronized(lock) {
             if (removed)
                 throw new IllegalStateException("Node has been removed.");
-            if (path.equals(""))
+            if (path.isEmpty())
                 return this;
             if (path.equals("/"))
                 return root;
@@ -911,7 +911,7 @@ public abstract class AbstractPreferences extends Preferences {
         throws BackingStoreException
     {
         synchronized(lock) {
-            if (path.equals(""))
+            if (path.isEmpty())
                 return !removed;
             if (removed)
                 throw new IllegalStateException("Node has been removed.");
