@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -434,6 +434,7 @@ public class BasicMenuItemUI extends MenuItemUI
         return null;
     }
 
+    @SuppressWarnings("deprecation")
     void updateAcceleratorBinding() {
         KeyStroke accelerator = menuItem.getAccelerator();
         InputMap windowInputMap = SwingUtilities.getUIInputMap(
@@ -450,6 +451,45 @@ public class BasicMenuItemUI extends MenuItemUI
                            JComponent.WHEN_IN_FOCUSED_WINDOW, windowInputMap);
             }
             windowInputMap.put(accelerator, "doClick");
+
+            int modifiers = accelerator.getModifiers();
+            if (((modifiers & InputEvent.ALT_DOWN_MASK) != 0) &&
+                    ((modifiers & InputEvent.ALT_GRAPH_DOWN_MASK) != 0)) {
+                //When both ALT and ALT_GRAPH are set, add the ALT only
+                // modifier keystroke which is used for left ALT key.
+                // Unsetting the ALT_GRAPH will do that as ALT is already set
+                modifiers &= ~InputEvent.ALT_GRAPH_DOWN_MASK;
+                modifiers &= ~InputEvent.ALT_GRAPH_MASK;
+                KeyStroke keyStroke = KeyStroke.getKeyStroke(accelerator.getKeyCode(),
+                        modifiers, accelerator.isOnKeyRelease());
+                windowInputMap.put(keyStroke, "doClick");
+            } else if (((modifiers & InputEvent.ALT_DOWN_MASK) != 0) && (
+                    (modifiers & InputEvent.ALT_GRAPH_DOWN_MASK) == 0)) {
+                //When only ALT modifier is set, add the ALT + ALT_GRAPH
+                // modifier keystroke which is used for right ALT key
+                modifiers |= InputEvent.ALT_GRAPH_DOWN_MASK;
+                KeyStroke keyStroke = KeyStroke.getKeyStroke(accelerator.getKeyCode(),
+                        modifiers, accelerator.isOnKeyRelease());
+                windowInputMap.put(keyStroke, "doClick");
+            } else if ((modifiers & InputEvent.ALT_GRAPH_DOWN_MASK) != 0) {
+                //When only ALT_GRAPH is set, remove the ALT_GRAPH only
+                // modifier and add the ALT and ALT+ALT_GRAPH modifiers
+                // keystroke which are used for left ALT key and right ALT
+                // respectively
+                modifiers &= ~InputEvent.ALT_GRAPH_DOWN_MASK;
+                modifiers &= ~InputEvent.ALT_GRAPH_MASK;
+
+                modifiers |= InputEvent.ALT_DOWN_MASK;
+                KeyStroke keyStroke = KeyStroke.getKeyStroke(accelerator.getKeyCode(),
+                        modifiers, accelerator.isOnKeyRelease());
+                windowInputMap.put(keyStroke, "doClick");
+
+                //Add ALT+ALT_GRAPH modifier which is used for right ALT key
+                modifiers |= InputEvent.ALT_GRAPH_DOWN_MASK;
+                keyStroke = KeyStroke.getKeyStroke(accelerator.getKeyCode(),
+                        modifiers, accelerator.isOnKeyRelease());
+                windowInputMap.put(keyStroke, "doClick");
+            }
         }
     }
 
@@ -676,7 +716,7 @@ public class BasicMenuItemUI extends MenuItemUI
 
     private void paintAccText(Graphics g, MenuItemLayoutHelper lh,
                               MenuItemLayoutHelper.LayoutResult lr) {
-        if (!lh.getAccText().equals("")) {
+        if (!lh.getAccText().isEmpty()) {
             ButtonModel model = lh.getMenuItem().getModel();
             g.setFont(lh.getAccFontMetrics().getFont());
             if (!model.isEnabled()) {
@@ -714,7 +754,7 @@ public class BasicMenuItemUI extends MenuItemUI
 
     private void paintText(Graphics g, MenuItemLayoutHelper lh,
                            MenuItemLayoutHelper.LayoutResult lr) {
-        if (!lh.getText().equals("")) {
+        if (!lh.getText().isEmpty()) {
             if (lh.getHtmlView() != null) {
                 // Text is HTML
                 lh.getHtmlView().paint(g, lr.getTextRect());
@@ -828,8 +868,8 @@ public class BasicMenuItemUI extends MenuItemUI
      */
     public MenuElement[] getPath() {
         MenuSelectionManager m = MenuSelectionManager.defaultManager();
-        MenuElement oldPath[] = m.getSelectedPath();
-        MenuElement newPath[];
+        MenuElement[] oldPath = m.getSelectedPath();
+        MenuElement[] newPath;
         int i = oldPath.length;
         if (i == 0)
             return new MenuElement[0];
@@ -865,7 +905,7 @@ public class BasicMenuItemUI extends MenuItemUI
         return newPath;
     }
 
-    void printMenuElementArray(MenuElement path[], boolean dumpStack) {
+    void printMenuElementArray(MenuElement[] path, boolean dumpStack) {
         System.out.println("Path is(");
         int i, j;
         for(i=0,j=path.length; i<j ;i++){
@@ -1049,9 +1089,9 @@ public class BasicMenuItemUI extends MenuItemUI
                 MenuSelectionManager.defaultManager().processMouseEvent(e);
             } else {
 
-                MenuElement path[] = manager.getSelectedPath();
+                MenuElement[] path = manager.getSelectedPath();
                 if (path.length > 1 && path[path.length-1] == menuItem) {
-                    MenuElement newPath[] = new MenuElement[path.length-1];
+                    MenuElement[] newPath = new MenuElement[path.length-1];
                     int i,c;
                     for(i=0,c=path.length-1;i<c;i++)
                         newPath[i] = path[i];
@@ -1071,12 +1111,12 @@ public class BasicMenuItemUI extends MenuItemUI
         //
         public void menuDragMouseEntered(MenuDragMouseEvent e) {
             MenuSelectionManager manager = e.getMenuSelectionManager();
-            MenuElement path[] = e.getPath();
+            MenuElement[] path = e.getPath();
             manager.setSelectedPath(path);
         }
         public void menuDragMouseDragged(MenuDragMouseEvent e) {
             MenuSelectionManager manager = e.getMenuSelectionManager();
-            MenuElement path[] = e.getPath();
+            MenuElement[] path = e.getPath();
             manager.setSelectedPath(path);
         }
         public void menuDragMouseExited(MenuDragMouseEvent e) {}
@@ -1085,7 +1125,7 @@ public class BasicMenuItemUI extends MenuItemUI
                 return;
             }
             MenuSelectionManager manager = e.getMenuSelectionManager();
-            MenuElement path[] = e.getPath();
+            MenuElement[] path = e.getPath();
             Point p = e.getPoint();
             if (p.x >= 0 && p.x < menuItem.getWidth() &&
                     p.y >= 0 && p.y < menuItem.getHeight()) {
@@ -1105,9 +1145,8 @@ public class BasicMenuItemUI extends MenuItemUI
             if (name == "labelFor" || name == "displayedMnemonic" ||
                 name == "accelerator") {
                 updateAcceleratorBinding();
-            } else if (name == "text" || "font" == name ||
-                       "foreground" == name ||
-                       "ancestor" == name || "graphicsConfiguration" == name) {
+            } else if (name == "text" || "font" == name || "foreground" == name
+                    || SwingUtilities2.isScaleChanged(e)) {
                 // remove the old html view client property if one
                 // existed, and install a new one if the text installed
                 // into the JLabel is html source.
