@@ -427,7 +427,7 @@ final class HotSpotJDKReflection extends HotSpotJVMCIReflection {
             case Double:
                 return JavaConstant.forDouble(isVolatile ? UNSAFE.getDoubleVolatile(obj, displacement) : UNSAFE.getDouble(obj, displacement));
             case Object:
-                return forObject(isVolatile ? UNSAFE.getObjectVolatile(obj, displacement) : UNSAFE.getObject(obj, displacement));
+                return forObject(isVolatile ? UNSAFE.getReferenceVolatile(obj, displacement) : UNSAFE.getReference(obj, displacement));
             default:
                 throw new IllegalArgumentException("Unsupported kind: " + kind);
 
