@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -31,16 +31,17 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.io.PrintStream;
 import java.io.Serializable;
+
 import java.lang.invoke.CallSite;
 import java.lang.invoke.ConstantCallSite;
 import java.lang.invoke.MethodHandle;
+import java.lang.module.ModuleDescriptor.Requires;
 import java.lang.ref.WeakReference;
-import java.nio.ByteBuffer;
-import java.nio.ByteOrder;
+
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.Formatter;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
@@ -48,8 +49,8 @@ import java.util.ServiceLoader;
 import java.util.function.Predicate;
 
 import jdk.internal.misc.Unsafe;
+
 import jdk.vm.ci.code.Architecture;
-import jdk.vm.ci.code.CompilationRequest;
 import jdk.vm.ci.code.CompilationRequestResult;
 import jdk.vm.ci.code.CompiledCode;
 import jdk.vm.ci.code.InstalledCode;
@@ -66,6 +67,7 @@ import jdk.vm.ci.runtime.JVMCICompiler;
 import jdk.vm.ci.runtime.JVMCICompilerFactory;
 import jdk.vm.ci.runtime.JVMCIRuntime;
 import jdk.vm.ci.services.JVMCIServiceLocator;
+import jdk.vm.ci.services.Services;
 
 /**
  * HotSpot implementation of a JVMCI runtime.
@@ -185,15 +187,9 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
                     // initialized.
                     JVMCI.getRuntime();
                 }
-                // Make sure all the primitive box caches are populated (required to properly
-                // materialize boxed primitives
+                // Make sure all the primitive box caches are populated (required to properly materialize boxed primitives
                 // during deoptimization).
-                Boolean.valueOf(false);
-                Byte.valueOf((byte) 0);
-                Short.valueOf((short) 0);
-                Character.valueOf((char) 0);
-                Integer.valueOf(0);
-                Long.valueOf(0);
+                Object[] boxCaches = { Boolean.valueOf(false), Byte.valueOf((byte)0), Short.valueOf((short) 0), Character.valueOf((char) 0), Integer.valueOf(0), Long.valueOf(0) };
             }
         }
         return result;
@@ -214,15 +210,6 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
         return o.toString();
     }
 
-    /**
-     * Set of recognized {@code "jvmci.*"} system properties. Entries not associated with an
-     * {@link Option} have this object as their value.
-     */
-    static final Map<String, Object> options = new HashMap<>();
-    static {
-        options.put("jvmci.class.path.append", options);
-    }
-
     /**
      * A list of all supported JVMCI options.
      */
@@ -236,8 +223,6 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
         // so that -XX:+JVMCIPrintProperties shows the option.
         InitTimer(Boolean.class, false, "Specifies if initialization timing is enabled."),
         PrintConfig(Boolean.class, false, "Prints VM configuration available via JVMCI."),
-        AuditHandles(Boolean.class, false, "Record stack trace along with scoped foreign object reference wrappers " +
-                "to debug issue with a wrapper being used after its scope has closed."),
         TraceMethodDataFilter(String.class, null,
                 "Enables tracing of profiling info when read by JVMCI.",
                 "Empty value: trace all methods",
@@ -258,7 +243,7 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
         private final Class<?> type;
         @NativeImageReinitialize private Object value;
         private final Object defaultValue;
-        private boolean isDefault = true;
+        private boolean isDefault;
         private final String[] helpLines;
 
         Option(Class<?> type, Object defaultValue, String... helpLines) {
@@ -266,37 +251,27 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
             this.type = type;
             this.defaultValue = defaultValue;
             this.helpLines = helpLines;
-            Object existing = options.put(getPropertyName(), this);
-            assert existing == null : getPropertyName();
-        }
-
-        @SuppressFBWarnings(value = "ES_COMPARING_STRINGS_WITH_EQ", justification = "sentinel must be String since it's a static final in an enum")
-        private void init(String propertyValue) {
-            assert value == null : "cannot re-initialize " + name();
-            if (propertyValue == null) {
-                this.value = defaultValue == null ? NULL_VALUE : defaultValue;
-                this.isDefault = true;
-            } else {
-                if (type == Boolean.class) {
-                    this.value = Boolean.parseBoolean(propertyValue);
-                } else if (type == String.class) {
-                    this.value = propertyValue;
-                } else {
-                    throw new JVMCIError("Unexpected option type " + type);
-                }
-                this.isDefault = false;
-            }
         }
 
         @SuppressFBWarnings(value = "ES_COMPARING_STRINGS_WITH_EQ", justification = "sentinel must be String since it's a static final in an enum")
         private Object getValue() {
-            if (value == NULL_VALUE) {
-                return null;
-            }
             if (value == null) {
-                return defaultValue;
+                String propertyValue = Services.getSavedProperty(getPropertyName());
+                if (propertyValue == null) {
+                    this.value = defaultValue == null ? NULL_VALUE : defaultValue;
+                    this.isDefault = true;
+                } else {
+                    if (type == Boolean.class) {
+                        this.value = Boolean.parseBoolean(propertyValue);
+                    } else if (type == String.class) {
+                        this.value = propertyValue;
+                    } else {
+                        throw new JVMCIError("Unexpected option type " + type);
+                    }
+                    this.isDefault = false;
+                }
             }
-            return value;
+            return value == NULL_VALUE ? null : value;
         }
 
         /**
@@ -357,69 +332,11 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
                 }
             }
         }
-
-        /**
-         * Compute string similarity based on Dice's coefficient.
-         *
-         * Ported from str_similar() in globals.cpp.
-         */
-        static float stringSimiliarity(String str1, String str2) {
-            int hit = 0;
-            for (int i = 0; i < str1.length() - 1; ++i) {
-                for (int j = 0; j < str2.length() - 1; ++j) {
-                    if ((str1.charAt(i) == str2.charAt(j)) && (str1.charAt(i + 1) == str2.charAt(j + 1))) {
-                        ++hit;
-                        break;
-                    }
-                }
-            }
-            return 2.0f * hit / (str1.length() + str2.length());
-        }
-
-        private static final float FUZZY_MATCH_THRESHOLD = 0.7F;
-
-        /**
-         * Parses all system properties starting with {@value #JVMCI_OPTION_PROPERTY_PREFIX} and
-         * initializes the options based on their values.
-         *
-         * @param runtime
-         */
-        static void parse(HotSpotJVMCIRuntime runtime) {
-            Map<String, String> savedProps = jdk.vm.ci.services.Services.getSavedProperties();
-            for (Map.Entry<String, String> e : savedProps.entrySet()) {
-                String name = e.getKey();
-                if (name.startsWith(Option.JVMCI_OPTION_PROPERTY_PREFIX)) {
-                    Object value = options.get(name);
-                    if (value == null) {
-                        List<String> matches = new ArrayList<>();
-                        for (String pn : options.keySet()) {
-                            float score = stringSimiliarity(pn, name);
-                            if (score >= FUZZY_MATCH_THRESHOLD) {
-                                matches.add(pn);
-                            }
-                        }
-                        Formatter msg = new Formatter();
-                        msg.format("Error parsing JVMCI options: Could not find option %s", name);
-                        if (!matches.isEmpty()) {
-                            msg.format("%nDid you mean one of the following?");
-                            for (String match : matches) {
-                                msg.format("%n    %s=<value>", match);
-                            }
-                        }
-                        msg.format("%nError: A fatal exception has occurred. Program will exit.%n");
-                        runtime.exitHotSpotWithMessage(1, msg.toString());
-                    } else if (value instanceof Option) {
-                        Option option = (Option) value;
-                        option.init(e.getValue());
-                    }
-                }
-            }
-        }
     }
 
     private static HotSpotJVMCIBackendFactory findFactory(String architecture) {
         Iterable<HotSpotJVMCIBackendFactory> factories = getHotSpotJVMCIBackendFactories();
-        assert factories != null : "sanity";
+assert factories != null : "sanity";
         for (HotSpotJVMCIBackendFactory factory : factories) {
             if (factory.getArchitecture().equalsIgnoreCase(architecture)) {
                 return factory;
@@ -472,35 +389,33 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
     @NativeImageReinitialize private volatile ClassValue<WeakReferenceHolder<HotSpotResolvedJavaType>> resolvedJavaType;
 
     /**
-     * To avoid calling ClassValue.remove to refresh the weak reference, which under certain
-     * circumstances can lead to an infinite loop, we use a permanent holder with a mutable field
-     * that we refresh.
+     * To avoid calling ClassValue.remove to refresh the weak reference, which
+     * under certain circumstances can lead to an infinite loop, we use a
+     * permanent holder with a mutable field that we refresh.
      */
     private static class WeakReferenceHolder<T> {
         private volatile WeakReference<T> ref;
-
         WeakReferenceHolder(T value) {
             set(value);
         }
-
         void set(T value) {
-            ref = new WeakReference<>(value);
+            ref = new WeakReference<T>(value);
         }
-
         T get() {
             return ref.get();
         }
-    }
+    };
 
     @NativeImageReinitialize private HashMap<Long, WeakReference<ResolvedJavaType>> resolvedJavaTypes;
 
     /**
-     * Stores the value set by {@link #excludeFromJVMCICompilation(Module...)} so that it can be
-     * read from the VM.
+     * Stores the value set by {@link #excludeFromJVMCICompilation(Module...)} so that it can
+     * be read from the VM.
      */
     @SuppressWarnings("unused")//
     @NativeImageReinitialize private Module[] excludeFromJVMCICompilation;
 
+
     private final Map<Class<? extends Architecture>, JVMCIBackend> backends = new HashMap<>();
 
     private volatile List<HotSpotVMEventListener> vmEventListeners;
@@ -535,9 +450,6 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
             System.setErr(vmLogStream);
         }
 
-        // Initialize the Option values.
-        Option.parse(this);
-
         String hostArchitecture = config.getHostArchitectureName();
 
         HotSpotJVMCIBackendFactory factory;
@@ -549,7 +461,7 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
             hostBackend = registerBackend(factory.createJVMCIBackend(this, null));
         }
 
-        compilerFactory = HotSpotJVMCICompilerConfig.getCompilerFactory(this);
+        compilerFactory = HotSpotJVMCICompilerConfig.getCompilerFactory();
         if (compilerFactory instanceof HotSpotJVMCICompilerFactory) {
             hsCompilerFactory = (HotSpotJVMCICompilerFactory) compilerFactory;
             if (hsCompilerFactory.getCompilationLevelAdjustment() != None) {
@@ -572,50 +484,10 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
         }
 
         if (Option.PrintConfig.getBoolean()) {
-            configStore.printConfig(this);
+            configStore.printConfig();
         }
     }
 
-    /**
-     * Sets the current thread's {@code JavaThread::_jvmci_reserved_oop<id>} field to {@code value}.
-     *
-     * @throws IllegalArgumentException if the {@code JavaThread::_jvmci_reserved_oop<id>} field
-     *             does not exist
-     */
-    public void setThreadLocalObject(int id, Object value) {
-        compilerToVm.setThreadLocalObject(id, value);
-    }
-
-    /**
-     * Get the value of the current thread's {@code JavaThread::_jvmci_reserved_oop<id>} field.
-     *
-     * @throws IllegalArgumentException if the {@code JavaThread::_jvmci_reserved_oop<id>} field
-     *             does not exist
-     */
-    public Object getThreadLocalObject(int id) {
-        return compilerToVm.getThreadLocalObject(id);
-    }
-
-    /**
-     * Sets the current thread's {@code JavaThread::_jvmci_reserved<id>} field to {@code value}.
-     *
-     * @throws IllegalArgumentException if the {@code JavaThread::_jvmci_reserved<id>} field does
-     *             not exist
-     */
-    public void setThreadLocalLong(int id, long value) {
-        compilerToVm.setThreadLocalLong(id, value);
-    }
-
-    /**
-     * Get the value of the current thread's {@code JavaThread::_jvmci_reserved<id>} field.
-     *
-     * @throws IllegalArgumentException if the {@code JavaThread::_jvmci_reserved<id>} field does
-     *             not exist
-     */
-    public long getThreadLocalLong(int id) {
-        return compilerToVm.getThreadLocalLong(id);
-    }
-
     HotSpotResolvedJavaType createClass(Class<?> javaClass) {
         if (javaClass.isPrimitive()) {
             return HotSpotResolvedPrimitiveType.forKind(JavaKind.fromJavaClass(javaClass));
@@ -634,7 +506,7 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
         if (resolvedJavaType == null) {
             synchronized (this) {
                 if (resolvedJavaType == null) {
-                    resolvedJavaType = new ClassValue<>() {
+                    resolvedJavaType = new ClassValue<WeakReferenceHolder<HotSpotResolvedJavaType>>() {
                         @Override
                         protected WeakReferenceHolder<HotSpotResolvedJavaType> computeValue(Class<?> type) {
                             return new WeakReferenceHolder<>(createClass(type));
@@ -648,7 +520,8 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
         HotSpotResolvedJavaType javaType = ref.get();
         if (javaType == null) {
             /*
-             * If the referent has become null, create a new value and update cached weak reference.
+             * If the referent has become null, create a new value and
+             * update cached weak reference.
              */
             javaType = createClass(javaClass);
             ref.set(javaType);
@@ -716,7 +589,7 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
      *            compiler.
      */
     public Predicate<ResolvedJavaType> getIntrinsificationTrustPredicate(Class<?>... compilerLeafClasses) {
-        return new Predicate<>() {
+        return new Predicate<ResolvedJavaType>() {
             @Override
             public boolean test(ResolvedJavaType type) {
                 if (type instanceof HotSpotResolvedObjectTypeImpl) {
@@ -744,19 +617,6 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
         return null;
     }
 
-    static class ErrorCreatingCompiler implements JVMCICompiler {
-        private final RuntimeException t;
-
-        ErrorCreatingCompiler(RuntimeException t) {
-            this.t = t;
-        }
-
-        @Override
-        public CompilationRequestResult compileMethod(CompilationRequest request) {
-            throw t;
-        }
-    }
-
     @Override
     public JVMCICompiler getCompiler() {
         if (compiler == null) {
@@ -764,19 +624,11 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
                 if (compiler == null) {
                     assert !creatingCompiler : "recursive compiler creation";
                     creatingCompiler = true;
-                    try {
-                        compiler = compilerFactory.createCompiler(this);
-                    } catch (RuntimeException t) {
-                        compiler = new ErrorCreatingCompiler(t);
-                    } finally {
-                        creatingCompiler = false;
-                    }
+                    compiler = compilerFactory.createCompiler(this);
+                    creatingCompiler = false;
                 }
             }
         }
-        if (compiler instanceof ErrorCreatingCompiler) {
-            throw ((ErrorCreatingCompiler) compiler).t;
-        }
         return compiler;
     }
 
@@ -835,11 +687,9 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
         return Collections.unmodifiableMap(backends);
     }
 
-    @SuppressWarnings("try")
     @VMEntryPoint
     private HotSpotCompilationRequestResult compileMethod(HotSpotResolvedJavaMethod method, int entryBCI, long compileState, int id) {
-        HotSpotCompilationRequest request = new HotSpotCompilationRequest(method, entryBCI, compileState, id);
-        CompilationRequestResult result = getCompiler().compileMethod(request);
+        CompilationRequestResult result = getCompiler().compileMethod(new HotSpotCompilationRequest(method, entryBCI, compileState, id));
         assert result != null : "compileMethod must always return something";
         HotSpotCompilationRequestResult hsResult;
         if (result instanceof HotSpotCompilationRequestResult) {
@@ -854,28 +704,21 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
                 hsResult = HotSpotCompilationRequestResult.success(inlinedBytecodes);
             }
         }
+
         return hsResult;
     }
 
-    /**
-     * Guard to ensure shut down actions are performed at most once.
-     */
-    private boolean isShutdown;
-
     /**
      * Shuts down the runtime.
      */
     @VMEntryPoint
-    private synchronized void shutdown() throws Exception {
-        if (!isShutdown) {
-            isShutdown = true;
-            // Cleaners are normally only processed when a new Cleaner is
-            // instantiated so process all remaining cleaners now.
-            Cleaner.clean();
+    private void shutdown() throws Exception {
+        // Cleaners are normally only processed when a new Cleaner is
+        // instantiated so process all remaining cleaners now.
+        Cleaner.clean();
 
-            for (HotSpotVMEventListener vmEventListener : getVmEventListeners()) {
-                vmEventListener.notifyShutdown();
-            }
+        for (HotSpotVMEventListener vmEventListener : getVmEventListeners()) {
+            vmEventListener.notifyShutdown();
         }
     }
 
@@ -916,46 +759,7 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
      * @throws IndexOutOfBoundsException if copying would cause access of data outside array bounds
      */
     public int writeDebugOutput(byte[] bytes, int offset, int length, boolean flush, boolean canThrow) {
-        return writeDebugOutput0(compilerToVm, bytes, offset, length, flush, canThrow);
-    }
-
-    /**
-     * @see #writeDebugOutput
-     */
-    static int writeDebugOutput0(CompilerToVM vm, byte[] bytes, int offset, int length, boolean flush, boolean canThrow) {
-        if (bytes == null) {
-            if (!canThrow) {
-                return -1;
-            }
-            throw new NullPointerException();
-        }
-        if (offset < 0 || length < 0 || offset + length > bytes.length) {
-            if (!canThrow) {
-                return -2;
-            }
-            throw new ArrayIndexOutOfBoundsException();
-        }
-        if (length <= 8) {
-            ByteBuffer buffer = ByteBuffer.wrap(bytes, offset, length);
-            if (length != 8) {
-                ByteBuffer buffer8 = ByteBuffer.allocate(8);
-                buffer8.put(buffer);
-                buffer8.position(8);
-                buffer = buffer8;
-            }
-            buffer.order(ByteOrder.nativeOrder());
-            vm.writeDebugOutput(buffer.getLong(0), length, flush);
-        } else {
-            Unsafe unsafe = UnsafeAccess.UNSAFE;
-            long buffer = unsafe.allocateMemory(length);
-            try {
-                unsafe.copyMemory(bytes, Unsafe.ARRAY_BYTE_BASE_OFFSET, null, buffer, length);
-                vm.writeDebugOutput(buffer, length, flush);
-            } finally {
-                unsafe.freeMemory(buffer);
-            }
-        }
-        return 0;
+        return compilerToVm.writeDebugOutput(bytes, offset, length, flush, canThrow);
     }
 
     /**
@@ -973,7 +777,7 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
                 } else if (len == 0) {
                     return;
                 }
-                writeDebugOutput(b, off, len, false, true);
+                compilerToVm.writeDebugOutput(b, off, len, false, true);
             }
 
             @Override
@@ -1107,8 +911,8 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
      * </pre>
      *
      * The implementation of the native {@code JCompile.compile0} method would be in the JVMCI
-     * shared library that contains the JVMCI compiler. The {@code JCompile.compile0} implementation
-     * must be exported as the following JNI-compatible symbol:
+     * shared library that contains the bulk of the JVMCI compiler. The {@code JCompile.compile0}
+     * implementation will be exported as the following JNI-compatible symbol:
      *
      * <pre>
      * Java_com_jcompile_JCompile_compile0
@@ -1119,18 +923,9 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
      * @see "https://docs.oracle.com/javase/8/docs/technotes/guides/jni/spec/invocation.html#invocation_api_functions"
      *
      *
-     * @return info about the Java VM in the JVMCI shared library {@code JavaVM*}. The info is
-     *         encoded in a long array as follows:
-     *
-     *         <pre>
-     *     long[] info = {
-     *         javaVM, // the {@code JavaVM*} value
-     *         javaVM->functions->reserved0,
-     *         javaVM->functions->reserved1,
-     *         javaVM->functions->reserved2
-     *     }
-     *         </pre>
-     *
+     * @return an array of 4 longs where the first value is the {@code JavaVM*} value representing
+     *         the Java VM in the JVMCI shared library, and the remaining values are the first 3
+     *         pointers in the Invocation API function table (i.e., {@code JNIInvokeInterface})
      * @throws NullPointerException if {@code clazz == null}
      * @throws UnsupportedOperationException if the JVMCI shared library is not enabled (i.e.
      *             {@code -XX:-UseJVMCINativeLibrary})
@@ -1201,14 +996,6 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
         return compilerToVm.isCurrentThreadAttached();
     }
 
-    /**
-     * Gets the address of the HotSpot {@code JavaThread} C++ object for the current thread. This
-     * will return {@code 0} if called from an unattached JVMCI shared library thread.
-     */
-    public long getCurrentJavaThread() {
-        return compilerToVm.getCurrentJavaThread();
-    }
-
     /**
      * Ensures the current thread is attached to the peer runtime.
      *
@@ -1219,8 +1006,6 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
      *             {@code -XX:-UseJVMCINativeLibrary})
      * @throws IllegalStateException if the peer runtime has not been initialized or there is an
      *             error while trying to attach the thread
-     * @throws ArrayIndexOutOfBoundsException if {@code javaVMInfo} is non-null and is shorter than
-     *             the length of the array returned by {@link #registerNativeMethods}
      */
     public boolean attachCurrentThread(boolean asDaemon) {
         return compilerToVm.attachCurrentThread(asDaemon);
@@ -1239,33 +1024,12 @@ public final class HotSpotJVMCIRuntime implements JVMCIRuntime {
     }
 
     /**
-     * Informs HotSpot that no method whose module is in {@code modules} is to be compiled with
-     * {@link #compileMethod}.
+     * Informs HotSpot that no method whose module is in {@code modules} is to be compiled
+     * with {@link #compileMethod}.
      *
      * @param modules the set of modules containing JVMCI compiler classes
      */
-    public void excludeFromJVMCICompilation(Module... modules) {
+    public void excludeFromJVMCICompilation(Module...modules) {
         this.excludeFromJVMCICompilation = modules.clone();
     }
-
-    /**
-     * Calls {@link System#exit(int)} in HotSpot's runtime.
-     */
-    public void exitHotSpot(int status) {
-        if (!IS_IN_NATIVE_IMAGE) {
-            System.exit(status);
-        }
-        compilerToVm.callSystemExit(status);
-    }
-
-    /**
-     * Writes a message to HotSpot's log stream and then calls {@link System#exit(int)} in HotSpot's
-     * runtime.
-     */
-    JVMCIError exitHotSpotWithMessage(int status, String format, Object... args) {
-        byte[] messageBytes = String.format(format, args).getBytes();
-        writeDebugOutput(messageBytes, 0, messageBytes.length, true, true);
-        exitHotSpot(status);
-        throw JVMCIError.shouldNotReachHere();
-    }
 }
