@@ -72,6 +72,7 @@ final class HotSpotResolvedObjectTypeImpl extends HotSpotResolvedJavaType implem
     private volatile HotSpotResolvedJavaField[] instanceFields;
     private volatile HotSpotResolvedObjectTypeImpl[] interfaces;
     private HotSpotConstantPool constantPool;
+    private HotSpotResolvedObjectType arrayOfType;
     private final JavaConstant mirror;
     private HotSpotResolvedObjectTypeImpl superClass;
 
@@ -102,24 +103,17 @@ final class HotSpotResolvedObjectTypeImpl extends HotSpotResolvedJavaType implem
      * Creates the JVMCI mirror for a {@link Class} object.
      *
      * <b>NOTE</b>: Creating an instance of this class does not install the mirror for the
-     * {@link Class} type.
+     * {@link Class} type. {@link #fromMetaspace} instead.
      * </p>
      *
      * @param metadataPointer the Klass* to create the mirror for
      */
-    @SuppressWarnings("try")
     HotSpotResolvedObjectTypeImpl(long metadataPointer, String name) {
         super(name);
-        assert metadataPointer != 0;
         this.metadataPointer = metadataPointer;
-
-        // The mirror object must be in the global scope since
-        // this object will be cached in HotSpotJVMCIRuntime.resolvedJavaTypes
-        // and live across more than one compilation.
-        try (HotSpotObjectConstantScope global = HotSpotObjectConstantScope.enterGlobalScope()) {
-            this.mirror = runtime().compilerToVm.getJavaMirror(this);
-            assert getName().charAt(0) != '[' || isArray() : getName();
-        }
+        this.mirror = runtime().compilerToVm.getJavaMirror(this);
+        assert metadataPointer != 0;
+        assert getName().charAt(0) != '[' || isArray() : getName();
     }
 
     /**
@@ -152,6 +146,18 @@ final class HotSpotResolvedObjectTypeImpl extends HotSpotResolvedJavaType implem
         return UNSAFE.getInt(getMetaspaceKlass() + config.klassAccessFlagsOffset);
     }
 
+    @Override
+    public HotSpotResolvedObjectType getArrayClass() {
+        if (arrayOfType == null) {
+            try {
+                arrayOfType = (HotSpotResolvedObjectType) runtime().compilerToVm.lookupType("[" + getName(), this, true);
+            } catch (ClassNotFoundException e) {
+                throw new JVMCIError(e);
+            }
+        }
+        return arrayOfType;
+    }
+
     @Override
     public ResolvedJavaType getComponentType() {
         return runtime().compilerToVm.getComponentType(this);
@@ -558,10 +564,6 @@ final class HotSpotResolvedObjectTypeImpl extends HotSpotResolvedJavaType implem
          * a deopt instead since they can't really be used if they aren't linked yet.
          */
         if (!declaredHolder.isAssignableFrom(this) || this.isArray() || this.equals(declaredHolder) || !isLinked() || isInterface()) {
-            if (hmethod.canBeStaticallyBound()) {
-                // No assumptions are required.
-                return new AssumptionResult<>(hmethod);
-            }
             ResolvedJavaMethod result = hmethod.uniqueConcreteMethod(declaredHolder);
             if (result != null) {
                 return new AssumptionResult<>(result, new ConcreteMethod(method, declaredHolder, result));
@@ -577,7 +579,6 @@ final class HotSpotResolvedObjectTypeImpl extends HotSpotResolvedJavaType implem
             // The type isn't known to implement the method.
             return null;
         }
-
         if (resolvedMethod.canBeStaticallyBound()) {
             // No assumptions are required.
             return new AssumptionResult<>(resolvedMethod);
@@ -995,8 +996,8 @@ final class HotSpotResolvedObjectTypeImpl extends HotSpotResolvedJavaType implem
     }
 
     @Override
-    public boolean isAnonymous() {
-        return (getMiscFlags() & config().instanceKlassMiscIsAnonymous) != 0;
+    public boolean isUnsafeAnonymous() {
+        return (getMiscFlags() & config().instanceKlassMiscIsUnsafeAnonymous) != 0;
     }
 
 }
