@@ -59,26 +59,29 @@
 package jdk.internal.org.objectweb.asm;
 
 /**
- * Defines the JVM opcodes, access flags and array type codes. This interface
- * does not define all the JVM opcodes because some opcodes are automatically
- * handled. For example, the xLOAD and xSTORE opcodes are automatically replaced
- * by xLOAD_n and xSTORE_n opcodes when possible. The xLOAD_n and xSTORE_n
- * opcodes are therefore not defined in this interface. Likewise for LDC,
- * automatically replaced by LDC_W or LDC2_W when necessary, WIDE, GOTO_W and
- * JSR_W.
+ * The JVM opcodes, access flags and array type codes. This interface does not define all the JVM
+ * opcodes because some opcodes are automatically handled. For example, the xLOAD and xSTORE opcodes
+ * are automatically replaced by xLOAD_n and xSTORE_n opcodes when possible. The xLOAD_n and
+ * xSTORE_n opcodes are therefore not defined in this interface. Likewise for LDC, automatically
+ * replaced by LDC_W or LDC2_W when necessary, WIDE, GOTO_W and JSR_W.
  *
+ * @see <a href="https://docs.oracle.com/javase/specs/jvms/se11/html/jvms-6.html">JVMS 6</a>
  * @author Eric Bruneton
  * @author Eugene Kuleshov
  */
+// DontCheck(InterfaceIsType): can't be fixed (for backward binary compatibility).
 public interface Opcodes {
 
-    // ASM API versions
+    // ASM API versions.
 
-    int ASM4 = 4 << 16 | 0 << 8 | 0;
-    int ASM5 = 5 << 16 | 0 << 8 | 0;
-    int ASM6 = 6 << 16 | 0 << 8 | 0;
+    int ASM4 = 4 << 16 | 0 << 8;
+    int ASM5 = 5 << 16 | 0 << 8;
+    int ASM6 = 6 << 16 | 0 << 8;
+    int ASM7 = 7 << 16 | 0 << 8;
 
-    // versions
+    // Java ClassFile versions (the minor version is stored in the 16 most
+    // significant bits, and the
+    // major version in the 16 least significant bits).
 
     int V1_1 = 3 << 16 | 45;
     int V1_2 = 0 << 16 | 46;
@@ -90,8 +93,23 @@ public interface Opcodes {
     int V1_8 = 0 << 16 | 52;
     int V9 = 0 << 16 | 53;
     int V10 = 0 << 16 | 54;
+    int V11 = 0 << 16 | 55;
+    int V12 = 0 << 16 | 56;
+    int V13 = 0 << 16 | 57;
 
-    // access flags
+    /**
+      * Version flag indicating that the class is using 'preview' features.
+      *
+      * <p>{@code version & V_PREVIEW == V_PREVIEW} tests if a version is flagged with {@code
+      * V_PREVIEW}.
+      */
+    int V_PREVIEW = 0xFFFF0000;
+
+    // Access flags values, defined in
+    // - https://docs.oracle.com/javase/specs/jvms/se9/html/jvms-4.html#jvms-4.1-200-E.1
+    // - https://docs.oracle.com/javase/specs/jvms/se9/html/jvms-4.html#jvms-4.5-200-A.1
+    // - https://docs.oracle.com/javase/specs/jvms/se9/html/jvms-4.html#jvms-4.6-200-A.1
+    // - https://docs.oracle.com/javase/specs/jvms/se9/html/jvms-4.html#jvms-4.7.25
 
     int ACC_PUBLIC = 0x0001; // class, field, method
     int ACC_PRIVATE = 0x0002; // class, field, method
@@ -117,12 +135,15 @@ public interface Opcodes {
     int ACC_MANDATED = 0x8000; // parameter, module, module *
     int ACC_MODULE = 0x8000; // class
 
-
-    // ASM specific pseudo access flags
+    // ASM specific access flags.
+    // WARNING: the 16 least significant bits must NOT be used, to avoid conflicts with standard
+    // access flags, and also to make sure that these flags are automatically filtered out when
+    // written in class files (because access flags are stored using 16 bits only).
 
     int ACC_DEPRECATED = 0x20000; // class, field, method
 
-    // types for NEWARRAY
+    // Possible values for the type operand of the NEWARRAY instruction.
+    // See https://docs.oracle.com/javase/specs/jvms/se9/html/jvms-6.html#jvms-6.5.newarray.
 
     int T_BOOLEAN = 4;
     int T_CHAR = 5;
@@ -133,7 +154,8 @@ public interface Opcodes {
     int T_INT = 10;
     int T_LONG = 11;
 
-    // tags for Handle
+    // Possible values for the reference_kind field of CONSTANT_MethodHandle_info structures.
+    // See https://docs.oracle.com/javase/specs/jvms/se9/html/jvms-4.html#jvms-4.4.8.
 
     int H_GETFIELD = 1;
     int H_GETSTATIC = 2;
@@ -145,57 +167,50 @@ public interface Opcodes {
     int H_NEWINVOKESPECIAL = 8;
     int H_INVOKEINTERFACE = 9;
 
-    // stack map frame types
+    // ASM specific stack map frame types, used in {@link ClassVisitor#visitFrame}.
 
-    /**
-     * Represents an expanded frame. See {@link ClassReader#EXPAND_FRAMES}.
-     */
+    /** An expanded frame. See {@link ClassReader#EXPAND_FRAMES}. */
     int F_NEW = -1;
 
-    /**
-     * Represents a compressed frame with complete frame data.
-     */
+    /** A compressed frame with complete frame data. */
     int F_FULL = 0;
 
     /**
-     * Represents a compressed frame where locals are the same as the locals in
-     * the previous frame, except that additional 1-3 locals are defined, and
-     * with an empty stack.
-     */
+      * A compressed frame where locals are the same as the locals in the previous frame, except that
+      * additional 1-3 locals are defined, and with an empty stack.
+      */
     int F_APPEND = 1;
 
     /**
-     * Represents a compressed frame where locals are the same as the locals in
-     * the previous frame, except that the last 1-3 locals are absent and with
-     * an empty stack.
-     */
+      * A compressed frame where locals are the same as the locals in the previous frame, except that
+      * the last 1-3 locals are absent and with an empty stack.
+      */
     int F_CHOP = 2;
 
     /**
-     * Represents a compressed frame with exactly the same locals as the
-     * previous frame and with an empty stack.
-     */
+      * A compressed frame with exactly the same locals as the previous frame and with an empty stack.
+      */
     int F_SAME = 3;
 
     /**
-     * Represents a compressed frame with exactly the same locals as the
-     * previous frame and with a single value on the stack.
-     */
+      * A compressed frame with exactly the same locals as the previous frame and with a single value
+      * on the stack.
+      */
     int F_SAME1 = 4;
 
-    // Do not try to change the following code to use auto-boxing,
-    // these values are compared by reference and not by value
-    // The constructor of Integer was deprecated in 9
-    // but we are stuck with it by backward compatibility
-    @SuppressWarnings("deprecation") Integer TOP = new Integer(0);
-    @SuppressWarnings("deprecation") Integer INTEGER = new Integer(1);
-    @SuppressWarnings("deprecation") Integer FLOAT = new Integer(2);
-    @SuppressWarnings("deprecation") Integer DOUBLE = new Integer(3);
-    @SuppressWarnings("deprecation") Integer LONG = new Integer(4);
-    @SuppressWarnings("deprecation") Integer NULL = new Integer(5);
-    @SuppressWarnings("deprecation") Integer UNINITIALIZED_THIS = new Integer(6);
+    // Standard stack map frame element types, used in {@link ClassVisitor#visitFrame}.
 
-    // opcodes // visit method (- = idem)
+    Integer TOP = Frame.ITEM_TOP;
+    Integer INTEGER = Frame.ITEM_INTEGER;
+    Integer FLOAT = Frame.ITEM_FLOAT;
+    Integer DOUBLE = Frame.ITEM_DOUBLE;
+    Integer LONG = Frame.ITEM_LONG;
+    Integer NULL = Frame.ITEM_NULL;
+    Integer UNINITIALIZED_THIS = Frame.ITEM_UNINITIALIZED_THIS;
+
+    // The JVM opcode values (with the MethodVisitor method name used to visit them in comment, and
+    // where '-' means 'same method name as on the previous line').
+    // See https://docs.oracle.com/javase/specs/jvms/se9/html/jvms-6.html.
 
     int NOP = 0; // visitInsn
     int ACONST_NULL = 1; // -
@@ -216,33 +231,11 @@ public interface Opcodes {
     int BIPUSH = 16; // visitIntInsn
     int SIPUSH = 17; // -
     int LDC = 18; // visitLdcInsn
-    // int LDC_W = 19; // -
-    // int LDC2_W = 20; // -
     int ILOAD = 21; // visitVarInsn
     int LLOAD = 22; // -
     int FLOAD = 23; // -
     int DLOAD = 24; // -
     int ALOAD = 25; // -
-    // int ILOAD_0 = 26; // -
-    // int ILOAD_1 = 27; // -
-    // int ILOAD_2 = 28; // -
-    // int ILOAD_3 = 29; // -
-    // int LLOAD_0 = 30; // -
-    // int LLOAD_1 = 31; // -
-    // int LLOAD_2 = 32; // -
-    // int LLOAD_3 = 33; // -
-    // int FLOAD_0 = 34; // -
-    // int FLOAD_1 = 35; // -
-    // int FLOAD_2 = 36; // -
-    // int FLOAD_3 = 37; // -
-    // int DLOAD_0 = 38; // -
-    // int DLOAD_1 = 39; // -
-    // int DLOAD_2 = 40; // -
-    // int DLOAD_3 = 41; // -
-    // int ALOAD_0 = 42; // -
-    // int ALOAD_1 = 43; // -
-    // int ALOAD_2 = 44; // -
-    // int ALOAD_3 = 45; // -
     int IALOAD = 46; // visitInsn
     int LALOAD = 47; // -
     int FALOAD = 48; // -
@@ -256,26 +249,6 @@ public interface Opcodes {
     int FSTORE = 56; // -
     int DSTORE = 57; // -
     int ASTORE = 58; // -
-    // int ISTORE_0 = 59; // -
-    // int ISTORE_1 = 60; // -
-    // int ISTORE_2 = 61; // -
-    // int ISTORE_3 = 62; // -
-    // int LSTORE_0 = 63; // -
-    // int LSTORE_1 = 64; // -
-    // int LSTORE_2 = 65; // -
-    // int LSTORE_3 = 66; // -
-    // int FSTORE_0 = 67; // -
-    // int FSTORE_1 = 68; // -
-    // int FSTORE_2 = 69; // -
-    // int FSTORE_3 = 70; // -
-    // int DSTORE_0 = 71; // -
-    // int DSTORE_1 = 72; // -
-    // int DSTORE_2 = 73; // -
-    // int DSTORE_3 = 74; // -
-    // int ASTORE_0 = 75; // -
-    // int ASTORE_1 = 76; // -
-    // int ASTORE_2 = 77; // -
-    // int ASTORE_3 = 78; // -
     int IASTORE = 79; // visitInsn
     int LASTORE = 80; // -
     int FASTORE = 81; // -
@@ -393,10 +366,7 @@ public interface Opcodes {
     int INSTANCEOF = 193; // -
     int MONITORENTER = 194; // visitInsn
     int MONITOREXIT = 195; // -
-    // int WIDE = 196; // NOT VISITED
     int MULTIANEWARRAY = 197; // visitMultiANewArrayInsn
     int IFNULL = 198; // visitJumpInsn
     int IFNONNULL = 199; // -
-    // int GOTO_W = 200; // -
-    // int JSR_W = 201; // -
 }
