@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1998, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,17 +25,21 @@
 
 package jdk.javadoc.internal.doclets.formats.html;
 
+import java.util.EnumMap;
+import java.util.Objects;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
 import jdk.javadoc.internal.doclets.formats.html.markup.Comment;
 import jdk.javadoc.internal.doclets.formats.html.markup.ContentBuilder;
+import jdk.javadoc.internal.doclets.formats.html.markup.Entity;
 import jdk.javadoc.internal.doclets.formats.html.markup.FixedStringContent;
-import jdk.javadoc.internal.doclets.formats.html.markup.RawHtml;
 import jdk.javadoc.internal.doclets.formats.html.markup.StringContent;
 import jdk.javadoc.internal.doclets.toolkit.Content;
 import jdk.javadoc.internal.doclets.toolkit.Resources;
 import jdk.javadoc.internal.doclets.toolkit.util.DocletConstants;
+import jdk.javadoc.internal.doclets.toolkit.util.VisibleMemberTable;
+
 
 /**
  * Constants and factory methods for common fragments of content
@@ -48,8 +52,6 @@ import jdk.javadoc.internal.doclets.toolkit.util.DocletConstants;
  * only created once per doclet-instance, instead of once per generated page.
  */
 public class Contents {
-    public static final Content SPACE = RawHtml.nbsp;
-    public static final Content ZERO_WIDTH_SPACE = RawHtml.zws;
 
     public final Content allClassesLabel;
     public final Content allImplementedInterfacesLabel;
@@ -114,7 +116,6 @@ public class Contents {
     public final Content indexLabel;
     public final Content interfaceLabel;
     public final Content interfaces;
-    public final Content interfacesItalic;
     public final Content methodDetailLabel;
     public final Content methodLabel;
     public final Content methodSummary;
@@ -140,11 +141,6 @@ public class Contents {
     public final Content navServices;
     public final Content nestedClassSummary;
     public final Content newPage;
-    public final Content nextClassLabel;
-    public final Content nextLabel;
-    public final Content nextLetter;
-    public final Content nextModuleLabel;
-    public final Content nextPackageLabel;
     public final Content noFramesLabel;
     public final Content noScriptMessage;
     public final Content openModuleLabel;
@@ -155,17 +151,13 @@ public class Contents {
     public final Content packageLabel;
     public final Content package_;
     public final Content packagesLabel;
-    public final Content prevClassLabel;
-    public final Content prevLabel;
-    public final Content prevLetter;
-    public final Content prevModuleLabel;
-    public final Content prevPackageLabel;
     public final Content properties;
     public final Content propertyLabel;
     public final Content propertyDetailsLabel;
-    public final Content propertySummary;
+    public final Content propertySummaryLabel;
     public final Content seeLabel;
     public final Content serializedForm;
+    public final Content servicesLabel;
     public final Content specifiedByLabel;
     public final Content subclassesLabel;
     public final Content subinterfacesLabel;
@@ -175,6 +167,8 @@ public class Contents {
     public final Content useLabel;
     public final Content valueLabel;
 
+    private final EnumMap<VisibleMemberTable.Kind, Content> navLinkLabels;
+
     private final Resources resources;
 
     /**
@@ -249,7 +243,6 @@ public class Contents {
         indexLabel = getContent("doclet.Index");
         interfaceLabel = getContent("doclet.Interface");
         interfaces = getContent("doclet.Interfaces");
-        interfacesItalic = getContent("doclet.Interfaces_Italic");
         methodDetailLabel = getContent("doclet.Method_Detail");
         methodSummary = getContent("doclet.Method_Summary");
         methodLabel = getContent("doclet.Method");
@@ -275,11 +268,6 @@ public class Contents {
         navServices = getContent("doclet.navServices");
         nestedClassSummary = getContent("doclet.Nested_Class_Summary");
         newPage = new Comment(resources.getText("doclet.New_Page"));
-        nextClassLabel = getNonBreakContent("doclet.Next_Class");
-        nextLabel = getNonBreakContent("doclet.Next");
-        nextLetter = getContent("doclet.Next_Letter");
-        nextModuleLabel = getNonBreakContent("doclet.Next_Module");
-        nextPackageLabel = getNonBreakContent("doclet.Next_Package");
         noFramesLabel = getNonBreakContent("doclet.No_Frames");
         noScriptMessage = getContent("doclet.No_Script_Message");
         openedTo = getContent("doclet.OpenedTo");
@@ -290,17 +278,13 @@ public class Contents {
         packageLabel = getContent("doclet.Package");
         package_ = getContent("doclet.package");
         packagesLabel = getContent("doclet.Packages");
-        prevClassLabel = getNonBreakContent("doclet.Prev_Class");
-        prevLabel = getContent("doclet.Prev");
-        prevLetter = getContent("doclet.Prev_Letter");
-        prevModuleLabel = getNonBreakContent("doclet.Prev_Module");
-        prevPackageLabel = getNonBreakContent("doclet.Prev_Package");
         properties = getContent("doclet.Properties");
         propertyLabel = getContent("doclet.Property");
         propertyDetailsLabel = getContent("doclet.Property_Detail");
-        propertySummary = getContent("doclet.Property_Summary");
+        propertySummaryLabel = getContent("doclet.Property_Summary");
         seeLabel = getContent("doclet.See");
         serializedForm = getContent("doclet.Serialized_Form");
+        servicesLabel = getContent("doclet.Services");
         specifiedByLabel = getContent("doclet.Specified_By");
         subclassesLabel = getContent("doclet.Subclasses");
         subinterfacesLabel = getContent("doclet.Subinterfaces");
@@ -309,6 +293,13 @@ public class Contents {
         typeLabel = getContent("doclet.Type");
         useLabel = getContent("doclet.navClassUse");
         valueLabel = getContent("doclet.Value");
+
+        navLinkLabels = new EnumMap<>(VisibleMemberTable.Kind.class);
+        navLinkLabels.put(VisibleMemberTable.Kind.INNER_CLASSES, getContent("doclet.navNested"));
+        navLinkLabels.put(VisibleMemberTable.Kind.ENUM_CONSTANTS, getContent("doclet.navEnum"));
+        navLinkLabels.put(VisibleMemberTable.Kind.FIELDS, getContent("doclet.navField"));
+        navLinkLabels.put(VisibleMemberTable.Kind.CONSTRUCTORS, getContent("doclet.navConstructor"));
+        navLinkLabels.put(VisibleMemberTable.Kind.METHODS, getContent("doclet.navMethod"));
     }
 
     /**
@@ -367,7 +358,7 @@ public class Contents {
         Matcher m = p.matcher(text);
         int start = 0;
         while (m.find(start)) {
-            c.addContent(text.substring(start, m.start()));
+            c.add(text.substring(start, m.start()));
 
             Object o = null;
             switch (m.group(1).charAt(0)) {
@@ -377,17 +368,17 @@ public class Contents {
             }
 
             if (o == null) {
-                c.addContent("{" + m.group(1) + "}");
+                c.add("{" + m.group(1) + "}");
             } else if (o instanceof String) {
-                c.addContent((String) o);
+                c.add((String) o);
             } else if (o instanceof Content) {
-                c.addContent((Content) o);
+                c.add((Content) o);
             }
 
             start = m.end();
         }
 
-        c.addContent(text.substring(start));
+        c.add(text.substring(start));
         return c;
     }
 
@@ -406,11 +397,20 @@ public class Contents {
         int start = 0;
         int p;
         while ((p = text.indexOf(" ", start)) != -1) {
-            c.addContent(text.substring(start, p));
-            c.addContent(RawHtml.nbsp);
+            c.add(text.substring(start, p));
+            c.add(Entity.NO_BREAK_SPACE);
             start = p + 1;
         }
-        c.addContent(text.substring(start));
+        c.add(text.substring(start));
         return c; // TODO: should be made immutable
     }
+
+    /**
+     * Returns a content for a visible member kind.
+     * @param kind the visible member table kind.
+     * @return the string content
+     */
+    public Content getNavLinkLabelContent(VisibleMemberTable.Kind kind) {
+        return Objects.requireNonNull(navLinkLabels.get(kind));
+    }
 }
