@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -51,18 +51,16 @@ import jdk.javadoc.internal.doclets.toolkit.util.DocPaths;
  *  deletion without notice.</b>
  */
 public class Head {
-    private final HtmlVersion htmlVersion;
     private final String docletVersion;
     private final DocPath pathToRoot;
     private String title;
     private String charset;
     private final List<String> keywords;
+    private String description;
+    private String generator;
     private boolean showTimestamp;
-    private boolean showGeneratedBy;    // temporary: for compatibility
-    private boolean showMetaCreated;    // temporary: for compatibility
-    private boolean useModuleDirectories;
-    private DocFile mainStylesheetFile;
-    private List<DocFile> additionalStylesheetFiles = Collections.emptyList();
+    private DocPath mainStylesheet;
+    private List<DocPath> additionalStylesheets = Collections.emptyList();
     private boolean index;
     private Script mainBodyScript;
     private final List<Script> scripts;
@@ -79,11 +77,9 @@ public class Head {
      * recording the time the file was created.
      * The doclet version should also be provided for recording in the file.
      * @param path the path for the file that will include this HEAD element
-     * @param htmlVersion the HTML version
      * @param docletVersion a string identifying the doclet version
      */
-    public Head(DocPath path, HtmlVersion htmlVersion, String docletVersion) {
-        this.htmlVersion = htmlVersion;
+    public Head(DocPath path, String docletVersion) {
         this.docletVersion = docletVersion;
         pathToRoot = path.parent().invert();
         keywords = new ArrayList<>();
@@ -115,6 +111,22 @@ public class Head {
         return this;
     }
 
+    /**
+     * Sets the content for the description META element.
+     */
+    public Head setDescription(String description) {
+        this.description = description;
+        return this;
+    }
+
+    /**
+     * Sets the content for the generator META element.
+     */
+    public Head setGenerator(String generator) {
+        this.generator = generator;
+        return this;
+    }
+
     /**
      * Adds a list of keywords to appear in META [@code keywords} elements.
      *
@@ -140,50 +152,21 @@ public class Head {
     // no 'Generated by javadoc' comment will be added.
     public Head setTimestamp(boolean timestamp) {
         showTimestamp = timestamp;
-        showGeneratedBy = true;
-        showMetaCreated = timestamp;
-        return this;
-    }
-
-    /**
-     * Sets whether or not timestamps should be recorded in the HEAD element.
-     * The timestamp will be recorded in a comment, and possibly in an appropriate META
-     * element, depending on the HTML version specified when this object was created.
-     *
-     * @param timestamp true if timestamps should be be added.
-     * @param metaCreated  true if a META element should be added containing the timestamp
-     * @return this object
-     */
-    // This method is for temporary compatibility. In time, all clients should use
-    // {@code setTimestamp(boolean)}.
-    public Head setTimestamp(boolean timestamp, boolean metaCreated) {
-        showTimestamp = timestamp;
-        showGeneratedBy = true;
-        showMetaCreated = metaCreated;
         return this;
     }
 
     /**
      * Sets the main and any additional stylesheets to be listed in the HEAD element.
+     * The paths for the stylesheets must be relative to the root of the generated
+     * documentation hierarchy.
      *
      * @param main the main stylesheet, or null to use the default
      * @param additional a list of any additional stylesheets to be included
      * @return  this object
      */
-    public Head setStylesheets(DocFile main, List<DocFile> additional) {
-        this.mainStylesheetFile = main;
-        this.additionalStylesheetFiles = additional;
-        return this;
-    }
-
-    /**
-     * Sets whether the module directories should be used. This is used to set the JavaScript variable.
-     *
-     * @param useModuleDirectories true if the module directories should be used
-     * @return  this object
-     */
-    public Head setUseModuleDirectories(boolean useModuleDirectories) {
-        this.useModuleDirectories = useModuleDirectories;
+    public Head setStylesheets(DocPath main, List<DocPath> additional) {
+        this.mainStylesheet = main;
+        this.additionalStylesheets = additional;
         return this;
     }
 
@@ -258,39 +241,40 @@ public class Head {
         Date now = showTimestamp ? calendar.getTime() : null;
 
         HtmlTree tree = new HtmlTree(HtmlTag.HEAD);
-        if (showGeneratedBy) {
-            tree.addContent(getGeneratedBy(showTimestamp, now));
-        }
-        tree.addContent(HtmlTree.TITLE(title));
+        tree.add(getGeneratedBy(showTimestamp, now));
+        tree.add(HtmlTree.TITLE(title));
 
         if (charset != null) { // compatibility; should this be allowed?
-            tree.addContent(HtmlTree.META("Content-Type", "text/html", charset));
+            tree.add(HtmlTree.META("Content-Type", "text/html", charset));
         }
 
-        if (showMetaCreated) {
+        if (showTimestamp) {
             SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
-            tree.addContent(HtmlTree.META(
-                    (htmlVersion == HtmlVersion.HTML5) ? "dc.created" : "date",
-                    dateFormat.format(now)));
+            tree.add(HtmlTree.META("dc.created", dateFormat.format(now)));
+        }
+
+        if (description != null) {
+            tree.add(HtmlTree.META("description", description));
+        }
+
+        if (generator != null) {
+            tree.add(HtmlTree.META("generator", generator));
         }
 
         for (String k : keywords) {
-            tree.addContent(HtmlTree.META("keywords", k));
-        }
-
-        for (Content c : extraContent) {
-            tree.addContent(c);
+            tree.add(HtmlTree.META("keywords", k));
         }
 
         if (canonicalLink != null) {
             HtmlTree link = new HtmlTree(HtmlTag.LINK);
-            link.addAttr(HtmlAttr.REL, "canonical");
-            link.addAttr(HtmlAttr.HREF, canonicalLink.getPath());
-            tree.addContent(link);
+            link.put(HtmlAttr.REL, "canonical");
+            link.put(HtmlAttr.HREF, canonicalLink.getPath());
+            tree.add(link);
         }
 
         addStylesheets(tree);
         addScripts(tree);
+        extraContent.forEach(tree::add);
 
         return tree;
     }
@@ -304,33 +288,28 @@ public class Head {
     }
 
     private void addStylesheets(HtmlTree tree) {
-        DocPath mainStylesheet;
-        if (mainStylesheetFile == null) {
+        if (mainStylesheet == null) {
             mainStylesheet = DocPaths.STYLESHEET;
-        } else {
-            mainStylesheet = DocPath.create(mainStylesheetFile.getName());
         }
         addStylesheet(tree, mainStylesheet);
 
-        for (DocFile file : additionalStylesheetFiles) {
-            addStylesheet(tree, DocPath.create(file.getName()));
+        for (DocPath path : additionalStylesheets) {
+            addStylesheet(tree, path);
         }
 
         if (index) {
-            // The order of the addStylesheet(...) calls is important
-            addStylesheet(tree, DocPaths.JQUERY_FILES.resolve(DocPaths.JQUERY_UI_CSS));
-            addStylesheet(tree, DocPaths.JQUERY_OVERRIDES_CSS);
+            addStylesheet(tree, DocPaths.JQUERY_FILES.resolve(DocPaths.JQUERY_STYLESHEET_FILE));
         }
     }
 
     private void addStylesheet(HtmlTree tree, DocPath stylesheet) {
-        tree.addContent(HtmlTree.LINK("stylesheet", "text/css",
+        tree.add(HtmlTree.LINK("stylesheet", "text/css",
                 pathToRoot.resolve(stylesheet).getPath(), "Style"));
     }
 
     private void addScripts(HtmlTree tree) {
         if (addDefaultScript) {
-            tree.addContent(HtmlTree.SCRIPT(pathToRoot.resolve(DocPaths.JAVASCRIPT).getPath()));
+            tree.add(HtmlTree.SCRIPT(pathToRoot.resolve(DocPaths.JAVASCRIPT).getPath()));
         }
         if (index) {
             if (pathToRoot != null && mainBodyScript != null) {
@@ -338,24 +317,23 @@ public class Head {
                 mainBodyScript.append("var pathtoroot = ")
                         .appendStringLiteral(ptrPath + "/")
                         .append(";\n")
-                        .append("var useModuleDirectories = " + useModuleDirectories + ";\n")
                         .append("loadScripts(document, \'script\');");
             }
             addJQueryFile(tree, DocPaths.JSZIP_MIN);
             addJQueryFile(tree, DocPaths.JSZIPUTILS_MIN);
-            tree.addContent(new RawHtml("<!--[if IE]>"));
+            tree.add(new RawHtml("<!--[if IE]>"));
             addJQueryFile(tree, DocPaths.JSZIPUTILS_IE_MIN);
-            tree.addContent(new RawHtml("<![endif]-->"));
+            tree.add(new RawHtml("<![endif]-->"));
+            addJQueryFile(tree, DocPaths.JQUERY_JS_3_4);
             addJQueryFile(tree, DocPaths.JQUERY_JS);
-            addJQueryFile(tree, DocPaths.JQUERY_UI_JS);
         }
         for (Script script : scripts) {
-            tree.addContent(script.asContent());
+            tree.add(script.asContent());
         }
     }
 
     private void addJQueryFile(HtmlTree tree, DocPath filePath) {
         DocPath jqueryFile = pathToRoot.resolve(DocPaths.JQUERY_FILES.resolve(filePath));
-        tree.addContent(HtmlTree.SCRIPT(jqueryFile.getPath()));
+        tree.add(HtmlTree.SCRIPT(jqueryFile.getPath()));
     }
 }
