@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -41,19 +41,24 @@ public enum HtmlStyle {
     allClassesContainer,
     allPackagesContainer,
     altColor,
+    annotations,
+    arguments,
     bar,
     block,
     blockList,
-    blockListLast,
     bottomNav,
     circle,
     classUseContainer,
+    classUses,
     colConstructorName,
     colDeprecatedItemName,
     colFirst,
     colLast,
     colSecond,
     constantsSummary,
+    constructorDetails,
+    constructorSummary,
+    constantDetails,
     constantValuesContainer,
     contentContainer,
     deprecatedLabel,
@@ -63,43 +68,66 @@ public enum HtmlStyle {
     description,
     descfrmTypeLabel,
     details,
+    detail,
     docSummary,
     emphasizedPhrase,
+    exceptions,
     externalLink,
+    fieldDetails,
+    fieldSummary,
     fixedNav,
     header,
+    helpSection,
+    hierarchy,
     horizontal,
     footer,
     implementationLabel,
     indexContainer,
     indexNav,
     inheritance,
+    inheritedList,
     interfaceName,
     leftContainer,
     leftTop,
     leftBottom,
     legalCopy,
     mainContainer,
+    memberDetails,
+    memberName,
     memberNameLabel,
     memberNameLink,
+    memberSignature,
     memberSummary,
+    methodDetails,
     methodSignature,
+    methodSummary,
+    modifiers,
+    moduleDescription,
     moduleLabelInPackage,
     moduleLabelInType,
+    moduleTags,
+    modulesSummary,
     nameValue,
     navBarCell1Rev,
     navList,
     navListSearch,
     navPadding,
+    nestedClassSummary,
     overrideSpecifyLabel,
     overviewSummary,
+    packages,
+    packageDescription,
     packageHierarchyLabel,
     packageLabelInType,
     packagesSummary,
+    packageUses,
     paramLabel,
+    propertyDetails,
+    propertySummary,
     providesSummary,
     requiresSummary,
     returnLabel,
+    returnType,
     rightContainer,
     rightIframe,
     rowColor,
@@ -107,6 +135,9 @@ public enum HtmlStyle {
     searchTagResult,
     seeLabel,
     serializedFormContainer,
+    serializedPackageContainer,
+    serializedClassDetails,
+    servicesSummary,
     simpleTagLabel,
     skipNav,
     sourceContainer,
@@ -122,6 +153,8 @@ public enum HtmlStyle {
     topNav,
     typeNameLabel,
     typeNameLink,
+    typeParameters,
+    typeParametersLong,
     typeSummary,
     useSummary,
     usesSummary
