@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -28,7 +28,6 @@ package jdk.javadoc.internal.doclets.toolkit;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
-import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -113,7 +112,7 @@ public class WorkArounds {
     }
 
     // TODO: fix this up correctly
-    public void initDocLint(Collection<String> opts, Collection<String> customTagNames, String htmlVersion) {
+    public void initDocLint(Collection<String> opts, Collection<String> customTagNames) {
         ArrayList<String> doclintOpts = new ArrayList<>();
         boolean msgOptionSeen = false;
 
@@ -138,12 +137,10 @@ public class WorkArounds {
             sep = DocLint.SEPARATOR;
         }
         doclintOpts.add(DocLint.XCUSTOM_TAGS_PREFIX + customTags.toString());
-        doclintOpts.add(DocLint.XHTML_VERSION_PREFIX + htmlVersion);
+        doclintOpts.add(DocLint.XHTML_VERSION_PREFIX + "html5");
 
         JavacTask t = BasicJavacTask.instance(toolEnv.context);
         doclint = new DocLint();
-        // standard doclet normally generates H1, H2
-        doclintOpts.add(DocLint.XIMPLICIT_HEADERS + "2");
         doclint.init(t, doclintOpts.toArray(new String[doclintOpts.size()]), false);
     }
 
@@ -200,10 +197,15 @@ public class WorkArounds {
 
     // TODO: needs to ported to jx.l.m.
     public TypeElement searchClass(TypeElement klass, String className) {
-        // search by qualified name first
-        TypeElement te = configuration.docEnv.getElementUtils().getTypeElement(className);
-        if (te != null) {
-            return te;
+        TypeElement te;
+
+        // search by qualified name in current module first
+        ModuleElement me = utils.containingModule(klass);
+        if (me != null) {
+            te = configuration.docEnv.getElementUtils().getTypeElement(me, className);
+            if (te != null) {
+                return te;
+            }
         }
 
         // search inner classes
@@ -251,6 +253,12 @@ public class WorkArounds {
             }
         }
 
+        // finally, search by qualified name in all modules
+        te = configuration.docEnv.getElementUtils().getTypeElement(className);
+        if (te != null) {
+            return te;
+        }
+
         return null; // not found
     }
 
