@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -125,9 +125,9 @@ public class FieldBuilder extends AbstractMemberBuilder {
             return;
         }
         if (!fields.isEmpty()) {
-            Content fieldDetailsTree = writer.getFieldDetailsTreeHeader(typeElement, memberDetailsTree);
+            Content fieldDetailsTreeHeader = writer.getFieldDetailsTreeHeader(typeElement, memberDetailsTree);
+            Content fieldDetailsTree = writer.getMemberTreeHeader();
 
-            Element lastElement = fields.get(fields.size() - 1);
             for (Element element : fields) {
                 currentElement = (VariableElement)element;
                 Content fieldDocTree = writer.getFieldDocTreeHeader(currentElement, fieldDetailsTree);
@@ -137,11 +137,10 @@ public class FieldBuilder extends AbstractMemberBuilder {
                 buildFieldComments(fieldDocTree);
                 buildTagInfo(fieldDocTree);
 
-                fieldDetailsTree.addContent(writer.getFieldDoc(
-                        fieldDocTree, currentElement == lastElement));
+                fieldDetailsTree.add(writer.getFieldDoc(fieldDocTree));
             }
-            memberDetailsTree.addContent(
-                    writer.getFieldDetails(fieldDetailsTree));
+            memberDetailsTree.add(
+                    writer.getFieldDetails(fieldDetailsTreeHeader, fieldDetailsTree));
         }
     }
 
@@ -151,7 +150,7 @@ public class FieldBuilder extends AbstractMemberBuilder {
      * @param fieldDocTree the content tree to which the documentation will be added
      */
     protected void buildSignature(Content fieldDocTree) {
-        fieldDocTree.addContent(writer.getSignature(currentElement));
+        fieldDocTree.add(writer.getSignature(currentElement));
     }
 
     /**
