@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -358,13 +358,13 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
             //add "[GS]ets the value of the property PROPERTY_NAME."
             if (isSetter) {
                 String text = MessageFormat.format(
-                        configuration.getText("doclet.PropertySetterWithName"),
+                        resources.getText("doclet.PropertySetterWithName"),
                         utils.propertyName((ExecutableElement)member));
                 fullBody.addAll(cmtutils.makeFirstSentenceTree(text));
             }
             if (isGetter) {
                 String text = MessageFormat.format(
-                        configuration.getText("doclet.PropertyGetterWithName"),
+                        resources.getText("doclet.PropertyGetterWithName"),
                         utils.propertyName((ExecutableElement) member));
                 fullBody.addAll(cmtutils.makeFirstSentenceTree(text));
             }
@@ -464,8 +464,8 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
                 Content inheritedTree = writer.getInheritedSummaryHeader(inheritedClass);
                 Content linksTree = writer.getInheritedSummaryLinksTree();
                 addSummaryFootNote(inheritedClass, inheritedMembers, linksTree, writer);
-                inheritedTree.addContent(linksTree);
-                summaryTreeList.add(writer.getMemberTree(inheritedTree));
+                inheritedTree.add(linksTree);
+                summaryTreeList.add(inheritedTree);
             }
         }
     }
@@ -497,7 +497,7 @@ public abstract class MemberSummaryBuilder extends AbstractMemberBuilder {
             buildInheritedSummary(writer, kind, summaryTreeList);
         if (!summaryTreeList.isEmpty()) {
             Content memberTree = writer.getMemberSummaryHeader(typeElement, memberSummaryTree);
-            summaryTreeList.stream().forEach(memberTree::addContent);
+            summaryTreeList.stream().forEach(memberTree::add);
             writer.addMemberTree(memberSummaryTree, memberTree);
         }
     }
