@@ -39,37 +39,22 @@ import javax.lang.model.element.TypeElement;
  *
  */
 public class DocPaths {
-    private final boolean useModuleDirectories;
     private final String moduleSeparator;
     private final Utils utils;
 
-    public DocPaths(Utils utils, boolean useModuleDirectories) {
+    public DocPaths(Utils utils) {
         this.utils = utils;
-        this.useModuleDirectories = useModuleDirectories;
-        moduleSeparator = useModuleDirectories ? "/module-" : "-";
+        moduleSeparator = "/module-";
     }
 
     public static final DocPath DOT_DOT = DocPath.create("..");
 
-    /** The name of the file for all classes, without using frames, when --no-frames is specified. */
-    public static final DocPath ALLCLASSES = DocPath.create("allclasses.html");
-
-    /** The name of the file for all classes, using frames. */
-    public static final DocPath ALLCLASSES_FRAME = DocPath.create("allclasses-frame.html");
-
     /** The name of the file for all classes index. */
     public static final DocPath ALLCLASSES_INDEX = DocPath.create("allclasses-index.html");
 
     /** The name of the file for all packages index. */
     public static final DocPath ALLPACKAGES_INDEX = DocPath.create("allpackages-index.html");
 
-    /** The name of the file for all classes, without using frames. */
-    public static final DocPath ALLCLASSES_NOFRAME = DocPath.create("allclasses-noframe.html");
-
-    public static DocPath AllClasses(boolean frames) {
-        return frames ? ALLCLASSES_NOFRAME : ALLCLASSES;
-    }
-
     /** The name of the sub-directory for storing class usage info. */
     public static final DocPath CLASS_USE = DocPath.create("class-use");
 
@@ -113,16 +98,13 @@ public class DocPaths {
     public static final DocPath JAVASCRIPT = DocPath.create("script.js");
 
     /** The name of the directory for the jQuery. */
-    public static final DocPath JQUERY_FILES = DocPath.create("jquery");
+    public static final DocPath JQUERY_FILES = DocPath.create("script-dir");
 
     /** The name of the default jQuery stylesheet file. */
     public static final DocPath JQUERY_STYLESHEET_FILE = DocPath.create("jquery-ui.css");
 
     /** The name of the default jQuery javascript file. */
-    public static final DocPath JQUERY_JS_3_3 = DocPath.create("jquery-3.3.1.js");
-
-    /** The name of jquery-migrate javascript file. */
-    public static final DocPath JQUERY_MIGRATE = DocPath.create("jquery-migrate-3.0.1.js");
+    public static final DocPath JQUERY_JS_3_4 = DocPath.create("jquery-3.4.1.js");
 
     /** The name of the default jQuery javascript file. */
     public static final DocPath JQUERY_JS = DocPath.create("jquery-ui.js");
@@ -163,22 +145,12 @@ public class DocPaths {
     /** The name of the module search index js file. */
     public static final DocPath MODULE_SEARCH_INDEX_JS = DocPath.create("module-search-index.js");
 
-    /** The name of the file for the overview frame. */
-    public static final DocPath OVERVIEW_FRAME = DocPath.create("overview-frame.html");
-
     /** The name of the file for the overview summary. */
     public static final DocPath OVERVIEW_SUMMARY = DocPath.create("overview-summary.html");
 
-    public static DocPath overviewSummary(boolean frames) {
-        return frames ? OVERVIEW_SUMMARY : INDEX;
-    }
-
     /** The name of the file for the overview tree. */
     public static final DocPath OVERVIEW_TREE = DocPath.create("overview-tree.html");
 
-    /** The name of the file for the package frame. */
-    public static final DocPath PACKAGE_FRAME = DocPath.create("package-frame.html");
-
     /** The name of the file for the package list. This is to support the legacy mode. */
     public static final DocPath PACKAGE_LIST = DocPath.create("package-list");
 
@@ -262,12 +234,8 @@ public class DocPaths {
         }
 
         DocPath pkgPath = DocPath.create(pkgElement.getQualifiedName().toString().replace('.', '/'));
-        if (useModuleDirectories) {
-            ModuleElement mdle = (ModuleElement) pkgElement.getEnclosingElement();
-            return forModule(mdle).resolve(pkgPath);
-        } else {
-            return pkgPath;
-        }
+        ModuleElement mdle = (ModuleElement) pkgElement.getEnclosingElement();
+        return forModule(mdle).resolve(pkgPath);
     }
 
     /**
@@ -305,15 +273,6 @@ public class DocPaths {
         return createModulePath(mdle, "doc-files");
     }
 
-    /**
-     * The path for the file for a module's frame page.
-     * @param mdle the module
-     * @return the path
-     */
-    public DocPath moduleFrame(ModuleElement mdle) {
-        return createModulePath(mdle, "frame.html");
-    }
-
     /**
      * The path for the file for a module's summary page.
      * @param mdle the module
@@ -332,15 +291,6 @@ public class DocPaths {
         return createModulePath(mdleName, "summary.html");
     }
 
-    /**
-     * The path for the file for a module's type frame page.
-     * @param mdle the module
-     * @return the path
-     */
-    public DocPath moduleTypeFrame(ModuleElement mdle) {
-        return createModulePath(mdle, "type-frame.html");
-    }
-
     private DocPath createModulePath(ModuleElement mdle, String path) {
         return DocPath.create(mdle.getQualifiedName() + moduleSeparator + path);
     }
@@ -349,9 +299,6 @@ public class DocPaths {
         return DocPath.create(moduleName + moduleSeparator + path);
     }
 
-    /** The name of the file for the module overview frame. */
-    public static final DocPath MODULE_OVERVIEW_FRAME = DocPath.create("module-overview-frame.html");
-
     /** The name of the sub-package from which resources are read. */
     public static final DocPath RESOURCES = DocPath.create("resources");
 
