@@ -25,8 +25,6 @@
 
 package jdk.jfr.internal.management;
 
-import java.io.IOException;
-import java.nio.file.Paths;
 import java.time.Duration;
 import java.util.ArrayList;
 import java.util.Collections;
@@ -100,12 +98,4 @@ public final class ManagementSupport {
         WriteableUserPath wup = pr.getDestination();
         return wup == null ? null : wup.getOriginalText();
     }
-
-    public static void checkSetDestination(Recording recording, String destination) throws IOException{
-        PlatformRecording pr = PrivateAccess.getInstance().getPlatformRecording(recording);
-        if(destination != null){
-            WriteableUserPath wup = new WriteableUserPath(Paths.get(destination));
-            pr.checkSetDestination(wup);
-        }
-    }
 }
