@@ -31,8 +31,12 @@ package jdk.nashorn.api.tree;
  * type serves as a super interface for {@link BreakTree} and
  * {@link ContinueTree}.
  *
+ * @deprecated Nashorn JavaScript script engine and APIs, and the jjs tool
+ * are deprecated with the intent to remove them in a future release.
+ *
  * @since 9
  */
+@Deprecated(since="11", forRemoval=true)
 public interface GotoTree extends StatementTree {
     /**
      * Label associated with this goto statement. This is null
