@@ -39,8 +39,12 @@ package jdk.nashorn.api.tree;
  *       <em>elseStatement</em>
  * </pre>
  *
+ * @deprecated Nashorn JavaScript script engine and APIs, and the jjs tool
+ * are deprecated with the intent to remove them in a future release.
+ *
  * @since 9
  */
+@Deprecated(since="11", forRemoval=true)
 public interface IfTree extends StatementTree {
     /**
      * Returns the condition expression of this 'if' statement.
