@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -68,14 +68,14 @@ class ZipFileStore extends FileStore {
 
     @Override
     public boolean supportsFileAttributeView(String name) {
-        return name.equals("basic") || name.equals("zip");
+        return "basic".equals(name) || "zip".equals(name);
     }
 
     @Override
     public <V extends FileStoreAttributeView> V getFileStoreAttributeView(Class<V> type) {
         if (type == null)
             throw new NullPointerException();
-        return (V)null;
+        return null;
     }
 
     @Override
@@ -108,7 +108,7 @@ class ZipFileStore extends FileStore {
         final FileStore fstore;
         final long size;
 
-        public ZipFileStoreAttributes(ZipFileStore fileStore)
+        ZipFileStoreAttributes(ZipFileStore fileStore)
             throws IOException
         {
             Path path = FileSystems.getDefault().getPath(fileStore.name());
@@ -116,17 +116,17 @@ class ZipFileStore extends FileStore {
             this.fstore = Files.getFileStore(path);
         }
 
-        public long totalSpace() {
+        long totalSpace() {
             return size;
         }
 
-        public long usableSpace() throws IOException {
+        long usableSpace() throws IOException {
             if (!fstore.isReadOnly())
                 return fstore.getUsableSpace();
             return 0;
         }
 
-        public long unallocatedSpace()  throws IOException {
+        long unallocatedSpace()  throws IOException {
             if (!fstore.isReadOnly())
                 return fstore.getUnallocatedSpace();
             return 0;
