@@ -24,8 +24,8 @@
  */
 
 module jdk.internal.vm.compiler {
-    // source file: file:///scratch/opt/mach5/mesos/work_dir/slaves/dd0bab7c-a530-4905-bb6b-daa7153eb2c1-S122/frameworks/1735e8a2-a1db-478c-8104-60c8b0af87dd-0196/executors/99f85cae-f7c3-4f44-ac5f-6a7e8717ec2c/runs/a85de1b3-d3dd-4e30-9975-ddd13487d7cc/workspace/open/src/jdk.internal.vm.compiler/share/classes/module-info.java
-    //              file:///scratch/opt/mach5/mesos/work_dir/slaves/dd0bab7c-a530-4905-bb6b-daa7153eb2c1-S122/frameworks/1735e8a2-a1db-478c-8104-60c8b0af87dd-0196/executors/99f85cae-f7c3-4f44-ac5f-6a7e8717ec2c/runs/a85de1b3-d3dd-4e30-9975-ddd13487d7cc/workspace/build/linux-x64/support/gensrc/jdk.internal.vm.compiler/module-info.java.extra
+    // source file: file:///scratch/opt/mach5/mesos/work_dir/slaves/fcf4c0c4-d73e-4321-860c-6613427db92b-S2589/frameworks/1735e8a2-a1db-478c-8104-60c8b0af87dd-0196/executors/7e0f51a1-6397-462e-a00e-f30e1df5387c/runs/2786f1af-1296-422d-8542-d335b44e3efc/workspace/open/src/jdk.internal.vm.compiler/share/classes/module-info.java
+    //              file:///scratch/opt/mach5/mesos/work_dir/slaves/fcf4c0c4-d73e-4321-860c-6613427db92b-S2589/frameworks/1735e8a2-a1db-478c-8104-60c8b0af87dd-0196/executors/7e0f51a1-6397-462e-a00e-f30e1df5387c/runs/2786f1af-1296-422d-8542-d335b44e3efc/workspace/build/linux-x64/support/gensrc/jdk.internal.vm.compiler/module-info.java.extra
     requires java.instrument;
     requires java.management;
     requires jdk.internal.vm.ci;
@@ -90,7 +90,8 @@ module jdk.internal.vm.compiler {
     provides jdk.vm.ci.services.JVMCIServiceLocator with org.graalvm.compiler.hotspot.HotSpotGraalJVMCIServiceLocator;
     provides org.graalvm.compiler.code.DisassemblerProvider with
         org.graalvm.compiler.code.HexCodeFileDisassemblerProvider,
-        org.graalvm.compiler.hotspot.meta.HotSpotDisassemblerProvider;
+        org.graalvm.compiler.hotspot.meta.HotSpotDisassemblerProvider,
+        org.graalvm.compiler.hotspot.meta.HotSpotObjdumpDisassemblerProvider;
     provides org.graalvm.compiler.core.match.MatchStatementSet with
         org.graalvm.compiler.core.aarch64.AArch64NodeMatchRules_MatchStatementSet,
         org.graalvm.compiler.core.amd64.AMD64NodeMatchRules_MatchStatementSet,
@@ -106,6 +107,8 @@ module jdk.internal.vm.compiler {
         org.graalvm.compiler.hotspot.sparc.SPARCHotSpotBackendFactory;
     provides org.graalvm.compiler.nodes.graphbuilderconf.NodeIntrinsicPluginFactory with
         org.graalvm.compiler.hotspot.PluginFactory_HotSpotBackend,
+        org.graalvm.compiler.hotspot.amd64.PluginFactory_AMD64X87MathIntrinsicNode,
+        org.graalvm.compiler.hotspot.amd64.PluginFactory_AMD64X87MathSnippets,
         org.graalvm.compiler.hotspot.nodes.PluginFactory_AcquiredCASLockNode,
         org.graalvm.compiler.hotspot.nodes.PluginFactory_AllocaNode,
         org.graalvm.compiler.hotspot.nodes.PluginFactory_BeginLockScopeNode,
@@ -135,6 +138,9 @@ module jdk.internal.vm.compiler {
         org.graalvm.compiler.hotspot.replacements.PluginFactory_CRC32Substitutions,
         org.graalvm.compiler.hotspot.replacements.PluginFactory_CipherBlockChainingSubstitutions,
         org.graalvm.compiler.hotspot.replacements.PluginFactory_ClassGetHubNode,
+        org.graalvm.compiler.hotspot.replacements.PluginFactory_CounterModeSubstitutions,
+        org.graalvm.compiler.hotspot.replacements.PluginFactory_DigestBaseSubstitutions,
+        org.graalvm.compiler.hotspot.replacements.PluginFactory_HotSpotClassSubstitutions,
         org.graalvm.compiler.hotspot.replacements.PluginFactory_HotSpotReplacementsUtil,
         org.graalvm.compiler.hotspot.replacements.PluginFactory_HubGetClassNode,
         org.graalvm.compiler.hotspot.replacements.PluginFactory_IdentityHashCodeNode,
@@ -142,6 +148,8 @@ module jdk.internal.vm.compiler {
         org.graalvm.compiler.hotspot.replacements.PluginFactory_NewObjectSnippets,
         org.graalvm.compiler.hotspot.replacements.PluginFactory_ObjectSubstitutions,
         org.graalvm.compiler.hotspot.replacements.PluginFactory_SHA2Substitutions,
+        org.graalvm.compiler.hotspot.replacements.PluginFactory_SHA5Substitutions,
+        org.graalvm.compiler.hotspot.replacements.PluginFactory_SHASubstitutions,
         org.graalvm.compiler.hotspot.replacements.PluginFactory_ThreadSubstitutions,
         org.graalvm.compiler.hotspot.replacements.PluginFactory_WriteBarrierSnippets,
         org.graalvm.compiler.hotspot.replacements.profiling.PluginFactory_ProbabilisticProfileSnippets,
@@ -153,6 +161,7 @@ module jdk.internal.vm.compiler {
         org.graalvm.compiler.hotspot.stubs.PluginFactory_UnwindExceptionToCallerStub,
         org.graalvm.compiler.nodes.PluginFactory_BreakpointNode,
         org.graalvm.compiler.nodes.PluginFactory_ComputeObjectAddressNode,
+        org.graalvm.compiler.nodes.PluginFactory_ConstantNode,
         org.graalvm.compiler.nodes.PluginFactory_DeoptimizeNode,
         org.graalvm.compiler.nodes.PluginFactory_GetObjectAddressNode,
         org.graalvm.compiler.nodes.PluginFactory_PauseNode,
@@ -163,6 +172,7 @@ module jdk.internal.vm.compiler {
         org.graalvm.compiler.nodes.debug.PluginFactory_DynamicCounterNode,
         org.graalvm.compiler.nodes.extended.PluginFactory_BranchProbabilityNode,
         org.graalvm.compiler.nodes.extended.PluginFactory_FixedValueAnchorNode,
+        org.graalvm.compiler.nodes.extended.PluginFactory_GuardedUnsafeLoadNode,
         org.graalvm.compiler.nodes.extended.PluginFactory_MembarNode,
         org.graalvm.compiler.nodes.extended.PluginFactory_NullCheckNode,
         org.graalvm.compiler.nodes.extended.PluginFactory_RawLoadNode,
@@ -184,15 +194,13 @@ module jdk.internal.vm.compiler {
         org.graalvm.compiler.replacements.aarch64.PluginFactory_AArch64IntegerArithmeticSnippets,
         org.graalvm.compiler.replacements.amd64.PluginFactory_AMD64ArrayIndexOf,
         org.graalvm.compiler.replacements.amd64.PluginFactory_AMD64ArrayIndexOfNode,
-        org.graalvm.compiler.replacements.amd64.PluginFactory_AMD64MathSubstitutions,
         org.graalvm.compiler.replacements.amd64.PluginFactory_AMD64StringLatin1InflateNode,
         org.graalvm.compiler.replacements.amd64.PluginFactory_AMD64StringLatin1Substitutions,
         org.graalvm.compiler.replacements.amd64.PluginFactory_AMD64StringSubstitutions,
         org.graalvm.compiler.replacements.amd64.PluginFactory_AMD64StringUTF16CompressNode,
         org.graalvm.compiler.replacements.amd64.PluginFactory_AMD64StringUTF16Substitutions,
         org.graalvm.compiler.replacements.arraycopy.PluginFactory_ArrayCopyCallNode,
-        org.graalvm.compiler.replacements.arraycopy.PluginFactory_ArrayCopySnippets,
-        org.graalvm.compiler.replacements.arraycopy.PluginFactory_ArrayCopyWithSlowPathNode,
+        org.graalvm.compiler.replacements.arraycopy.PluginFactory_ArrayCopyWithDelayedLoweringNode,
         org.graalvm.compiler.replacements.arraycopy.PluginFactory_CheckcastArrayCopyCallNode,
         org.graalvm.compiler.replacements.arraycopy.PluginFactory_GenericArrayCopyCallNode,
         org.graalvm.compiler.replacements.nodes.PluginFactory_ArrayCompareToNode,
@@ -205,7 +213,8 @@ module jdk.internal.vm.compiler {
         org.graalvm.compiler.replacements.nodes.PluginFactory_CStringConstant,
         org.graalvm.compiler.replacements.nodes.PluginFactory_DirectStoreNode,
         org.graalvm.compiler.replacements.nodes.PluginFactory_ExplodeLoopNode,
-        org.graalvm.compiler.replacements.nodes.PluginFactory_UnaryMathIntrinsicNode;
+        org.graalvm.compiler.replacements.nodes.PluginFactory_UnaryMathIntrinsicNode,
+        org.graalvm.compiler.replacements.nodes.PluginFactory_ZeroMemoryNode;
     provides org.graalvm.compiler.options.OptionDescriptors with
         org.graalvm.compiler.core.GraalCompilerOptions_OptionDescriptors,
         org.graalvm.compiler.core.common.GraalOptions_OptionDescriptors,
