@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -29,14 +29,14 @@
  * <em>{@index pack200 pack200 tool}</em> and
  * <em>{@index unpack200 unpack200 tool}</em> tools.
  *
- * <dl style="font-family:'DejaVu Sans', Arial, Helvetica, sans serif">
- * <dt class="simpleTagLabel">Tool Guides:
- * <dd>{@extLink pack200_tool_reference pack200},
- *     {@extLink unpack200_tool_reference unpack200}
- * </dl>
+ * @toolGuide pack200
+ * @toolGuide unpack200
  *
  * @moduleGraph
+ * @deprecated This module is deprecated, and is planned for removal in a
+ *             future release.
  * @since 9
  */
+@Deprecated(since="11", forRemoval=true)
 module jdk.pack {
 }
