@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -36,11 +36,11 @@
  *      {@link java.nio.file.FileSystems#newFileSystem
  *      FileSystems.newFileSystem(URI.create("jrt:/"))}.
  *      </dd>
- * <dt class="simpleTagLabel" style="font-family:'DejaVu Sans', Arial, Helvetica, sans serif">Tool Guides:</dt>
- * <dd style="font-family:'DejaVu Sans', Arial, Helvetica, sans serif"> {@extLink java_tool_reference java launcher},
- *      {@extLink keytool_tool_reference keytool}</dd>
  * </dl>
  *
+ * @toolGuide java java launcher
+ * @toolGuide keytool
+ *
  * @provides java.nio.file.spi.FileSystemProvider
  *
  * @uses java.lang.System.LoggerFinder
@@ -75,8 +75,8 @@
  * @since 9
  */
 module java.base {
-    // source file: file:///scratch/opt/mach5/mesos/work_dir/slaves/dd0bab7c-a530-4905-bb6b-daa7153eb2c1-S122/frameworks/1735e8a2-a1db-478c-8104-60c8b0af87dd-0196/executors/99f85cae-f7c3-4f44-ac5f-6a7e8717ec2c/runs/a85de1b3-d3dd-4e30-9975-ddd13487d7cc/workspace/open/src/java.base/share/classes/module-info.java
-    //              file:///scratch/opt/mach5/mesos/work_dir/slaves/dd0bab7c-a530-4905-bb6b-daa7153eb2c1-S122/frameworks/1735e8a2-a1db-478c-8104-60c8b0af87dd-0196/executors/99f85cae-f7c3-4f44-ac5f-6a7e8717ec2c/runs/a85de1b3-d3dd-4e30-9975-ddd13487d7cc/workspace/open/src/java.base/unix/classes/module-info.java.extra
+    // source file: file:///scratch/opt/mach5/mesos/work_dir/slaves/fcf4c0c4-d73e-4321-860c-6613427db92b-S2589/frameworks/1735e8a2-a1db-478c-8104-60c8b0af87dd-0196/executors/7e0f51a1-6397-462e-a00e-f30e1df5387c/runs/2786f1af-1296-422d-8542-d335b44e3efc/workspace/open/src/java.base/share/classes/module-info.java
+    //              file:///scratch/opt/mach5/mesos/work_dir/slaves/fcf4c0c4-d73e-4321-860c-6613427db92b-S2589/frameworks/1735e8a2-a1db-478c-8104-60c8b0af87dd-0196/executors/7e0f51a1-6397-462e-a00e-f30e1df5387c/runs/2786f1af-1296-422d-8542-d335b44e3efc/workspace/open/src/java.base/unix/classes/module-info.java.extra
     exports java.io;
     exports java.lang;
     exports java.lang.annotation;
@@ -200,6 +200,7 @@ module java.base {
         java.sql,
         java.sql.rowset,
         jdk.dynalink,
+        jdk.internal.vm.ci,
         jdk.scripting.nashorn,
         jdk.unsupported;
     exports jdk.internal.util.jar to jdk.jartool;
