@@ -149,13 +149,6 @@ public class FloatStamp extends PrimitiveStamp {
         return nonNaN;
     }
 
-    /**
-     * Returns true if NaN is included in the value described by this stamp.
-     */
-    public boolean canBeNaN() {
-        return !nonNaN;
-    }
-
     /**
      * Returns true if this stamp represents the NaN value.
      */
