@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -43,6 +43,8 @@ import org.graalvm.compiler.nodes.graphbuilderconf.InvocationPlugins;
 import org.graalvm.compiler.nodes.graphbuilderconf.InvocationPlugins.Registration;
 import org.graalvm.compiler.nodes.spi.LIRLowerable;
 import org.graalvm.compiler.nodes.spi.NodeLIRBuilderTool;
+import org.graalvm.compiler.phases.OptimisticOptimizations;
+import org.graalvm.compiler.phases.tiers.HighTierContext;
 import org.junit.Test;
 
 import jdk.vm.ci.meta.JavaKind;
@@ -411,6 +413,7 @@ public class CountedLoopTest extends GraalCompilerTest {
         public void rewrite(LoopsData loops) {
             InductionVariable inductionVariable = loops.getInductionVariable(iv);
             assert inductionVariable != null;
+            assertTrue(inductionVariable.getLoop().isCounted(), "must be counted");
             ValueNode node = null;
             if (staticCheck != null) {
                 assert staticProperty != null;
@@ -490,6 +493,12 @@ public class CountedLoopTest extends GraalCompilerTest {
         return true;
     }
 
+    @Override
+    protected HighTierContext getDefaultHighTierContext() {
+        // Don't convert unreached paths into Guard
+        return new HighTierContext(getProviders(), getDefaultGraphBuilderSuite(), OptimisticOptimizations.NONE);
+    }
+
     private Object[] argsToBind;
 
     @Override
