@@ -124,7 +124,6 @@ import org.graalvm.compiler.phases.tiers.TargetProvider;
 import org.graalvm.compiler.phases.util.Providers;
 import org.graalvm.compiler.printer.GraalDebugHandlersFactory;
 import org.graalvm.compiler.runtime.RuntimeProvider;
-import org.graalvm.compiler.serviceprovider.JavaVersionUtil;
 import org.graalvm.compiler.test.AddExports;
 import org.graalvm.compiler.test.GraalTest;
 import org.graalvm.compiler.test.JLModule;
@@ -151,9 +150,9 @@ import jdk.vm.ci.meta.ResolvedJavaType;
 import jdk.vm.ci.meta.SpeculationLog;
 
 /**
- * Base class for compiler unit tests.
+ * Base class for Graal compiler unit tests.
  * <p>
- * White box tests for compiler transformations use this pattern:
+ * White box tests for Graal compiler transformations use this pattern:
  * <ol>
  * <li>Create a graph by {@linkplain #parseEager parsing} a method.</li>
  * <li>Manually modify the graph (e.g. replace a parameter node with a constant).</li>
@@ -195,7 +194,7 @@ public abstract class GraalCompilerTest extends GraalTest {
      * as of JDK 9.
      */
     protected final void exportPackage(Class<?> moduleMember, String packageName) {
-        if (JavaVersionUtil.JAVA_SPEC > 8) {
+        if (!Java8OrEarlier) {
             JLModule.exportPackageTo(moduleMember, packageName, getClass());
         }
     }
@@ -225,27 +224,27 @@ public abstract class GraalCompilerTest extends GraalTest {
      * Can be overridden by unit tests to verify properties of the graph.
      *
      * @param graph the graph at the end of HighTier
-     * @throws AssertionError if the verification fails
      */
-    protected void checkHighTierGraph(StructuredGraph graph) {
+    protected boolean checkHighTierGraph(StructuredGraph graph) {
+        return true;
     }
 
     /**
      * Can be overridden by unit tests to verify properties of the graph.
      *
      * @param graph the graph at the end of MidTier
-     * @throws AssertionError if the verification fails
      */
-    protected void checkMidTierGraph(StructuredGraph graph) {
+    protected boolean checkMidTierGraph(StructuredGraph graph) {
+        return true;
     }
 
     /**
      * Can be overridden by unit tests to verify properties of the graph.
      *
      * @param graph the graph at the end of LowTier
-     * @throws AssertionError if the verification fails
      */
-    protected void checkLowTierGraph(StructuredGraph graph) {
+    protected boolean checkLowTierGraph(StructuredGraph graph) {
+        return true;
     }
 
     protected static void breakpoint() {
@@ -289,7 +288,7 @@ public abstract class GraalCompilerTest extends GraalTest {
 
             @Override
             protected void run(StructuredGraph graph) {
-                checkHighTierGraph(graph);
+                assert checkHighTierGraph(graph) : "failed HighTier graph check";
             }
 
             @Override
@@ -306,7 +305,7 @@ public abstract class GraalCompilerTest extends GraalTest {
 
             @Override
             protected void run(StructuredGraph graph) {
-                checkMidTierGraph(graph);
+                assert checkMidTierGraph(graph) : "failed MidTier graph check";
             }
 
             @Override
@@ -323,7 +322,7 @@ public abstract class GraalCompilerTest extends GraalTest {
 
             @Override
             protected void run(StructuredGraph graph) {
-                checkLowTierGraph(graph);
+                assert checkLowTierGraph(graph) : "failed LowTier graph check";
             }
 
             @Override
@@ -1116,14 +1115,8 @@ public abstract class GraalCompilerTest extends GraalTest {
         return graph;
     }
 
-    @SuppressWarnings("try")
     protected void applyFrontEnd(StructuredGraph graph) {
-        DebugContext debug = graph.getDebug();
-        try (DebugContext.Scope s = debug.scope("FrontEnd", graph)) {
-            GraalCompiler.emitFrontEnd(getProviders(), getBackend(), graph, getDefaultGraphBuilderSuite(), getOptimisticOptimizations(), graph.getProfilingInfo(), createSuites(graph.getOptions()));
-        } catch (Throwable e) {
-            throw debug.handle(e);
-        }
+        GraalCompiler.emitFrontEnd(getProviders(), getBackend(), graph, getDefaultGraphBuilderSuite(), getOptimisticOptimizations(), graph.getProfilingInfo(), createSuites(graph.getOptions()));
     }
 
     protected StructuredGraph lastCompiledGraph;
@@ -1507,8 +1500,4 @@ public abstract class GraalCompilerTest extends GraalTest {
     protected boolean isArchitecture(String name) {
         return name.equals(backend.getTarget().arch.getName());
     }
-
-    protected CanonicalizerPhase createCanonicalizerPhase() {
-        return new CanonicalizerPhase();
-    }
 }
