@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -24,20 +24,23 @@
 
 package org.graalvm.compiler.hotspot.test;
 
-import static org.graalvm.compiler.hotspot.replacements.HotSpotReplacementsUtil.config;
 import static org.graalvm.compiler.hotspot.replacements.HotSpotReplacementsUtil.referentOffset;
 
 import java.lang.ref.WeakReference;
 
+import org.graalvm.compiler.api.replacements.Fold;
 import org.graalvm.compiler.debug.DebugContext;
 import org.graalvm.compiler.hotspot.GraalHotSpotVMConfig;
-import org.graalvm.compiler.hotspot.nodes.G1PostWriteBarrier;
-import org.graalvm.compiler.hotspot.nodes.G1PreWriteBarrier;
-import org.graalvm.compiler.hotspot.nodes.G1ReferentFieldReadBarrier;
-import org.graalvm.compiler.hotspot.nodes.SerialWriteBarrier;
+import org.graalvm.compiler.hotspot.GraalHotSpotVMConfigBase;
+import org.graalvm.compiler.hotspot.gc.g1.G1PostWriteBarrier;
+import org.graalvm.compiler.hotspot.gc.g1.G1PreWriteBarrier;
+import org.graalvm.compiler.hotspot.gc.g1.G1ReferentFieldReadBarrier;
+import org.graalvm.compiler.hotspot.gc.shared.SerialWriteBarrier;
 import org.graalvm.compiler.hotspot.phases.WriteBarrierAdditionPhase;
 import org.graalvm.compiler.nodes.StructuredGraph;
 import org.graalvm.compiler.nodes.StructuredGraph.AllowAssumptions;
+import org.graalvm.compiler.nodes.graphbuilderconf.InvocationPlugins;
+import org.graalvm.compiler.nodes.graphbuilderconf.NodeIntrinsicPluginFactory;
 import org.graalvm.compiler.nodes.memory.HeapAccess.BarrierType;
 import org.graalvm.compiler.nodes.memory.ReadNode;
 import org.graalvm.compiler.nodes.memory.WriteNode;
@@ -51,6 +54,9 @@ import org.graalvm.compiler.phases.common.inlining.InliningPhase;
 import org.graalvm.compiler.phases.common.inlining.policy.InlineEverythingPolicy;
 import org.graalvm.compiler.phases.tiers.HighTierContext;
 import org.graalvm.compiler.phases.tiers.MidTierContext;
+import org.graalvm.compiler.replacements.NodeIntrinsificationProvider;
+import org.graalvm.compiler.word.Word;
+import jdk.internal.vm.compiler.word.WordFactory;
 import org.junit.Assert;
 import org.junit.Test;
 
@@ -71,7 +77,6 @@ import sun.misc.Unsafe;
 public class WriteBarrierAdditionTest extends HotSpotGraalCompilerTest {
 
     private final GraalHotSpotVMConfig config = runtime().getVMConfig();
-    private static final long referentOffset = referentOffset();
 
     public static class Container {
 
@@ -167,8 +172,20 @@ public class WriteBarrierAdditionTest extends HotSpotGraalCompilerTest {
         testHelper("test5Snippet", config.useG1GC ? 1 : 0);
     }
 
+    @Override
+    protected void registerInvocationPlugins(InvocationPlugins invocationPlugins) {
+        NodeIntrinsicPluginFactory.InjectionProvider injection = new NodeIntrinsificationProvider(getMetaAccess(), getSnippetReflection(), getProviders().getForeignCalls(), null);
+        new PluginFactory_WriteBarrierAdditionTest().registerPlugins(invocationPlugins, injection);
+        super.registerInvocationPlugins(invocationPlugins);
+    }
+
+    @Fold
+    public static boolean useCompressedOops(@Fold.InjectedParameter GraalHotSpotVMConfig config) {
+        return config.useCompressedOops;
+    }
+
     public static Object test5Snippet() throws Exception {
-        return UNSAFE.getObject(wr, config(null).useCompressedOops ? 12L : 16L);
+        return UNSAFE.getObject(wr, useCompressedOops(GraalHotSpotVMConfigBase.INJECTED_VMCONFIG) ? 12L : 16L);
     }
 
     /**
@@ -177,7 +194,7 @@ public class WriteBarrierAdditionTest extends HotSpotGraalCompilerTest {
      */
     @Test
     public void test6() throws Exception {
-        test2("testUnsafeLoad", UNSAFE, wr, new Long(referentOffset), null);
+        test2("testUnsafeLoad", UNSAFE, wr, Long.valueOf(referentOffset(getMetaAccess())), null);
     }
 
     /**
@@ -186,7 +203,7 @@ public class WriteBarrierAdditionTest extends HotSpotGraalCompilerTest {
      */
     @Test
     public void test7() throws Exception {
-        test2("testUnsafeLoad", UNSAFE, con, new Long(referentOffset), null);
+        test2("testUnsafeLoad", UNSAFE, con, Long.valueOf(referentOffset(getMetaAccess())), null);
     }
 
     /**
@@ -196,7 +213,7 @@ public class WriteBarrierAdditionTest extends HotSpotGraalCompilerTest {
      */
     @Test
     public void test8() throws Exception {
-        test2("testUnsafeLoad", UNSAFE, wr, new Long(config.useCompressedOops ? 20 : 32), null);
+        test2("testUnsafeLoad", UNSAFE, wr, Long.valueOf(config.useCompressedOops ? 20 : 32), null);
     }
 
     /**
@@ -206,7 +223,7 @@ public class WriteBarrierAdditionTest extends HotSpotGraalCompilerTest {
      */
     @Test
     public void test10() throws Exception {
-        test2("testUnsafeLoad", UNSAFE, wr, new Long(config.useCompressedOops ? 6 : 8), new Integer(config.useCompressedOops ? 6 : 8));
+        test2("testUnsafeLoad", UNSAFE, wr, Long.valueOf(config.useCompressedOops ? 6 : 8), Integer.valueOf(config.useCompressedOops ? 6 : 8));
     }
 
     /**
@@ -216,7 +233,7 @@ public class WriteBarrierAdditionTest extends HotSpotGraalCompilerTest {
      */
     @Test
     public void test9() throws Exception {
-        test2("testUnsafeLoad", UNSAFE, wr, new Long(config.useCompressedOops ? 10 : 16), new Integer(config.useCompressedOops ? 10 : 16));
+        test2("testUnsafeLoad", UNSAFE, wr, Long.valueOf(config.useCompressedOops ? 10 : 16), Integer.valueOf(config.useCompressedOops ? 10 : 16));
     }
 
     static Object[] src = new Object[1];
@@ -240,6 +257,34 @@ public class WriteBarrierAdditionTest extends HotSpotGraalCompilerTest {
         test2("testArrayCopy", src, dst, dst.length);
     }
 
+    public static class WordContainer {
+        public Word word;
+    }
+
+    public static void testWordFieldSnippet() {
+        WordContainer wordContainer = new WordContainer();
+        wordContainer.word = WordFactory.signed(42);
+    }
+
+    @Test
+    public void testWordField() throws Exception {
+        testHelper("testWordFieldSnippet", 0);
+    }
+
+    public static Word[] testWordArraySnippet(int length) {
+        Word fortyTwo = WordFactory.signed(42);
+        Word[] words = new Word[length];
+        for (int i = 0; i < length; i++) {
+            words[i] = fortyTwo;
+        }
+        return words;
+    }
+
+    @Test
+    public void testWordArray() throws Exception {
+        testHelper("testWordArraySnippet", 0);
+    }
+
     public static Object testUnsafeLoad(Unsafe theUnsafe, Object a, Object b, Object c) throws Exception {
         final int offset = (c == null ? 0 : ((Integer) c).intValue());
         final long displacement = (b == null ? 0 : ((Long) b).longValue());
@@ -299,10 +344,11 @@ public class WriteBarrierAdditionTest extends HotSpotGraalCompilerTest {
                     Assert.assertTrue(read.getAddress() instanceof OffsetAddressNode);
                     JavaConstant constDisp = ((OffsetAddressNode) read.getAddress()).getOffset().asJavaConstant();
                     Assert.assertNotNull(constDisp);
-                    Assert.assertEquals(referentOffset, constDisp.asLong());
-                    Assert.assertTrue(config.useG1GC);
-                    Assert.assertEquals(BarrierType.PRECISE, read.getBarrierType());
-                    Assert.assertTrue(read.next() instanceof G1ReferentFieldReadBarrier);
+                    Assert.assertEquals(referentOffset(getMetaAccess()), constDisp.asLong());
+                    Assert.assertEquals(BarrierType.WEAK_FIELD, read.getBarrierType());
+                    if (config.useG1GC) {
+                        Assert.assertTrue(read.next() instanceof G1ReferentFieldReadBarrier);
+                    }
                 }
             }
         } catch (Throwable e) {
