@@ -179,7 +179,8 @@ public class GraalHotSpotVMConfig extends GraalHotSpotVMConfigBase {
         return universeCollectedHeap + collectedHeapTotalCollectionsOffset;
     }
 
-    public final boolean useDeferredInitBarriers = getFlag("ReduceInitialCardMarks", Boolean.class);
+    // Disable support for ReduceInitialCardMarks due to JDK-8225064.
+    public final boolean useDeferredInitBarriers = false;
 
     // Compressed Oops related values.
     public final boolean useCompressedOops = getFlag("UseCompressedOops", Boolean.class);
