@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -28,12 +28,8 @@ import static jdk.vm.ci.common.InitTimer.timer;
 import static jdk.vm.ci.hotspot.HotSpotJVMCIRuntime.runtime;
 import static org.graalvm.compiler.core.common.GraalOptions.GeneratePIC;
 import static org.graalvm.compiler.core.common.GraalOptions.HotSpotPrintInlining;
-import static org.graalvm.compiler.debug.DebugContext.DEFAULT_LOG_STREAM;
-import static org.graalvm.compiler.hotspot.HotSpotGraalRuntime.HotSpotGC.CMS;
-import static org.graalvm.compiler.hotspot.HotSpotGraalRuntime.HotSpotGC.G1;
-import static org.graalvm.compiler.hotspot.HotSpotGraalRuntime.HotSpotGC.Parallel;
-import static org.graalvm.compiler.hotspot.HotSpotGraalRuntime.HotSpotGC.Serial;
 
+import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.EnumMap;
 import java.util.List;
@@ -49,6 +45,7 @@ import org.graalvm.compiler.api.runtime.GraalRuntime;
 import org.graalvm.compiler.core.CompilationWrapper.ExceptionAction;
 import org.graalvm.compiler.core.common.CompilationIdentifier;
 import org.graalvm.compiler.core.common.GraalOptions;
+import org.graalvm.compiler.core.common.spi.ForeignCallsProvider;
 import org.graalvm.compiler.core.target.Backend;
 import org.graalvm.compiler.debug.DebugContext;
 import org.graalvm.compiler.debug.DebugContext.Description;
@@ -90,6 +87,7 @@ import jdk.vm.ci.meta.ResolvedJavaMethod;
 import jdk.vm.ci.meta.ResolvedJavaType;
 import jdk.vm.ci.runtime.JVMCI;
 import jdk.vm.ci.runtime.JVMCIBackend;
+import jdk.vm.ci.services.Services;
 
 //JaCoCo Exclude
 
@@ -98,6 +96,8 @@ import jdk.vm.ci.runtime.JVMCIBackend;
  */
 public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
 
+    private static final boolean IS_AOT = Boolean.parseBoolean(Services.getSavedProperties().get("com.oracle.graalvm.isaot"));
+
     private static boolean checkArrayIndexScaleInvariants(MetaAccessProvider metaAccess) {
         assert metaAccess.getArrayIndexScale(JavaKind.Byte) == 1;
         assert metaAccess.getArrayIndexScale(JavaKind.Boolean) == 1;
@@ -134,40 +134,6 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
     private final DiagnosticsOutputDirectory outputDirectory;
     private final Map<ExceptionAction, Integer> compilationProblemsPerAction;
 
-    /**
-     * Constants denoting the GC algorithms available in HotSpot.
-     */
-    public enum HotSpotGC {
-        Serial("UseSerialGC"),
-        Parallel("UseParallelGC", "UseParallelOldGC", "UseParNewGC"),
-        CMS("UseConcMarkSweepGC"),
-        G1("UseG1GC"),
-        Epsilon("UseEpsilonGC"),
-        Z("UseZGC");
-
-        HotSpotGC(String... flags) {
-            this.flags = flags;
-        }
-
-        private final String[] flags;
-
-        public boolean isSelected(GraalHotSpotVMConfig config) {
-            for (String flag : flags) {
-                final boolean notPresent = false;
-                if (config.getFlag(flag, Boolean.class, notPresent)) {
-                    return true;
-                }
-            }
-            return false;
-        }
-
-    }
-
-    /**
-     * Set of GCs supported by Graal.
-     */
-    private static final HotSpotGC[] SUPPORTED_GCS = {Serial, Parallel, CMS, G1};
-
     /**
      * @param nameQualifier a qualifier to be added to this runtime's {@linkplain #getName() name}
      * @param compilerConfigurationFactory factory for the compiler configuration
@@ -187,24 +153,7 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         }
         OptionValues options = optionsRef.get();
 
-        HotSpotGC selected = null;
-        for (HotSpotGC gc : SUPPORTED_GCS) {
-            if (gc.isSelected(config)) {
-                selected = gc;
-                break;
-            }
-        }
-        if (selected == null) {
-            for (HotSpotGC gc : HotSpotGC.values()) {
-                if (gc.isSelected(config)) {
-                    selected = gc;
-                    break;
-                }
-            }
-            String unsupportedGC = selected != null ? selected.name() : "<unknown>";
-            throw new GraalError(unsupportedGC + " garbage collector is not supported by Graal");
-        }
-        garbageCollector = selected;
+        garbageCollector = getSelectedGC();
 
         outputDirectory = new DiagnosticsOutputDirectory(options);
         compilationProblemsPerAction = new EnumMap<>(ExceptionAction.class);
@@ -213,9 +162,13 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         compilerConfigurationName = compilerConfigurationFactory.getName();
 
         compiler = new HotSpotGraalCompiler(jvmciRuntime, this, options);
-        management = GraalServices.loadSingle(HotSpotGraalManagementRegistration.class, false);
-        if (management != null) {
-            management.initialize(this);
+        if (IS_AOT) {
+            management = null;
+        } else {
+            management = GraalServices.loadSingle(HotSpotGraalManagementRegistration.class, false);
+            if (management != null) {
+                management.initialize(this);
+            }
         }
 
         BackendMap backendMap = compilerConfigurationFactory.createBackendMap();
@@ -265,6 +218,54 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         bootstrapJVMCI = config.getFlag("BootstrapJVMCI", Boolean.class);
     }
 
+    /**
+     * Constants denoting the GC algorithms available in HotSpot.
+     */
+    public enum HotSpotGC {
+        // Supported GCs
+        Serial(true, "UseSerialGC"),
+        Parallel(true, "UseParallelGC", "UseParallelOldGC", "UseParNewGC"),
+        CMS(true, "UseConcMarkSweepGC"),
+        G1(true, "UseG1GC"),
+
+        // Unsupported GCs
+        Epsilon(false, "UseEpsilonGC"),
+        Z(false, "UseZGC");
+
+        HotSpotGC(boolean supported, String... flags) {
+            this.supported = supported;
+            this.flags = flags;
+        }
+
+        final boolean supported;
+        private final String[] flags;
+
+        public boolean isSelected(GraalHotSpotVMConfig config) {
+            for (String flag : flags) {
+                final boolean notPresent = false;
+                if (config.getFlag(flag, Boolean.class, notPresent)) {
+                    return true;
+                }
+            }
+            return false;
+        }
+
+    }
+
+    private HotSpotGC getSelectedGC() throws GraalError {
+        for (HotSpotGC gc : HotSpotGC.values()) {
+            if (gc.isSelected(config)) {
+                if (!gc.supported) {
+                    throw new GraalError(gc.name() + " garbage collector is not supported by Graal");
+                }
+                return gc;
+            }
+        }
+        // As of JDK 9, exactly one GC flag is guaranteed to be selected.
+        // On JDK 8, the default GC is Serial when no GC flag is true.
+        return HotSpotGC.Serial;
+    }
+
     private HotSpotBackend registerBackend(HotSpotBackend backend) {
         Class<? extends Architecture> arch = backend.getTarget().arch.getClass();
         HotSpotBackend oldValue = backends.put(arch, backend);
@@ -283,7 +284,7 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
     }
 
     @Override
-    public DebugContext openDebugContext(OptionValues compilationOptions, CompilationIdentifier compilationId, Object compilable, Iterable<DebugHandlersFactory> factories) {
+    public DebugContext openDebugContext(OptionValues compilationOptions, CompilationIdentifier compilationId, Object compilable, Iterable<DebugHandlersFactory> factories, PrintStream logStream) {
         if (management != null && management.poll(false) != null) {
             if (compilable instanceof HotSpotResolvedJavaMethod) {
                 HotSpotResolvedObjectType type = ((HotSpotResolvedJavaMethod) compilable).getDeclaringClass();
@@ -301,7 +302,7 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
             }
         }
         Description description = new Description(compilable, compilationId.toString(CompilationIdentifier.Verbosity.ID));
-        return DebugContext.create(compilationOptions, description, metricValues, DEFAULT_LOG_STREAM, factories);
+        return DebugContext.create(compilationOptions, description, metricValues, logStream, factories);
     }
 
     @Override
@@ -335,8 +336,12 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
             return (T) this;
         } else if (clazz == SnippetReflectionProvider.class) {
             return (T) getHostProviders().getSnippetReflection();
+        } else if (clazz == GraalHotSpotVMConfig.class) {
+            return (T) getVMConfig();
         } else if (clazz == StampProvider.class) {
             return (T) getHostProviders().getStampProvider();
+        } else if (ForeignCallsProvider.class.isAssignableFrom(clazz)) {
+            return (T) getHostProviders().getForeignCalls();
         }
         return null;
     }
@@ -573,7 +578,7 @@ public final class HotSpotGraalRuntime implements HotSpotGraalRuntimeProvider {
         EconomicMap<OptionKey<?>, Object> extra = EconomicMap.create();
         extra.put(DebugOptions.Dump, filter);
         extra.put(DebugOptions.PrintGraphHost, host);
-        extra.put(DebugOptions.PrintBinaryGraphPort, port);
+        extra.put(DebugOptions.PrintGraphPort, port);
         OptionValues compileOptions = new OptionValues(getOptions(), extra);
         compiler.compileMethod(new HotSpotCompilationRequest(hotSpotMethod, -1, 0L), false, compileOptions);
     }
