@@ -127,7 +127,7 @@ public class CompilationResultBuilder {
     public final Assembler asm;
     public final DataBuilder dataBuilder;
     public final CompilationResult compilationResult;
-    public final Register uncompressedNullRegister;
+    public final Register nullRegister;
     public final TargetDescription target;
     public final CodeCacheProvider codeCache;
     public final ForeignCallsProvider foreignCalls;
@@ -171,44 +171,17 @@ public class CompilationResultBuilder {
      */
     private boolean conservativeLabelOffsets = false;
 
-    public final boolean mustReplaceWithUncompressedNullRegister(JavaConstant nullConstant) {
-        return !uncompressedNullRegister.equals(Register.None) && JavaConstant.NULL_POINTER.equals(nullConstant);
+    public final boolean mustReplaceWithNullRegister(JavaConstant nullConstant) {
+        return !nullRegister.equals(Register.None) && JavaConstant.NULL_POINTER.equals(nullConstant);
     }
 
-    public CompilationResultBuilder(CodeCacheProvider codeCache,
-                    ForeignCallsProvider foreignCalls,
-                    FrameMap frameMap,
-                    Assembler asm,
-                    DataBuilder dataBuilder,
-                    FrameContext frameContext,
-                    OptionValues options,
-                    DebugContext debug,
-                    CompilationResult compilationResult,
-                    Register uncompressedNullRegister) {
-        this(codeCache,
-                        foreignCalls,
-                        frameMap,
-                        asm,
-                        dataBuilder,
-                        frameContext,
-                        options,
-                        debug,
-                        compilationResult,
-                        uncompressedNullRegister,
-                        EconomicMap.create(Equivalence.DEFAULT));
+    public CompilationResultBuilder(CodeCacheProvider codeCache, ForeignCallsProvider foreignCalls, FrameMap frameMap, Assembler asm, DataBuilder dataBuilder, FrameContext frameContext,
+                    OptionValues options, DebugContext debug, CompilationResult compilationResult, Register nullRegister) {
+        this(codeCache, foreignCalls, frameMap, asm, dataBuilder, frameContext, options, debug, compilationResult, nullRegister, EconomicMap.create(Equivalence.DEFAULT));
     }
 
-    public CompilationResultBuilder(CodeCacheProvider codeCache,
-                    ForeignCallsProvider foreignCalls,
-                    FrameMap frameMap,
-                    Assembler asm,
-                    DataBuilder dataBuilder,
-                    FrameContext frameContext,
-                    OptionValues options,
-                    DebugContext debug,
-                    CompilationResult compilationResult,
-                    Register uncompressedNullRegister,
-                    EconomicMap<Constant, Data> dataCache) {
+    public CompilationResultBuilder(CodeCacheProvider codeCache, ForeignCallsProvider foreignCalls, FrameMap frameMap, Assembler asm, DataBuilder dataBuilder, FrameContext frameContext,
+                    OptionValues options, DebugContext debug, CompilationResult compilationResult, Register nullRegister, EconomicMap<Constant, Data> dataCache) {
         this.target = codeCache.getTarget();
         this.codeCache = codeCache;
         this.foreignCalls = foreignCalls;
@@ -216,7 +189,7 @@ public class CompilationResultBuilder {
         this.asm = asm;
         this.dataBuilder = dataBuilder;
         this.compilationResult = compilationResult;
-        this.uncompressedNullRegister = uncompressedNullRegister;
+        this.nullRegister = nullRegister;
         this.frameContext = frameContext;
         this.options = options;
         this.debug = debug;
