@@ -24,6 +24,8 @@
 
 package org.graalvm.compiler.lir.framemap;
 
+import java.util.BitSet;
+
 import org.graalvm.compiler.core.common.LIRKind;
 import org.graalvm.compiler.lir.VirtualStackSlot;
 
@@ -35,14 +37,21 @@ import jdk.vm.ci.code.TargetDescription;
  */
 public class VirtualStackSlotRange extends VirtualStackSlot {
 
+    private final BitSet objects;
     private final int slots;
 
-    public VirtualStackSlotRange(int id, int slots, LIRKind kind) {
+    public VirtualStackSlotRange(int id, int slots, BitSet objects, LIRKind kind) {
         super(id, kind);
         this.slots = slots;
+        this.objects = (BitSet) objects.clone();
     }
 
     public int getSlots() {
         return slots;
     }
+
+    public BitSet getObjects() {
+        return (BitSet) objects.clone();
+    }
+
 }
