@@ -60,9 +60,9 @@ import org.graalvm.compiler.lir.StandardOp;
 import org.graalvm.compiler.lir.StandardOp.BlockEndOp;
 import org.graalvm.compiler.lir.StandardOp.LabelOp;
 import org.graalvm.compiler.lir.StandardOp.SaveRegistersOp;
+import org.graalvm.compiler.lir.hashing.Hasher;
 import org.graalvm.compiler.lir.SwitchStrategy;
 import org.graalvm.compiler.lir.Variable;
-import org.graalvm.compiler.lir.hashing.Hasher;
 import org.graalvm.compiler.options.Option;
 import org.graalvm.compiler.options.OptionKey;
 import org.graalvm.compiler.options.OptionType;
@@ -229,31 +229,11 @@ public abstract class LIRGenerator implements LIRGeneratorTool {
         append(moveFactory.createMove(dst, src));
     }
 
-    @Override
-    public Variable emitReadRegister(Register register, ValueKind<?> kind) {
-        return emitMove(register.asValue(kind));
-    }
-
-    @Override
-    public void emitWriteRegister(Register dst, Value src, ValueKind<?> kind) {
-        emitMove(dst.asValue(kind), src);
-    }
-
     @Override
     public void emitMoveConstant(AllocatableValue dst, Constant src) {
         append(moveFactory.createLoad(dst, src));
     }
 
-    @Override
-    public boolean canInlineConstant(Constant constant) {
-        return moveFactory.canInlineConstant(constant);
-    }
-
-    @Override
-    public boolean mayEmbedConstantLoad(Constant constant) {
-        return moveFactory.mayEmbedConstantLoad(constant);
-    }
-
     @Override
     public Value emitConstant(LIRKind kind, Constant constant) {
         if (moveFactory.canInlineConstant(constant)) {
