@@ -35,7 +35,6 @@ import org.graalvm.compiler.graph.spi.Simplifiable;
 import org.graalvm.compiler.graph.spi.SimplifierTool;
 import org.graalvm.compiler.nodeinfo.NodeInfo;
 import org.graalvm.compiler.nodeinfo.Verbosity;
-import org.graalvm.compiler.nodes.BeginNode;
 import org.graalvm.compiler.nodes.CallTargetNode;
 import org.graalvm.compiler.nodes.FixedGuardNode;
 import org.graalvm.compiler.nodes.Invoke;
@@ -44,7 +43,7 @@ import org.graalvm.compiler.nodes.NodeView;
 import org.graalvm.compiler.nodes.PiNode;
 import org.graalvm.compiler.nodes.StructuredGraph;
 import org.graalvm.compiler.nodes.ValueNode;
-import org.graalvm.compiler.nodes.extended.AnchoringNode;
+import org.graalvm.compiler.nodes.extended.ValueAnchorNode;
 import org.graalvm.compiler.nodes.spi.UncheckedInterfaceProvider;
 import org.graalvm.compiler.nodes.type.StampTool;
 
@@ -180,38 +179,37 @@ public class MethodCallTargetNode extends CallTargetNode implements IterableNode
         Assumptions assumptions = graph().getAssumptions();
         /*
          * Even though we are not registering an assumption (see comment below), the optimization is
-         * only valid when speculative optimizations are enabled. We need to check the invoke kind
-         * to avoid recursive simplification for virtual interface methods calls.
+         * only valid when speculative optimizations are enabled.
          */
-        if (invokeKind().isInterface() && assumptions != null) {
+        if (invokeKind().isIndirect() && invokeKind().isInterface() && assumptions != null) {
+
             // check if the type of the receiver can narrow the result
             ValueNode receiver = receiver();
 
             // try to turn a interface call into a virtual call
             ResolvedJavaType declaredReceiverType = targetMethod().getDeclaringClass();
-            ResolvedJavaType referencedReceiverType = referencedType();
-            if (referencedReceiverType != null) {
-                if (declaredReceiverType.isInterface()) {
-                    ResolvedJavaType singleImplementor = referencedReceiverType.getSingleImplementor();
-                    // If singleImplementor is equal to declaredReceiverType it means that there are
-                    // multiple implementors.
-                    if (singleImplementor != null && !singleImplementor.equals(declaredReceiverType)) {
-                        TypeReference speculatedType = TypeReference.createTrusted(assumptions, singleImplementor);
-                        if (tryCheckCastSingleImplementor(receiver, speculatedType)) {
-                            return;
-                        }
+
+            /*
+             * We need to check the invoke kind to avoid recursive simplification for virtual
+             * interface methods calls.
+             */
+            if (declaredReceiverType.isInterface()) {
+                ResolvedJavaType singleImplementor = declaredReceiverType.getSingleImplementor();
+                if (singleImplementor != null && !singleImplementor.equals(declaredReceiverType)) {
+                    TypeReference speculatedType = TypeReference.createTrusted(assumptions, singleImplementor);
+                    if (tryCheckCastSingleImplementor(receiver, speculatedType)) {
+                        return;
                     }
                 }
+            }
 
-                if (receiver instanceof UncheckedInterfaceProvider) {
-                    UncheckedInterfaceProvider uncheckedInterfaceProvider = (UncheckedInterfaceProvider) receiver;
-                    Stamp uncheckedStamp = uncheckedInterfaceProvider.uncheckedStamp();
-                    if (uncheckedStamp != null) {
-                        TypeReference speculatedType = StampTool.typeReferenceOrNull(uncheckedStamp);
-                        // speculatedType must be related to the referencedReceiverType.
-                        if (speculatedType != null && referencedReceiverType.isAssignableFrom(speculatedType.getType())) {
-                            tryCheckCastSingleImplementor(receiver, speculatedType);
-                        }
+            if (receiver instanceof UncheckedInterfaceProvider) {
+                UncheckedInterfaceProvider uncheckedInterfaceProvider = (UncheckedInterfaceProvider) receiver;
+                Stamp uncheckedStamp = uncheckedInterfaceProvider.uncheckedStamp();
+                if (uncheckedStamp != null) {
+                    TypeReference speculatedType = StampTool.typeReferenceOrNull(uncheckedStamp);
+                    if (speculatedType != null) {
+                        tryCheckCastSingleImplementor(receiver, speculatedType);
                     }
                 }
             }
@@ -228,13 +226,17 @@ public class MethodCallTargetNode extends CallTargetNode implements IterableNode
                  * with an invoke virtual.
                  *
                  * To do so we need to ensure two properties: 1) the receiver must implement the
-                 * interface (referencedReceiverType). The verifier does not prove this so we need a
+                 * interface (declaredReceiverType). The verifier does not prove this so we need a
                  * dynamic check. 2) we need to ensure that there is still only one implementor of
                  * this interface, i.e. that we are calling the right method. We could do this with
                  * an assumption but as we need an instanceof check anyway we can verify both
                  * properties by checking of the receiver is an instance of the single implementor.
                  */
-                AnchoringNode anchor = BeginNode.prevBegin(invoke().asNode());
+                ValueAnchorNode anchor = new ValueAnchorNode(null);
+                if (anchor != null) {
+                    graph().add(anchor);
+                    graph().addBeforeFixed(invoke().asNode(), anchor);
+                }
                 LogicNode condition = graph().addOrUniqueWithInputs(InstanceOfNode.create(speculatedType, receiver, getProfile(), anchor));
                 FixedGuardNode guard = graph().add(new FixedGuardNode(condition, DeoptimizationReason.OptimizedTypeCheckViolated, DeoptimizationAction.InvalidateRecompile, false));
                 graph().addBeforeFixed(invoke().asNode(), guard);
@@ -272,8 +274,4 @@ public class MethodCallTargetNode extends CallTargetNode implements IterableNode
         }
         return null;
     }
-
-    public void setJavaTypeProfile(JavaTypeProfile profile) {
-        this.profile = profile;
-    }
 }
