@@ -50,10 +50,15 @@ public interface LoweringProvider {
     /**
      * Indicates the smallest width for comparing an integer value on the target platform.
      */
-    Integer smallestCompareWidth();
+    default Integer smallestCompareWidth() {
+        // most platforms only support 32 and 64 bit compares
+        return 32;
+    }
 
     /**
      * Indicates whether the target platform supports bulk zeroing instruction.
      */
-    boolean supportBulkZeroing();
+    default boolean supportBulkZeroing() {
+        return false;
+    }
 }
