@@ -79,6 +79,7 @@ import org.graalvm.compiler.options.OptionValues;
 import org.graalvm.compiler.phases.BasePhase;
 import org.graalvm.compiler.phases.Phase;
 import org.graalvm.compiler.phases.schedule.SchedulePhase;
+import org.graalvm.compiler.phases.tiers.PhaseContext;
 import jdk.internal.vm.compiler.word.LocationIdentity;
 
 import jdk.vm.ci.meta.ConstantReflectionProvider;
@@ -91,7 +92,7 @@ import jdk.vm.ci.meta.SpeculationLog.Speculation;
 /**
  * Processes all {@link Lowerable} nodes to do their lowering.
  */
-public class LoweringPhase extends BasePhase<CoreProviders> {
+public class LoweringPhase extends BasePhase<PhaseContext> {
 
     @NodeInfo(cycles = CYCLES_IGNORED, size = SIZE_IGNORED)
     static final class DummyGuardHandle extends ValueNode implements GuardedNode {
@@ -127,12 +128,12 @@ public class LoweringPhase extends BasePhase<CoreProviders> {
 
     final class LoweringToolImpl implements LoweringTool {
 
-        private final CoreProviders context;
+        private final PhaseContext context;
         private final NodeBitMap activeGuards;
         private AnchoringNode guardAnchor;
         private FixedWithNextNode lastFixedNode;
 
-        LoweringToolImpl(CoreProviders context, AnchoringNode guardAnchor, NodeBitMap activeGuards, FixedWithNextNode lastFixedNode) {
+        LoweringToolImpl(PhaseContext context, AnchoringNode guardAnchor, NodeBitMap activeGuards, FixedWithNextNode lastFixedNode) {
             this.context = context;
             this.guardAnchor = guardAnchor;
             this.activeGuards = activeGuards;
@@ -251,7 +252,7 @@ public class LoweringPhase extends BasePhase<CoreProviders> {
      * @param graph a graph that was just {@linkplain #lower lowered}
      * @throws AssertionError if the check fails
      */
-    private boolean checkPostLowering(StructuredGraph graph, CoreProviders context) {
+    private boolean checkPostLowering(StructuredGraph graph, PhaseContext context) {
         Mark expectedMark = graph.getMark();
         lower(graph, context, LoweringMode.VERIFY_LOWERING);
         Mark mark = graph.getMark();
@@ -260,13 +261,13 @@ public class LoweringPhase extends BasePhase<CoreProviders> {
     }
 
     @Override
-    protected void run(final StructuredGraph graph, CoreProviders context) {
+    protected void run(final StructuredGraph graph, PhaseContext context) {
         lower(graph, context, LoweringMode.LOWERING);
         assert checkPostLowering(graph, context);
     }
 
-    private void lower(StructuredGraph graph, CoreProviders context, LoweringMode mode) {
-        IncrementalCanonicalizerPhase<CoreProviders> incrementalCanonicalizer = new IncrementalCanonicalizerPhase<>(canonicalizer);
+    private void lower(StructuredGraph graph, PhaseContext context, LoweringMode mode) {
+        IncrementalCanonicalizerPhase<PhaseContext> incrementalCanonicalizer = new IncrementalCanonicalizerPhase<>(canonicalizer);
         incrementalCanonicalizer.appendPhase(new Round(context, mode, graph.getOptions()));
         incrementalCanonicalizer.apply(graph, context);
         assert graph.verify();
@@ -350,12 +351,12 @@ public class LoweringPhase extends BasePhase<CoreProviders> {
 
     private final class Round extends Phase {
 
-        private final CoreProviders context;
+        private final PhaseContext context;
         private final LoweringMode mode;
         private ScheduleResult schedule;
         private final SchedulePhase schedulePhase;
 
-        private Round(CoreProviders context, LoweringMode mode, OptionValues options) {
+        private Round(PhaseContext context, LoweringMode mode, OptionValues options) {
             this.context = context;
             this.mode = mode;
 
