@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -43,7 +43,7 @@ public class StringSubstitutionsTest extends MethodSubstitutionTest {
         StructuredGraph graph = testGraph(testMethodName);
 
         // Check to see if the resulting graph contains the expected node
-        StructuredGraph replacement = getReplacements().getSubstitution(realMethod, -1, false, null);
+        StructuredGraph replacement = getReplacements().getSubstitution(realMethod, -1, false, null, graph.getOptions());
         if (replacement == null && !optional) {
             assertInGraph(graph, intrinsicClass);
         }
@@ -102,4 +102,44 @@ public class StringSubstitutionsTest extends MethodSubstitutionTest {
         return a.equals(b);
     }
 
+    @Test
+    public void testIndexOfConstant() {
+        test("indexOfConstant");
+    }
+
+    public int indexOfConstant() {
+        String foobar = "foobar";
+        String bar = "bar";
+        return foobar.indexOf(bar);
+    }
+
+    @Test
+    public void testIndexOfConstantUTF16() {
+        test("indexOfConstantUTF16case1");
+        test("indexOfConstantUTF16case2");
+        test("indexOfConstantUTF16case3");
+    }
+
+    public int indexOfConstantUTF16case1() {
+        return ("grga " + ((char) 0x10D) + "varak").indexOf(((char) 0x10D) + "varak");
+    }
+
+    public int indexOfConstantUTF16case2() {
+        int index = ("grga " + ((char) 0xD) + "varak").indexOf(((char) 0x10D) + "varak");
+        return index;
+    }
+
+    public int indexOfConstantUTF16case3() {
+        int index = ("grga " + ((char) 0x100) + "varak").indexOf(((char) 0x10D) + "varak");
+        return index;
+    }
+
+    @Test
+    public void testCompareTo() {
+        test("compareTo");
+    }
+
+    public int compareTo() {
+        return "ofar".compareTo("rafo");
+    }
 }
