@@ -28,7 +28,6 @@ import static java.lang.Thread.currentThread;
 
 import java.io.IOException;
 import java.io.InputStream;
-import java.lang.reflect.Method;
 import java.util.Arrays;
 import java.util.ArrayList;
 import java.util.HashMap;
@@ -40,8 +39,6 @@ import java.util.ServiceLoader;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.function.Supplier;
 
-import jdk.vm.ci.meta.ConstantPool;
-import jdk.vm.ci.meta.JavaType;
 import org.graalvm.compiler.serviceprovider.SpeculationReasonGroup.SpeculationContextObject;
 
 import jdk.vm.ci.code.BytecodePosition;
@@ -92,15 +89,7 @@ public final class GraalServices {
             synchronized (servicesCache) {
                 ArrayList<S> providersList = new ArrayList<>();
                 for (S provider : providers) {
-                    /*
-                     * When building libgraal, we want providers that comes from the Graal community
-                     * and enterprise modules but not those available on the native-image class
-                     * path.
-                     */
-                    Module module = provider.getClass().getModule();
-                    if (module.isNamed()) {
-                        providersList.add(provider);
-                    }
+                    providersList.add(provider);
                 }
                 providers = providersList;
                 servicesCache.put(service, providersList);
@@ -112,7 +101,7 @@ public final class GraalServices {
     }
 
     protected static <S> Iterable<S> load0(Class<S> service) {
-        Iterable<S> iterable = ServiceLoader.load(service);
+        Iterable<S> iterable = ServiceLoader.load(service, GraalServices.class.getClassLoader());
         return new Iterable<>() {
             @Override
             public Iterator<S> iterator() {
@@ -147,9 +136,7 @@ public final class GraalServices {
      * @param other all JVMCI packages will be opened to the module defining this class
      */
     static void openJVMCITo(Class<?> other) {
-        if (IS_IN_NATIVE_IMAGE) {
-            return;
-        }
+        if (IS_IN_NATIVE_IMAGE) return;
 
         Module jvmciModule = JVMCI_MODULE;
         Module otherModule = other.getModule();
@@ -534,36 +521,11 @@ public final class GraalServices {
         return Math.fma(a, b, c);
     }
 
-    public static VirtualObject createVirtualObject(ResolvedJavaType type, int id, boolean isAutoBox) {
-        return VirtualObject.get(type, id, isAutoBox);
-    }
-
-    private static final Method constantPoolLookupReferencedType;
-
-    static {
-        Method lookupReferencedType = null;
-        Class<?> constantPool = ConstantPool.class;
-        try {
-            lookupReferencedType = constantPool.getDeclaredMethod("lookupReferencedType", Integer.TYPE, Integer.TYPE);
-        } catch (NoSuchMethodException e) {
-        }
-        constantPoolLookupReferencedType = lookupReferencedType;
-    }
-
-    public static JavaType lookupReferencedType(ConstantPool constantPool, int cpi, int opcode) {
-        if (constantPoolLookupReferencedType != null) {
-            try {
-                return (JavaType) constantPoolLookupReferencedType.invoke(constantPool, cpi, opcode);
-            } catch (Error e) {
-                throw e;
-            } catch (Throwable throwable) {
-                throw new InternalError(throwable);
-            }
-        }
-        throw new InternalError("This JVMCI version doesn't support ConstantPool.lookupReferencedType()");
-    }
-
-    public static boolean hasLookupReferencedType() {
-        return constantPoolLookupReferencedType != null;
+    /**
+     * Set the flag in the {@link VirtualObject} that indicates that it is a boxed primitive that
+     * was produced as a result of a call to a {@code valueOf} method.
+     */
+    public static void markVirtualObjectAsAutoBox(VirtualObject virtualObject) {
+       virtualObject.setIsAutoBox(true);
     }
 }
