@@ -44,6 +44,7 @@ import org.graalvm.compiler.debug.DebugHandlersFactory;
 import org.graalvm.compiler.debug.GlobalMetrics;
 import org.graalvm.compiler.options.OptionValues;
 import org.graalvm.compiler.serviceprovider.GraalServices;
+import org.graalvm.compiler.serviceprovider.JavaVersionUtil;
 import org.junit.After;
 import org.junit.Assert;
 import org.junit.AssumptionViolatedException;
@@ -72,8 +73,8 @@ public class GraalTest {
         }
     }
 
-    public static final boolean Java8OrEarlier = GraalServices.Java8OrEarlier;
-    public static final boolean Java11OrEarlier = GraalServices.Java11OrEarlier;
+    public static final boolean Java8OrEarlier = JavaVersionUtil.Java8OrEarlier;
+    public static final boolean Java11OrEarlier = JavaVersionUtil.Java11OrEarlier;
 
     protected Method getMethod(String methodName) {
         return getMethod(getClass(), methodName);
