@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -77,7 +77,7 @@ public class FcFontManager extends SunFontManager {
             for (int i=0; i<fontConfigFonts.length; i++) {
                 if ("sans".equals(fontConfigFonts[i].fcFamily) &&
                     0 == fontConfigFonts[i].style) {
-                    info[0] = fontConfigFonts[i].firstFont.familyName;
+                    info[0] = fontConfigFonts[i].firstFont.fullName;
                     info[1] = fontConfigFonts[i].firstFont.fontFile;
                     break;
                 }
@@ -90,7 +90,7 @@ public class FcFontManager extends SunFontManager {
         if (info[0] == null) {
             if (fontConfigFonts != null && fontConfigFonts.length > 0 &&
                 fontConfigFonts[0].firstFont.fontFile != null) {
-                info[0] = fontConfigFonts[0].firstFont.familyName;
+                info[0] = fontConfigFonts[0].firstFont.fullName;
                 info[1] = fontConfigFonts[0].firstFont.fontFile;
             } else {
                 info[0] = "Dialog";
