@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2018, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -1250,24 +1250,15 @@ public abstract class FontConfiguration {
         return filenamesMap.get(platformName);
     }
 
-    private static final String fontconfigErrorMessage =
-            "Fontconfig head is null, check your fonts or fonts configuration";
-
     /**
      * Returns a configuration specific path to be appended to the font
      * search path.
      */
     public String getExtraFontPath() {
-        if (head == null) {
-            throw new RuntimeException(fontconfigErrorMessage);
-        }
         return getString(head[INDEX_appendedfontpath]);
     }
 
     public String getVersion() {
-        if (head == null) {
-            throw new RuntimeException(fontconfigErrorMessage);
-        }
         return getString(head[INDEX_version]);
     }
 
@@ -1615,7 +1606,7 @@ public abstract class FontConfiguration {
         }
     }
 
-    private static boolean contains(short IDs[], short id, int limit) {
+    private static boolean contains(short[] IDs, short id, int limit) {
         for (int i = 0; i < limit; i++) {
             if (IDs[i] == id) {
                 return true;
@@ -1767,7 +1758,7 @@ public abstract class FontConfiguration {
         return (short)(stringIDNum - 1);
     }
 
-    private static short getShortArrayID(short sa[]) {
+    private static short getShortArrayID(short[] sa) {
         char[] cc = new char[sa.length];
         for (int i = 0; i < sa.length; i ++) {
             cc[i] = (char)sa[i];
