@@ -93,6 +93,14 @@ public abstract class UNIXToolkit extends SunToolkit
         }
     }
 
+    @Override
+    public String getDesktop() {
+        String gsi = AccessController.doPrivileged(
+                        (PrivilegedAction<String>) ()
+                                -> System.getenv("GNOME_DESKTOP_SESSION_ID"));
+        return (gsi != null) ? "gnome" : null;
+    }
+
     /**
      * Returns true if the native GTK libraries are capable of being
      * loaded and are expected to work properly, false otherwise.  Note
