@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -941,7 +941,7 @@ public class XBaseWindow {
 
     static void checkSecurity() {
         if (XToolkit.isSecurityWarningEnabled() && XToolkit.isToolkitThread()) {
-            StackTraceElement stack[] = (new Throwable()).getStackTrace();
+            StackTraceElement[] stack = (new Throwable()).getStackTrace();
             log.warning(stack[1] + ": Security violation: calling user code on toolkit thread");
         }
     }
