@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -233,7 +233,7 @@ public class XPopupMenuPeer extends XMenuWindow implements PopupMenuPeer {
      */
     protected Dimension getCaptionSize() {
         String s = getTargetLabel();
-        if (s.equals("")) {
+        if (s.isEmpty()) {
             return null;
         }
         Graphics g = getGraphics();
@@ -260,7 +260,7 @@ public class XPopupMenuPeer extends XMenuWindow implements PopupMenuPeer {
      */
     protected void paintCaption(Graphics g, Rectangle rect) {
         String s = getTargetLabel();
-        if (s.equals("")) {
+        if (s.isEmpty()) {
             return;
         }
         g.setFont(getTargetFont());
