@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -22,19 +22,22 @@
  *
  *
  */
+
 package sun.awt.X11;
 
-import java.awt.*;
-import java.awt.peer.*;
+import java.awt.GraphicsConfiguration;
+import java.awt.Rectangle;
+import java.awt.Toolkit;
+import java.awt.peer.RobotPeer;
 import java.security.AccessController;
-import sun.security.action.GetPropertyAction;
 
 import sun.awt.AWTAccessor;
 import sun.awt.SunToolkit;
 import sun.awt.UNIXToolkit;
 import sun.awt.X11GraphicsConfig;
+import sun.security.action.GetPropertyAction;
 
-class XRobotPeer implements RobotPeer {
+final class XRobotPeer implements RobotPeer {
 
     static final boolean tryGtk;
     static {
@@ -69,11 +72,6 @@ class XRobotPeer implements RobotPeer {
         useGtk = (tryGtk && isGtkSupported);
     }
 
-    @Override
-    public void dispose() {
-        // does nothing
-    }
-
     @Override
     public void mouseMove(int x, int y) {
         mouseMoveImpl(xgc, xgc.scaleUp(x), xgc.scaleUp(y));
