@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -511,7 +511,7 @@ public abstract class X11InputMethodBase extends InputMethodAdapter {
             flush += composedText.toString();
         }
 
-        if (!flush.equals("")) {
+        if (!flush.isEmpty()) {
             AttributedString attrstr = new AttributedString(flush);
             postInputMethodEvent(InputMethodEvent.INPUT_METHOD_TEXT_CHANGED,
                                  attrstr.getIterator(),
