@@ -542,7 +542,7 @@ final class Stroker implements PathConsumer2D, MarlinConst {
 
                 // basic rejection criteria:
                 if (sideCode == 0) {
-                    // overlap clip:
+                    // ovelap clip:
                     if (subdivide) {
                         // avoid reentrance
                         subdivide = false;
@@ -636,9 +636,6 @@ final class Stroker implements PathConsumer2D, MarlinConst {
         emitReverse();
 
         this.prev = CLOSE;
-        this.cx0 = sx0;
-        this.cy0 = sy0;
-        this.cOutCode = sOutCode;
 
         if (opened) {
             // do not emit close
@@ -673,9 +670,7 @@ final class Stroker implements PathConsumer2D, MarlinConst {
         //          i.e. if caps must be drawn or not ?
         // Solution: use the ClosedPathDetector before Stroker to determine
         // if the path is a closed path or not
-        if (rdrCtx.closedPath) {
-            emitReverse();
-        } else {
+        if (!rdrCtx.closedPath) {
             if (outcode == 0) {
                 // current point = end's cap:
                 if (capStyle == CAP_ROUND) {
@@ -700,6 +695,8 @@ final class Stroker implements PathConsumer2D, MarlinConst {
                     }
                 }
             }
+        } else {
+            emitReverse();
         }
         emitClose();
     }
@@ -1063,7 +1060,7 @@ final class Stroker implements PathConsumer2D, MarlinConst {
 
                 // basic rejection criteria:
                 if (sideCode == 0) {
-                    // overlap clip:
+                    // ovelap clip:
                     if (subdivide) {
                         // avoid reentrance
                         subdivide = false;
@@ -1211,7 +1208,7 @@ final class Stroker implements PathConsumer2D, MarlinConst {
 
                 // basic rejection criteria:
                 if (sideCode == 0) {
-                    // overlap clip:
+                    // ovelap clip:
                     if (subdivide) {
                         // avoid reentrance
                         subdivide = false;
