@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -25,8 +25,16 @@
 package sun.jvm.hotspot.debugger.linux;
 
 import java.io.File;
+import java.io.IOException;
+import java.io.UncheckedIOException;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Map;
+import java.util.NoSuchElementException;
+import java.util.stream.Collectors;
 
 import sun.jvm.hotspot.debugger.Address;
 import sun.jvm.hotspot.debugger.DebuggerBase;
@@ -72,6 +80,10 @@ public class LinuxDebuggerLocal extends DebuggerBase implements LinuxDebugger {
     private List threadList;
     private List loadObjectList;
 
+    // PID namespace support
+    // It maps the LWPID in the host to the LWPID in the container.
+    private Map<Integer, Integer> nspidMap;
+
     // called by native method lookupByAddress0
     private ClosestSymbol createClosestSymbol(String name, long offset) {
        return new ClosestSymbol(name, offset);
@@ -89,6 +101,7 @@ public class LinuxDebuggerLocal extends DebuggerBase implements LinuxDebugger {
 
     private native static void init0()
                                 throws DebuggerException;
+    private native void setSAAltRoot0(String altroot);
     private native void attach0(int pid)
                                 throws DebuggerException;
     private native void attach0(String execName, String coreName)
@@ -254,11 +267,58 @@ public class LinuxDebuggerLocal extends DebuggerBase implements LinuxDebugger {
         }
     }
 
+    // Get namespace PID from /proc/<PID>/status.
+    private int getNamespacePID(Path statusPath) {
+        try (var lines = Files.lines(statusPath)) {
+            return lines.map(s -> s.split("\\s+"))
+                        .filter(a -> a.length == 3)
+                        .filter(a -> a[0].equals("NSpid:"))
+                        .mapToInt(a -> Integer.valueOf(a[2]))
+                        .findFirst()
+                        .getAsInt();
+        } catch (IOException | NoSuchElementException e) {
+            return Integer.valueOf(statusPath.getParent()
+                                             .toFile()
+                                             .getName());
+        }
+    }
+
+    // Get LWPID in the host from the container's LWPID.
+    // Returns -1 if the process is running in the host.
+    public int getHostPID(int id) {
+        return (nspidMap == null) ? -1 : nspidMap.get(id);
+    }
+
+    // Fill namespace PID map from procfs.
+    // This method scans all tasks (/proc/<PID>/task) in the process.
+    private void fillNSpidMap(Path proc) {
+        Path task = Paths.get(proc.toString(), "task");
+        try (var tasks = Files.list(task)) {
+            nspidMap = tasks.filter(p -> !p.toString().startsWith("."))
+                            .collect(Collectors.toMap(p -> Integer.valueOf(getNamespacePID(Paths.get(p.toString(), "status"))),
+                                                      p -> Integer.valueOf(p.toFile().getName())));
+        } catch (IOException e) {
+            throw new UncheckedIOException(e);
+        }
+    }
+
     /** From the Debugger interface via JVMDebugger */
     public synchronized void attach(int processID) throws DebuggerException {
         checkAttached();
         threadList = new ArrayList();
         loadObjectList = new ArrayList();
+
+        Path proc = Paths.get("/proc", Integer.toString(processID));
+        int NSpid = getNamespacePID(Paths.get(proc.toString(), "status"));
+        if (NSpid != processID) {
+            // If PID different from namespace PID, we can assume the process
+            // is running in the container.
+            // So we need to set SA_ALTROOT environment variable that SA reads
+            // binaries in the container.
+            setSAAltRoot0(Paths.get(proc.toString(), "root").toString());
+            fillNSpidMap(proc);
+        }
+
         class AttachTask implements WorkerThreadTask {
            int pid;
            public void doit(LinuxDebuggerLocal debugger) {
