@@ -80,6 +80,11 @@ public class EpsilonHeap extends CollectedHeap {
     return space;
   }
 
+  @Override
+  public void liveRegionsIterate(LiveRegionsClosure closure) {
+    closure.doLiveRegions(space());
+  }
+
   @Override
   public void printOn(PrintStream tty) {
      MemRegion mr = reservedRegion();
