@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -45,10 +45,10 @@ import java.io.PrintStream;
 import java.io.UnsupportedEncodingException;
 import java.lang.module.Configuration;
 import java.lang.module.ModuleDescriptor;
+import java.lang.module.ModuleDescriptor.Requires;
 import java.lang.module.ModuleDescriptor.Exports;
 import java.lang.module.ModuleDescriptor.Opens;
 import java.lang.module.ModuleDescriptor.Provides;
-import java.lang.module.ModuleDescriptor.Requires;
 import java.lang.module.ModuleFinder;
 import java.lang.module.ModuleReference;
 import java.lang.module.ResolvedModule;
@@ -62,8 +62,8 @@ import java.nio.charset.Charset;
 import java.nio.file.DirectoryStream;
 import java.nio.file.Files;
 import java.nio.file.Path;
-import java.text.MessageFormat;
 import java.text.Normalizer;
+import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
@@ -87,7 +87,6 @@ import jdk.internal.module.ModuleBootstrap;
 import jdk.internal.module.Modules;
 import jdk.internal.platform.Container;
 import jdk.internal.platform.Metrics;
-import sun.util.calendar.ZoneInfoFile;
 
 
 public final class LauncherHelper {
@@ -120,7 +119,6 @@ public final class LauncherHelper {
 
     private static final String defaultBundleName =
             "sun.launcher.resources.launcher";
-
     private static class ResourceBundleHolder {
         private static final ResourceBundle RB =
                 ResourceBundle.getBundle(defaultBundleName);
@@ -281,8 +279,6 @@ public final class LauncherHelper {
                 Locale.getDefault(Category.DISPLAY).getDisplayName());
         ostream.println(INDENT + "default format locale = " +
                 Locale.getDefault(Category.FORMAT).getDisplayName());
-        ostream.println(INDENT + "tzdata version = " +
-                ZoneInfoFile.getVersion());
         printLocales();
         ostream.println();
     }
@@ -327,121 +323,91 @@ public final class LauncherHelper {
             return;
         }
 
-        final long longRetvalNotSupported = -2;
-
         ostream.println(INDENT + "Provider: " + c.getProvider());
         ostream.println(INDENT + "Effective CPU Count: " + c.getEffectiveCpuCount());
-        ostream.println(formatCpuVal(c.getCpuPeriod(), INDENT + "CPU Period: ", longRetvalNotSupported));
-        ostream.println(formatCpuVal(c.getCpuQuota(), INDENT + "CPU Quota: ", longRetvalNotSupported));
-        ostream.println(formatCpuVal(c.getCpuShares(), INDENT + "CPU Shares: ", longRetvalNotSupported));
+        ostream.println(INDENT + "CPU Period: " + c.getCpuPeriod() +
+               (c.getCpuPeriod() == -1 ? "" : "us"));
+        ostream.println(INDENT + "CPU Quota: " + c.getCpuQuota() +
+               (c.getCpuQuota() == -1 ? "" : "us"));
+        ostream.println(INDENT + "CPU Shares: " + c.getCpuShares());
 
         int cpus[] = c.getCpuSetCpus();
-        if (cpus != null) {
-            ostream.println(INDENT + "List of Processors, "
-                    + cpus.length + " total: ");
+        ostream.println(INDENT + "List of Processors, "
+                + cpus.length + " total: ");
 
-            ostream.print(INDENT);
-            for (int i = 0; i < cpus.length; i++) {
-                ostream.print(cpus[i] + " ");
-            }
-            if (cpus.length > 0) {
-                ostream.println("");
-            }
-        } else {
-            ostream.println(INDENT + "List of Processors: N/A");
+        ostream.print(INDENT);
+        for (int i = 0; i < cpus.length; i++) {
+            ostream.print(cpus[i] + " ");
+        }
+        if (cpus.length > 0) {
+            ostream.println("");
         }
 
         cpus = c.getEffectiveCpuSetCpus();
-        if (cpus != null) {
-            ostream.println(INDENT + "List of Effective Processors, "
-                    + cpus.length + " total: ");
+        ostream.println(INDENT + "List of Effective Processors, "
+                + cpus.length + " total: ");
 
-            ostream.print(INDENT);
-            for (int i = 0; i < cpus.length; i++) {
-                ostream.print(cpus[i] + " ");
-            }
-            if (cpus.length > 0) {
-                ostream.println("");
-            }
-        } else {
-            ostream.println(INDENT + "List of Effective Processors: N/A");
+        ostream.print(INDENT);
+        for (int i = 0; i < cpus.length; i++) {
+            ostream.print(cpus[i] + " ");
+        }
+        if (cpus.length > 0) {
+            ostream.println("");
         }
 
         int mems[] = c.getCpuSetMems();
-        if (mems != null) {
-            ostream.println(INDENT + "List of Memory Nodes, "
-                    + mems.length + " total: ");
+        ostream.println(INDENT + "List of Memory Nodes, "
+                + mems.length + " total: ");
 
-            ostream.print(INDENT);
-            for (int i = 0; i < mems.length; i++) {
-                ostream.print(mems[i] + " ");
-            }
-            if (mems.length > 0) {
-                ostream.println("");
-            }
-        } else {
-            ostream.println(INDENT + "List of Memory Nodes: N/A");
+        ostream.print(INDENT);
+        for (int i = 0; i < mems.length; i++) {
+            ostream.print(mems[i] + " ");
+        }
+        if (mems.length > 0) {
+            ostream.println("");
         }
 
         mems = c.getEffectiveCpuSetMems();
-        if (mems != null) {
-            ostream.println(INDENT + "List of Available Memory Nodes, "
-                    + mems.length + " total: ");
+        ostream.println(INDENT + "List of Available Memory Nodes, "
+                + mems.length + " total: ");
 
-            ostream.print(INDENT);
-            for (int i = 0; i < mems.length; i++) {
-                ostream.print(mems[i] + " ");
-            }
-            if (mems.length > 0) {
-                ostream.println("");
-            }
-        } else {
-            ostream.println(INDENT + "List of Available Memory Nodes: N/A");
+        ostream.print(INDENT);
+        for (int i = 0; i < mems.length; i++) {
+            ostream.print(mems[i] + " ");
         }
+        if (mems.length > 0) {
+            ostream.println("");
+        }
+
+        ostream.println(INDENT + "CPUSet Memory Pressure Enabled: "
+                + c.isCpuSetMemoryPressureEnabled());
 
         long limit = c.getMemoryLimit();
-        ostream.println(formatLimitString(limit, INDENT + "Memory Limit: ", longRetvalNotSupported));
+        ostream.println(INDENT + "Memory Limit: " +
+                ((limit >= 0) ? SizePrefix.scaleValue(limit) : "Unlimited"));
 
         limit = c.getMemorySoftLimit();
-        ostream.println(formatLimitString(limit, INDENT + "Memory Soft Limit: ", longRetvalNotSupported));
+        ostream.println(INDENT + "Memory Soft Limit: " +
+                ((limit >= 0) ? SizePrefix.scaleValue(limit) : "Unlimited"));
 
         limit = c.getMemoryAndSwapLimit();
-        ostream.println(formatLimitString(limit, INDENT + "Memory & Swap Limit: ", longRetvalNotSupported));
+        ostream.println(INDENT + "Memory & Swap Limit: " +
+                ((limit >= 0) ? SizePrefix.scaleValue(limit) : "Unlimited"));
+
+        limit = c.getKernelMemoryLimit();
+        ostream.println(INDENT + "Kernel Memory Limit: " +
+                ((limit >= 0) ? SizePrefix.scaleValue(limit) : "Unlimited"));
+
+        limit = c.getTcpMemoryLimit();
+        ostream.println(INDENT + "TCP Memory Limit: " +
+                ((limit >= 0) ? SizePrefix.scaleValue(limit) : "Unlimited"));
+
+        ostream.println(INDENT + "Out Of Memory Killer Enabled: "
+                + c.isMemoryOOMKillEnabled());
 
-        limit = c.getPidsMax();
-        ostream.println(formatLimitString(limit, INDENT + "Maximum Processes Limit: ",
-                                          longRetvalNotSupported, false));
         ostream.println("");
     }
 
-    private static String formatLimitString(long limit, String prefix, long unavailable) {
-        return formatLimitString(limit, prefix, unavailable, true);
-    }
-
-    private static String formatLimitString(long limit, String prefix, long unavailable, boolean scale) {
-        if (limit >= 0) {
-            if (scale) {
-                return prefix + SizePrefix.scaleValue(limit);
-            } else {
-                return prefix + limit;
-            }
-        } else if (limit == unavailable) {
-            return prefix + "N/A";
-        } else {
-            return prefix + "Unlimited";
-        }
-    }
-
-    private static String formatCpuVal(long cpuVal, String prefix, long unavailable) {
-        if (cpuVal >= 0) {
-            return prefix + cpuVal + "us";
-        } else if (cpuVal == unavailable) {
-            return prefix + "N/A";
-        } else {
-            return prefix + cpuVal;
-        }
-    }
-
     private enum SizePrefix {
 
         KILO(1024, "K"),
@@ -855,7 +821,7 @@ public final class LauncherHelper {
         } catch (Throwable e) {
             if (mainClass.getModule().isNamed()) {
                 abort(e, "java.launcher.module.error5",
-                      mainClass.getName(), mainClass.getModule(),
+                      mainClass.getName(), mainClass.getModule().getName(),
                       e.getClass().getName(), e.getLocalizedMessage());
             } else {
                 abort(e, "java.launcher.cls.error7", mainClass.getName(),
