@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1994, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1994, 2016, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -118,13 +118,13 @@ public class HttpClient extends NetworkClient {
                                          recomputing the value of keepingAlive */
     int keepAliveConnections = -1;    /* number of keep-alives left */
 
-    /*
-     * The timeout if specified by the server. Following values possible
-     *  0: the server specified no keep alive headers
-     * -1: the server provided "Connection: keep-alive" but did not specify a
-     *     a particular time in a "Keep-Alive:" headers
-     * Positive values are the number of seconds specified by the server
-     * in a "Keep-Alive" header
+    /**Idle timeout value, in milliseconds. Zero means infinity,
+     * iff keepingAlive=true.
+     * Unfortunately, we can't always believe this one.  If I'm connected
+     * through a Netscape proxy to a server that sent me a keep-alive
+     * time of 15 sec, the proxy unilaterally terminates my connection
+     * after 5 sec.  So we have to hard code our effective timeout to
+     * 4 sec for the case where we're using a proxy. *SIGH*
      */
     int keepAliveTimeout = 0;
 
@@ -140,27 +140,12 @@ public class HttpClient extends NetworkClient {
     // Traffic capture tool, if configured. See HttpCapture class for info
     private HttpCapture capture = null;
 
-    /* "jdk.https.negotiate.cbt" property can be set to "always" (always sent), "never" (never sent) or
-     * "domain:a,c.d,*.e.f" (sent to host a, or c.d or to the domain e.f and any of its subdomains). This is
-     * a comma separated list of arbitrary length with no white-space allowed.
-     * If enabled (for a particular destination) then Negotiate/SPNEGO authentication requests will include
-     * a channel binding token for the destination server. The default behavior and setting for the
-     * property is "never"
-     */
-    private static final String spnegoCBT;
-
     private static final PlatformLogger logger = HttpURLConnection.getHttpLogger();
-
     private static void logFinest(String msg) {
         if (logger.isLoggable(PlatformLogger.Level.FINEST)) {
             logger.finest(msg);
         }
     }
-    private static void logError(String msg) {
-        if (logger.isLoggable(PlatformLogger.Level.SEVERE)) {
-            logger.severe(msg);
-        }
-    }
 
     protected volatile String authenticatorKey;
 
@@ -176,18 +161,6 @@ public class HttpClient extends NetworkClient {
         return keepAliveTimeout;
     }
 
-    static String normalizeCBT(String s) {
-        if (s == null || s.equals("never")) {
-            return "never";
-        }
-        if (s.equals("always") || s.startsWith("domain:")) {
-            return s;
-        } else {
-            logError("Unexpected value for \"jdk.https.negotiate.cbt\" system property");
-            return "never";
-        }
-    }
-
     static {
         Properties props = GetPropertyAction.privilegedGetProperties();
         String keepAlive = props.getProperty("http.keepAlive");
@@ -195,9 +168,6 @@ public class HttpClient extends NetworkClient {
         String cacheNTLM = props.getProperty("jdk.ntlm.cache");
         String cacheSPNEGO = props.getProperty("jdk.spnego.cache");
 
-        String s = props.getProperty("jdk.https.negotiate.cbt");
-        spnegoCBT = normalizeCBT(s);
-
         if (keepAlive != null) {
             keepAliveProp = Boolean.parseBoolean(keepAlive);
         } else {
@@ -232,10 +202,6 @@ public class HttpClient extends NetworkClient {
     }
 
 
-    public String getSpnegoCBT() {
-        return spnegoCBT;
-    }
-
     protected HttpClient() {
     }
 
@@ -250,7 +216,7 @@ public class HttpClient extends NetworkClient {
     }
 
     /* This package-only CTOR should only be used for FTP piggy-backed on HTTP
-     * HTTP URL's that use this won't take advantage of keep-alive.
+     * URL's that use this won't take advantage of keep-alive.
      * Additionally, this constructor may be used as a last resort when the
      * first HttpClient gotten through New() failed (probably b/c of a
      * Keep-Alive mismatch).
@@ -337,7 +303,7 @@ public class HttpClient extends NetworkClient {
             ret = kac.get(url, null);
             if (ret != null && httpuc != null &&
                 httpuc.streaming() &&
-                "POST".equals(httpuc.getRequestMethod())) {
+                httpuc.getRequestMethod() == "POST") {
                 if (!ret.available()) {
                     ret.inCache = false;
                     ret.closeServer();
@@ -853,7 +819,7 @@ public class HttpClient extends NetworkClient {
                             responses.findValue("Keep-Alive"));
                         /* default should be larger in case of proxy */
                         keepAliveConnections = p.findInt("max", usingProxy?50:5);
-                        keepAliveTimeout = p.findInt("timeout", -1);
+                        keepAliveTimeout = p.findInt("timeout", usingProxy?60:5);
                     }
                 } else if (b[7] != '0') {
                     /*
@@ -1093,10 +1059,6 @@ public class HttpClient extends NetworkClient {
         } catch (Exception e) {}
     }
 
-    public boolean getUsingProxy() {
-        return usingProxy;
-    }
-
     /**
      * @return the proxy host being used for this client, or null
      *          if we're not going through a proxy
