@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -457,7 +457,7 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
      */
     private PrintService getServiceByName(PrinterName nameAttr) {
         String name = nameAttr.getValue();
-        if (name == null || name.equals("") || !checkPrinterName(name)) {
+        if (name == null || name.isEmpty() || !checkPrinterName(name)) {
             return null;
         }
         /* check if all printers are already available */
@@ -823,7 +823,7 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
         for (int i=0; i < names.length; i++) {
             if (!names[i].equals("_default") &&
                 !names[i].equals(defaultPrinter) &&
-                !names[i].equals("")) {
+                !names[i].isEmpty()) {
                 printerNames.add(names[i]);
             }
         }
