@@ -324,9 +324,6 @@ final class P11AEADCipher extends CipherSpi {
         try {
             initialize();
         } catch (PKCS11Exception e) {
-            if (e.getErrorCode() == CKR_MECHANISM_PARAM_INVALID) {
-                throw new InvalidAlgorithmParameterException("Bad params", e);
-            }
             throw new InvalidKeyException("Could not initialize cipher", e);
         }
     }
@@ -381,6 +378,9 @@ final class P11AEADCipher extends CipherSpi {
 
         long p11KeyID = p11Key.getKeyID();
         try {
+            if (session == null) {
+                session = token.getOpSession();
+            }
             CK_MECHANISM mechWithParams;
             switch (blockMode) {
                 case MODE_GCM:
@@ -390,9 +390,6 @@ final class P11AEADCipher extends CipherSpi {
                 default:
                     throw new ProviderException("Unsupported mode: " + blockMode);
             }
-            if (session == null) {
-                session = token.getOpSession();
-            }
             if (encrypt) {
                 token.p11.C_EncryptInit(session.id(), mechWithParams,
                     p11KeyID);
@@ -401,6 +398,7 @@ final class P11AEADCipher extends CipherSpi {
                     p11KeyID);
             }
         } catch (PKCS11Exception e) {
+            //e.printStackTrace();
             p11Key.releaseKeyID();
             session = token.releaseSession(session);
             throw e;
@@ -720,9 +718,7 @@ final class P11AEADCipher extends CipherSpi {
                    errorCode == CKR_ENCRYPTED_DATA_LEN_RANGE) {
             throw (IllegalBlockSizeException)
                     (new IllegalBlockSizeException(e.toString()).initCause(e));
-        } else if (errorCode == CKR_ENCRYPTED_DATA_INVALID ||
-                // Solaris-specific
-                errorCode == CKR_GENERAL_ERROR) {
+        } else if (errorCode == CKR_ENCRYPTED_DATA_INVALID) {
             throw (BadPaddingException)
                     (new BadPaddingException(e.toString()).initCause(e));
         }
