@@ -160,12 +160,13 @@ public final class P11TlsMasterSecretGenerator extends KeyGeneratorSpi {
             ckMechanism = new CK_MECHANISM(mechanism, params);
         }
         Session session = null;
+        long p11KeyID = p11Key.getKeyID();
         try {
             session = token.getObjSession();
             CK_ATTRIBUTE[] attributes = token.getAttributes(O_GENERATE,
                 CKO_SECRET_KEY, CKK_GENERIC_SECRET, new CK_ATTRIBUTE[0]);
             long keyID = token.p11.C_DeriveKey(session.id(),
-                    ckMechanism, p11Key.keyID, attributes);
+                    ckMechanism, p11KeyID, attributes);
             int major, minor;
             if (ckVersion == null) {
                 major = -1;
@@ -174,12 +175,12 @@ public final class P11TlsMasterSecretGenerator extends KeyGeneratorSpi {
                 major = ckVersion.major;
                 minor = ckVersion.minor;
             }
-            SecretKey key = P11Key.masterSecretKey(session, keyID,
+            return P11Key.masterSecretKey(session, keyID,
                 "TlsMasterSecret", 48 << 3, attributes, major, minor);
-            return key;
         } catch (Exception e) {
             throw new ProviderException("Could not generate key", e);
         } finally {
+            p11Key.releaseKeyID();
             token.releaseSession(session);
         }
     }
