@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -81,7 +81,6 @@ public class AlgorithmId implements Serializable, DerEncoder {
      */
     protected DerValue          params;
 
-    private transient byte[] encodedParams;
 
     /**
      * Constructs an algorithm ID which will be initialized
@@ -110,17 +109,6 @@ public class AlgorithmId implements Serializable, DerEncoder {
         algid = oid;
         algParams = algparams;
         constructedFromDer = false;
-        if (algParams != null) {
-            try {
-                encodedParams = algParams.getEncoded();
-            } catch (IOException ioe) {
-                // It should be safe to ignore this.
-                // This exception can occur if AlgorithmParameters was not
-                // initialized (which should not occur), or if it was
-                // initialized with bogus parameters, which should have
-                // been detected when init was called.
-            }
-        }
     }
 
     private AlgorithmId(ObjectIdentifier oid, DerValue params)
@@ -128,15 +116,14 @@ public class AlgorithmId implements Serializable, DerEncoder {
         this.algid = oid;
         this.params = params;
         if (this.params != null) {
-            encodedParams = params.toByteArray();
             decodeParams();
         }
     }
 
     protected void decodeParams() throws IOException {
-        String algidName = getName();
+        String algidString = algid.toString();
         try {
-            algParams = AlgorithmParameters.getInstance(algidName);
+            algParams = AlgorithmParameters.getInstance(algidString);
         } catch (NoSuchAlgorithmException e) {
             /*
              * This algorithm parameter type is not supported, so we cannot
@@ -147,7 +134,7 @@ public class AlgorithmId implements Serializable, DerEncoder {
         }
 
         // Decode (parse) the parameters
-        algParams.init(encodedParams.clone());
+        algParams.init(params.toByteArray());
     }
 
     /**
@@ -166,7 +153,6 @@ public class AlgorithmId implements Serializable, DerEncoder {
      *
      * @exception IOException on encoding error.
      */
-    @Override
     public void derEncode (OutputStream out) throws IOException {
         DerOutputStream bytes = new DerOutputStream();
         DerOutputStream tmp = new DerOutputStream();
@@ -175,12 +161,7 @@ public class AlgorithmId implements Serializable, DerEncoder {
         // Setup params from algParams since no DER encoding is given
         if (constructedFromDer == false) {
             if (algParams != null) {
-                if (encodedParams == null) {
-                    // call getEncoded again in case algParams were initialized
-                    // after being passed in to ctor.
-                    encodedParams = algParams.getEncoded();
-                }
-                params = new DerValue(encodedParams);
+                params = new DerValue(algParams.getEncoded());
             } else {
                 params = null;
             }
@@ -190,15 +171,15 @@ public class AlgorithmId implements Serializable, DerEncoder {
 
             // Several AlgorithmId should omit the whole parameter part when
             // it's NULL. They are ---
-            // rfc3370 2.1: Implementations SHOULD generate SHA-1
+            // RFC 3370 2.1: Implementations SHOULD generate SHA-1
             // AlgorithmIdentifiers with absent parameters.
-            // rfc3447 C1: When id-sha1, id-sha224, id-sha256, id-sha384 and
+            // RFC 3447 C1: When id-sha1, id-sha224, id-sha256, id-sha384 and
             // id-sha512 are used in an AlgorithmIdentifier the parameters
             // (which are optional) SHOULD be omitted.
-            // rfc3279 2.3.2: The id-dsa algorithm syntax includes optional
+            // RFC 3279 2.3.2: The id-dsa algorithm syntax includes optional
             // domain parameters... When omitted, the parameters component
             // MUST be omitted entirely
-            // rfc3370 3.1: When the id-dsa-with-sha1 algorithm identifier
+            // RFC 3370 3.1: When the id-dsa-with-sha1 algorithm identifier
             // is used, the AlgorithmIdentifier parameters field MUST be absent.
             /*if (
                 algid.equals((Object)SHA_oid) ||
@@ -218,7 +199,7 @@ public class AlgorithmId implements Serializable, DerEncoder {
                 // RFC 4055 3.3: when an RSASSA-PSS key does not require
                 // parameter validation, field is absent.
             } else {
-            bytes.putNull();
+                bytes.putNull();
             }
         } else {
             bytes.putDerValue(params);
@@ -264,7 +245,7 @@ public class AlgorithmId implements Serializable, DerEncoder {
         if ((params != null) && algid.equals((Object)specifiedWithECDSA_oid)) {
             try {
                 AlgorithmId paramsId =
-                        AlgorithmId.parse(new DerValue(encodedParams));
+                        AlgorithmId.parse(new DerValue(getEncodedParams()));
                 String paramsName = paramsId.getName();
                 algName = makeSigAlg(paramsName, "EC");
             } catch (IOException e) {
@@ -282,14 +263,10 @@ public class AlgorithmId implements Serializable, DerEncoder {
      * Returns the DER encoded parameter, which can then be
      * used to initialize java.security.AlgorithmParamters.
      *
-     * Note that this* method should always return a new array as it is called
-     * directly by the JDK implementation of X509Certificate.getSigAlgParams()
-     * and X509CRL.getSigAlgParams().
-     *
      * @return DER encoded parameters, or null not present.
      */
     public byte[] getEncodedParams() throws IOException {
-        return (encodedParams == null) ? null : encodedParams.clone();
+        return (params == null) ? null : params.toByteArray();
     }
 
     /**
@@ -297,8 +274,9 @@ public class AlgorithmId implements Serializable, DerEncoder {
      * with the same parameters.
      */
     public boolean equals(AlgorithmId other) {
-        return algid.equals((Object)other.algid) &&
-            Arrays.equals(encodedParams, other.encodedParams);
+        boolean paramsEqual =
+          (params == null ? other.params == null : params.equals(other.params));
+        return (algid.equals((Object)other.algid) && paramsEqual);
     }
 
     /**
@@ -308,7 +286,6 @@ public class AlgorithmId implements Serializable, DerEncoder {
      *
      * @param other preferably an AlgorithmId, else an ObjectIdentifier
      */
-    @Override
     public boolean equals(Object other) {
         if (this == other) {
             return true;
@@ -335,11 +312,11 @@ public class AlgorithmId implements Serializable, DerEncoder {
      *
      * @return a hashcode for this AlgorithmId.
      */
-    @Override
     public int hashCode() {
-        int hashCode = algid.hashCode();
-        hashCode = 31 * hashCode + Arrays.hashCode(encodedParams);
-        return hashCode;
+        StringBuilder sbuf = new StringBuilder();
+        sbuf.append(algid.toString());
+        sbuf.append(paramsToString());
+        return sbuf.toString().hashCode();
     }
 
     /**
@@ -347,10 +324,10 @@ public class AlgorithmId implements Serializable, DerEncoder {
      * This may be redefined by subclasses which parse those parameters.
      */
     protected String paramsToString() {
-        if (encodedParams == null) {
+        if (params == null) {
             return "";
         } else if (algParams != null) {
-            return ", " + algParams.toString();
+            return algParams.toString();
         } else {
             return ", params unparsed";
         }
@@ -359,7 +336,6 @@ public class AlgorithmId implements Serializable, DerEncoder {
     /**
      * Returns a string describing the algorithm and its parameters.
      */
-    @Override
     public String toString() {
         return getName() + paramsToString();
     }
@@ -584,15 +560,6 @@ public class AlgorithmId implements Serializable, DerEncoder {
             || name.equalsIgnoreCase("SHA1/RSA")) {
             return AlgorithmId.sha1WithRSAEncryption_oid;
         }
-        if (name.equalsIgnoreCase("SHA256WithRSA")) {
-            return AlgorithmId.sha256WithRSAEncryption_oid;
-        }
-        if (name.equalsIgnoreCase("SHA384WithRSA")) {
-            return AlgorithmId.sha384WithRSAEncryption_oid;
-        }
-        if (name.equalsIgnoreCase("SHA512WithRSA")) {
-            return AlgorithmId.sha512WithRSAEncryption_oid;
-        }
         if (name.equalsIgnoreCase("SHA1withECDSA")
                 || name.equalsIgnoreCase("ECDSA")) {
             return AlgorithmId.sha1WithECDSA_oid;
@@ -1103,8 +1070,6 @@ public class AlgorithmId implements Serializable, DerEncoder {
             case "RSA":
                 return ifcFfcStrength(KeyUtil.getKeySize(k))
                     + "withRSA";
-            case "RSASSA-PSS":
-                return "RSASSA-PSS";
             default:
                 return null;
         }
