@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
  * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  *
  *
@@ -99,6 +99,7 @@ public class X509CRLImpl extends X509CRL implements DerEncoder {
     private List<X509CRLEntry> revokedList = new LinkedList<>();
     private CRLExtensions    extensions = null;
     private static final boolean isExplicit = true;
+    private static final long YR_2050 = 2524636800000L;
 
     private boolean readOnly = false;
 
@@ -285,13 +286,13 @@ public class X509CRLImpl extends X509CRL implements DerEncoder {
                 throw new CRLException("Null Issuer DN not allowed in v1 CRL");
             issuer.encode(tmp);
 
-            if (thisUpdate.getTime() < CertificateValidity.YR_2050)
+            if (thisUpdate.getTime() < YR_2050)
                 tmp.putUTCTime(thisUpdate);
             else
                 tmp.putGeneralizedTime(thisUpdate);
 
             if (nextUpdate != null) {
-                if (nextUpdate.getTime() < CertificateValidity.YR_2050)
+                if (nextUpdate.getTime() < YR_2050)
                     tmp.putUTCTime(nextUpdate);
                 else
                     tmp.putGeneralizedTime(nextUpdate);
@@ -370,7 +371,7 @@ public class X509CRLImpl extends X509CRL implements DerEncoder {
         }
         Signature   sigVerf = null;
         String sigName = sigAlgId.getName();
-        if (sigProvider.length() == 0) {
+        if (sigProvider.isEmpty()) {
             sigVerf = Signature.getInstance(sigName);
         } else {
             sigVerf = Signature.getInstance(sigName, sigProvider);
@@ -490,7 +491,7 @@ public class X509CRLImpl extends X509CRL implements DerEncoder {
             if (readOnly)
                 throw new CRLException("cannot over-write existing CRL");
             Signature sigEngine = null;
-            if ((provider == null) || (provider.length() == 0))
+            if (provider == null || provider.isEmpty())
                 sigEngine = Signature.getInstance(algorithm);
             else
                 sigEngine = Signature.getInstance(algorithm, provider);
